#include "DapCoin.h"
#include <QDebug>

namespace Dap
{
    Coin::Coin()
    : m_value(uint256_0)
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] Construct value = 0";
#endif
    }

    Coin::Coin(const uint256_t& other)
        : m_value(other)
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] Construct (other uint256_t) value = " << this->toCoinsString();
#endif
    }

    Coin::Coin(const QString& str)
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] (str) value = " << str;
#endif
        if(isValid(str))
        {
            m_value = dap_uint256_scan_decimal(str.toStdString().data());
        }
        else
        {
            qDebug() << "Value is not valid: " << str;
            m_value = uint256_0;
        }
    }

    Coin:: Coin(const Coin& other)
        : m_value(other.m_value)
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] Construct (other uint256_t) value = " << other.toCoinsString();
#endif
    }

    Coin Coin::operator=(const Coin& other)
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] Assignment other coin = " << other.toCoinsString();
#endif
        if (this != &other)
        {
            m_value = other.m_value;
        }
        return *this;
    }

    Coin Coin::operator=(const QString& str)
    {
#ifdef IS_DEBUG_COIN

        qDebug() << "[Coin] Assignment str = " << str;
#endif
        m_value = dap_uint256_scan_decimal(str.toStdString().data());
        return *this;
    }

    Coin Coin::operator=(const double& dec_val)
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] Assignment double = " << dec_val;
#endif
        if(dec_val < 0)
        {
            qDebug() << "Value is not valid: " << dec_val;
            m_value = uint256_0;
        }
        else
        {
            QString tmpVal = QString::number(dec_val);
            m_value = dap_uint256_scan_decimal(tmpVal.toStdString().data());
        }

        return *this;
    }

    Coin Coin::operator=(const int int_val)
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] Assignment int = " << int_val;
#endif
        if(int_val < 0)
        {
            qDebug() << "Value is not valid: " << int_val;
            m_value = uint256_0;
        }
        else
        {
            QString tmpVal = QString::number(int_val);
            tmpVal += ".0";
            m_value = dap_uint256_scan_decimal(tmpVal.toStdString().data());
        }

        return *this;
    }

    Coin Coin::operator+(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] the addition operation: " << this->toCoinsString() << " + " << other.toCoinsString();
#endif
        uint256_t result;
        SUM_256_256(m_value, other.m_value, &result);       
        return std::move(result);
    }

    Coin Coin::operator-(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] the subtraction operation: " << this->toCoinsString() << " - " << other.toCoinsString();
#endif
        uint256_t result;
        if(*this < other)
        {
            qDebug() << "[Coin] There is no support for negative numbers, the result is zero.";
            return uint256_0;
        }
        SUBTRACT_256_256(m_value, other.m_value, &result);
        return std::move(result);
    }

    Coin Coin::operator*(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] the multiplication operation: " << this->toCoinsString() << " * " << other.toCoinsString();
#endif
        uint256_t result;
        MULT_256_COIN(m_value, other.m_value, &result);
        return std::move(result);
    }

    Coin Coin::operator /(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] the division operation: " << this->toCoinsString() << " / " << other.toCoinsString();
#endif
        if(other == uint256_0)
        {
            qDebug() << "[Coin] An attempt to divide by zero.";
            return uint256_0;
        }
        uint256_t result;
        DIV_256_COIN(m_value, other.m_value, &result);
        return std::move(result);
    }

    Coin Coin::operator *(const int int_val) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] the multiplication (int) operation: " << this->toCoinsString() << " * " << int_val;
#endif
        uint256_t result;
        if(int_val < 0)
        {
            qDebug() << "Value is not valid: " << int_val;
            result = uint256_0;
        }
        else
        {
            QString tmpVal = QString::number(int_val);
            tmpVal += ".0";
            uint256_t tmpValUint =  dap_uint256_scan_decimal(tmpVal.toStdString().data());
            MULT_256_COIN(m_value, tmpValUint, &result);
        }
        return std::move(result);
    }

    Coin Coin::operator/(const int int_val) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] the division (int) operation: " << this->toCoinsString() << " / " << int_val;
#endif
        uint256_t result;
        if(int_val <= 0)
        {
            qDebug() << "Value is not valid: " << int_val;
            result = uint256_0;
        }
        else
        {
            QString tmpVal = QString::number(int_val);
            tmpVal += ".0";
            uint256_t tmpValUint =  dap_uint256_scan_decimal(tmpVal.toStdString().data());
            DIV_256_COIN(m_value, tmpValUint, &result);
        }
        return std::move(result);
    }

    Coin Coin::operator+=(const Coin& other)
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] addition and assignment operation: " << this->toCoinsString() << " += " << other.toCoinsString();
#endif
        SUM_256_256(m_value, other.m_value, &m_value);
        return *this;
    }

    bool Coin::operator==(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] : " << this->toCoinsString() << " == " << other.toCoinsString();
#endif
        return compare256(m_value, other.m_value) == 0;
    }

    bool Coin::operator!=(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] : " << this->toCoinsString() << " != " << other.toCoinsString();
#endif
        return compare256(m_value, other.m_value) != 0;
    }

    bool Coin::operator>=(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] : " << this->toCoinsString() << " >= " << other.toCoinsString();
#endif
        return compare256(m_value, other.m_value) >= 0;
    }

    bool Coin::operator<=(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] : " << this->toCoinsString() << " <= " << other.toCoinsString();
#endif
        return compare256(m_value, other.m_value) <= 0;
    }

    bool Coin::operator>(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] : " << this->toCoinsString() << " > " << other.toCoinsString();
#endif
        return compare256(m_value, other.m_value) > 0;
    }

    bool Coin::operator<(const Coin& other) const
    {
#ifdef IS_DEBUG_COIN
        qDebug() << "[Coin] : " << this->toCoinsString() << " < " << other.toCoinsString();
#endif
        return compare256(m_value, other.m_value) < 0;
    }

    QString Coin::toCoinsString() const
    {
        return dap_uint256_decimal_to_char(m_value);
    }

    QString Coin::toDatoshiString() const
    {
        return dap_uint256_uninteger_to_char(m_value);
    }

    bool Coin::isValid(const QString& str) const
    {
        if(str.size() > 72)
        {
            return false;
        }
        return str.contains(REGULAR_VALID_VALUE);
    }

    std::ostream& operator<<(std::ostream& os, const Coin& v)
    {
        os << v.toCoinsString() << " " << v.toDatoshiString();
        return os;
    }

    void Coin::fromDatoshi(const QString& datoshi)
    {
        if(datoshi.isEmpty())
        {
            qWarning() << "[Coin] datoshi is empty";
            return;
        }
        if(!datoshi.contains(REGULAR_VALID_DATOSHI_VALUE))
        {
            qWarning() << "[Coin] Value is not valid. datoshi: " << datoshi;
            return;
        }
        m_value = dap_uint256_scan_uninteger(datoshi.toStdString().data());
    }
}
