#pragma once

#include <dap_chain_common.h>
#include <iostream>
#include <QString>
#include <QRegularExpression>

//#define IS_DEBUG_COIN

namespace Dap {
    class Coin
    {
    public:
        Coin();
        Coin(const uint256_t& other);
        Coin(const QString& str);
        Coin(const Coin& other);

        Coin operator=(const Coin& other);
        Coin operator=(const QString& str);
        Coin operator=(const double& dec_val);
        Coin operator=(const int other);

        Coin operator+(const Coin& other) const;
        Coin operator-(const Coin& other) const;
        Coin operator *(const Coin& other) const;
        Coin operator/(const Coin& other) const;
        Coin operator *(const int int_val) const;
        Coin operator/(const int int_val) const;
        Coin operator+=(const Coin& other);

        bool operator==(const Coin& other) const;
        bool operator!=(const Coin& other) const;
        bool operator>=(const Coin& other) const;
        bool operator<=(const Coin& other) const;
        bool operator>(const Coin& other) const;
        bool operator<(const Coin& other) const;

        void fromDatoshi(const QString& datoshi);

        QString toCoinsString() const;
        QString toDatoshiString() const;

    private:
        bool isValid(const QString& str) const;
    private:
        uint256_t m_value;

        const QRegularExpression REGULAR_VALID_VALUE = QRegularExpression(R"((?=.*[0-9])(?:\d+|\d*\.\d+)$)");
        const QRegularExpression REGULAR_VALID_DATOSHI_VALUE = QRegularExpression(R"(^[1-9][0-9]*$)");
    };

    std::ostream& operator<<(std::ostream& os, const Coin& v);

}
