#ifndef DAPCONFIGTOOLCONTROLLER_H
#define DAPCONFIGTOOLCONTROLLER_H

#include <QObject>
#include <QProcess>
#include <QFileInfo>
#include <QJsonObject>
#include <QDebug>
#include <QThread>
#include <QTimer>
#include <QRegularExpression>

#ifdef WIN32
#include <windows.h>
#endif


class DapConfigToolController : public QObject
{
    Q_OBJECT
    DapConfigToolController(QObject *parent = nullptr);

private:
    QTimer * m_initTimer;

public:
    static DapConfigToolController &getInstance();

    Q_PROPERTY(bool statusProcessNode READ statusProcessNode NOTIFY statusProcessNodeChanged)
    Q_PROPERTY(bool statusServiceNode READ statusServiceNode NOTIFY statusServiceNodeChanged)
    Q_INVOKABLE bool statusServiceNode(){return m_statusServiceNode;}
    Q_INVOKABLE bool statusProcessNode(){return m_statusProcessNode;}

    /// status on|off|all
    Q_INVOKABLE QStringList getConfigNetworkList(const QString& status = "on");

    bool m_statusServiceNode{false};
    bool m_statusProcessNode{false};
    bool m_statusInitConfTool{false};
    QString m_nodeConfToolPath{""};
    bool m_flagUserStopAutostart{false};

    enum TypeServiceCommands{
        Status = 0,
        EnableAutostart = 1,
        DisableAutostart,
        Stop,
        Start,
        Restart
    };


public:
    QJsonObject serviceCommand(TypeServiceCommands type);
    QString sendRequest(QString req);

    bool runNode();

    Q_INVOKABLE void stopNode();
    Q_INVOKABLE void startNode();
    Q_INVOKABLE void getStatusNode();
    Q_INVOKABLE void swithServiceEnabled(bool flag);

    void setConfigParam(const QString& configName, const QString& block, const QString& param, const QString& value);
    QString getConfigParam(const QString& configName, const QString& block, const QString& param);
    QStringList parseArrayConfigParam(QString str);


    bool checkConftool();
    void initPathConftool();

#ifdef WIN32
    LONG GetDWORDRegKey(HKEY hKey, const std::wstring &strValueName, DWORD &nValue, DWORD nDefaultValue);
    LONG GetBoolRegKey(HKEY hKey, const std::wstring &strValueName, bool &bValue, bool bDefaultValue);
    LONG GetStringRegKey(HKEY hKey, const std::wstring &strValueName, std::wstring &strValue, const std::wstring &strDefaultValue);
#endif

private:
    bool initConfTool();
    QString getResult(QString find, QStringList list);


signals:
    //QML
    void statusProcessNodeChanged();
    void statusServiceNodeChanged();

    //C++
    void changeStatusConftoolInit(bool status);
};

#endif // DAPCONFIGTOOLCONTROLLER_H
