#include "DapNodeInstallManager.h"
#include <QPointer>

DapNodeInstallManager::DapNodeInstallManager(bool flag_RK, QObject *parent)
    : QObject(parent)
{
    QString branch = "master";

    QString procArch = QSysInfo::currentCpuArchitecture();
    QString latest = "latest-"+procArch;
    if(procArch == "x86_64")
        latest = "latest-amd64";


#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)

    m_url = QUrl(QString("https://pub.cellframe.net/linux/cellframe-node/%1/%2").arg(branch).arg(latest));
    m_suffix =  QString(".deb");
    m_labelUrlFile = "-amd64";
    m_baseUrl = QString("https://pub.cellframe.net/linux/cellframe-node/%1").arg(branch);

#elif defined (Q_OS_MACOS)

    m_url = QUrl(QString("https://pub.cellframe.net/macos/cellframe-node/%1/%2").arg(branch).arg(latest));//todo: need branch
    m_suffix =  QString(".pkg");
    m_labelUrlFile = "-amd64-signed";
    m_baseUrl = QString("https://pub.cellframe.net/macos/cellframe-node/%1").arg(branch);

#elif defined (Q_OS_WIN)

    m_url = QUrl(QString("https://pub.cellframe.net/windows/cellframe-node/%1/%2").arg(branch).arg(latest)); //todo: need branch
    m_suffix =  QString(".exe");
    m_labelUrlFile = "-installer";
    m_baseUrl = QString("https://pub.cellframe.net/windows/cellframe-node/%1").arg(branch);

#elif defined Q_OS_ANDROID
    m_url = QUrl("");
    m_suffix =  QString(".apk");
    m_labelUrlFile = "";
    m_baseUrl = QString("");
    qDebug()<<"No pack for platform";
#else
    m_url = QUrl("");
    m_baseUrl = QString("");
    m_labelUrlFile = "";
    qDebug()<<"No pack for platform";
#endif
}

DapNodeInstallManager::~DapNodeInstallManager()
{
}

QString DapNodeInstallManager::getUrlForDownload()
{
    return m_url.toString();
}

QString DapNodeInstallManager::getUrl(const QString& ver)
{
    return QString("%1/cellframe-node-%2%3%4").arg(m_baseUrl).arg(ver).arg(m_labelUrlFile).arg(m_suffix);
}

void DapNodeInstallManager::checkUpdateNode(const QString& url)
{
    //todo: this func receive install-pack name and fill m_fileName
    QPointer<QNetworkAccessManager> manager = new QNetworkAccessManager(this);
    QNetworkRequest request((QUrl(url)));

    qDebug() << "[Test url] Try check url. URL: " << url;

    QTimer *timer = new QTimer(this);
    timer->setSingleShot(true);
    connect(timer, &QTimer::timeout, [this, manager]() {
        qDebug() << "[Test url] [Timeout] Request timed out.";
        emit singnalReadyUpdateToNode(false);
        manager->deleteLater();
    });

    timer->start(TIMEOUT_CHECK_URL);

    connect(manager, &QNetworkAccessManager::finished, [this, timer, manager](QNetworkReply *reply) {
        timer->stop();
        qDebug() << "[DapNodeInstallManager] [onGetFileName] ";
        if(!reply)
        {
            emit singnalReadyUpdateToNode(false);
            return;
        }
        auto url = reply->url();
        auto state = reply->error();
        if (state == QNetworkReply::NoError
            || state == QNetworkReply::ContentNotFoundError)
        {
            qDebug() << "[Test url] Check url, no errors. URL: " << url;
            QByteArray content = reply->readAll();
            QTextCodec *codec = QTextCodec::codecForName("utf8");
            QString str = codec->toUnicode(content.data());
            QRegExp rw("[\\w+\\s+-\\.]+" + m_suffix);

            rw.indexIn(str);
            m_fileName = rw.cap(0);

            emit singnalReadyUpdateToNode(true);
        }
        else
        {
            qDebug() << "[Test url] Check url, Have a error. URL: " << url;
            qWarning()<<reply->errorString();
            m_fileName = "";

            emit singnalReadyUpdateToNode(false);
        }
        reply->deleteLater();
        manager->deleteLater();
    });

    manager->get(request);
}
