#ifndef DAPNODEPATHMANAGER_H
#define DAPNODEPATHMANAGER_H

#include <QObject>
#include <QSharedMemory>
#include <QDataStream>
#include <QDebug>
#include <QBuffer>
#include <QFileInfo>
#include <QDir>
#include <QRegularExpression>
#include <QRegularExpressionMatch>

#include "DapNodeInstallManager.h"
#include "DapConfigToolController.h"

#ifdef WIN32
#include <windows.h>
#endif

class DapNodePathManager: public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(DapNodePathManager)

    explicit DapNodePathManager(QObject *parent = nullptr);

public:
    static DapNodePathManager &getInstance();

    Q_PROPERTY(int installNode READ installNode NOTIFY installNodeChanged)
    Q_INVOKABLE int installNode() const {return nodePaths.nodeInstallType;}

    bool getInitFlag(){return m_initMemFlag;}
    QString getTarget(){return m_target;}
    QString getUrlForNodeDownload();
    void checkNeedDownload();

    void init();

    struct NodePaths{
        QString nodeDirPath = "";
        QString nodePath = "";
        QString nodePath_cli = "";
        QString nodePath_tool = "";
        QString nodeConfigPath = "";
        int nodeInstallType = Unknown;

        void reset(int type){
            nodeDirPath = "";
            nodePath = "";
            nodePath_cli = "";
            nodePath_tool = "";
            nodeConfigPath = "";
            nodeInstallType = type;
        }

    }nodePaths;

    QString getNodeUrl(const QString& ver = "");

    void tryCheckUrl(const QString& url);
public slots:
    void slotCheckUpdateNode(QString currentNodeVersion);
    void slotReinit();
    //    void slotBreakInstallNode();
signals:
    Q_INVOKABLE void signalIsNeedInstallNode(bool isNeed, QString url);

    void installNodeChanged();

    void checkedUrlSignal(bool check);
    void signalReinitFinish();
private:

    bool initMem();
    bool readMem();
    bool writeMem();

    void fillNodePath();
private:
    DapNodeInstallManager *m_instMngr;
    DapConfigToolController *m_cfgToolCtrl;
    QString m_keyName = "NodePathInit";
    QString m_target{""};
    QSharedMemory m_sharedMemory;
    bool m_initMemFlag{false};

    enum NodeInstallTypes{
        Unknown = -1,
        NoInstall = 0,
        OldInstall,
        NewInstall
    };



};

#endif // DAPNODEPATHMANAGER_H
