#include "DapNetworkStr.h"
#include <QMap>

const QMap<QString, QString> DapNetworkStr::s_stateStrings = {
    { "NET_STATE_OFFLINE", "OFFLINE"},
    { "NET_STATE_ONLINE", "ONLINE"},
    { "NET_STATE_LINKS_PREPARE", "LINKS PREPARE"},
    { "NET_STATE_LINKS_CONNECTING", "LINKS CONNECTING"},
    { "NET_STATE_SYNC_CHAINS", "SYNC CHAINS"},
    { "NET_STATE_SYNC_GDB", "SYNC GDB"},
    { "UNDEFINED", "ERROR"}
};

DapNetworkStr::DapNetworkStr(const QString& a_name, QObject * a_parent /*= nullptr*/)
    : QObject(a_parent)
    , m_name(a_name)
{
}

DapNetworkStr::DapNetworkStr(const DapNetworkStr &aNetwork):
    QObject (aNetwork.parent()),
    m_name(aNetwork.m_name),
    m_networkState(aNetwork.m_networkState),
    m_targetState(aNetwork.m_targetState),
    m_StateColor(aNetwork.m_StateColor),
    m_linksCount(aNetwork.m_linksCount),
    m_activeLinksCount(aNetwork.m_activeLinksCount),
    m_nodeAddress(aNetwork.m_nodeAddress)

{
}

DapNetworkStr &DapNetworkStr::operator=(const DapNetworkStr &aNetwork)
{
    if (aNetwork.m_nodeAddress != m_nodeAddress) {
        m_name = aNetwork.m_name;
        m_networkState = aNetwork.m_networkState;
        m_targetState = aNetwork.m_targetState;
        m_StateColor = aNetwork.m_StateColor;
        m_linksCount = aNetwork.m_linksCount;
        m_activeLinksCount = aNetwork.m_activeLinksCount;
        m_nodeAddress = aNetwork.m_nodeAddress;
    }
    return (*this);
}

QDataStream& operator << (QDataStream& aOut, const DapNetworkStr& aNetwork)
{
    aOut   << aNetwork.m_name
           << aNetwork.m_linksCount
           << aNetwork.m_activeLinksCount
           << aNetwork.m_networkState
           << aNetwork.m_targetState
           << aNetwork.m_StateColor
           << aNetwork.m_nodeAddress;

    return aOut;
}

QDataStream& operator >> (QDataStream& aIn, DapNetworkStr& aNetwork)
{
    aIn >> aNetwork.m_name
        >> aNetwork.m_linksCount
        >> aNetwork.m_activeLinksCount
        >> aNetwork.m_networkState
        >> aNetwork.m_targetState
        >> aNetwork.m_StateColor
        >> aNetwork.m_nodeAddress;

    return aIn;
}

QString DapNetworkStr::getName() const
{
    return m_name;
}

QString DapNetworkStr::getNetworkState() const
{
    return m_networkState;
}

QString DapNetworkStr::getTargetState() const
{
    return m_targetState;
}

QString DapNetworkStr::getStateColor() const
{
    return m_StateColor;
}

QString DapNetworkStr::getErrorMessage() const
{
    return m_ErrorMessage;
}

QString DapNetworkStr::getActiveLinksCount() const
{
    return m_activeLinksCount;
}

QString DapNetworkStr::getLinksCount() const
{
    return m_linksCount;
}

QString DapNetworkStr::getNodeAddress() const
{
    return m_nodeAddress;
}

QString DapNetworkStr::fromStateStringToShortStateString(const QString &a_state)
{
    return s_stateStrings[a_state];
}

void DapNetworkStr::setName(const QString &a_name)
{
    if (m_name != a_name)
        m_name = a_name;
}

void DapNetworkStr::setNetworkState(const QString &a_networkState)
{
    if (m_networkState != a_networkState) {
        m_networkState = a_networkState;
        setStateColor(a_networkState);
    }
}

void DapNetworkStr::setTargetState(const QString &a_targetState)
{
    if (m_targetState != a_targetState)
        m_targetState = a_targetState;
}

void DapNetworkStr::setStateColor(const QString &a_state)
{
    if (a_state == "ONLINE" || a_state == "ESTABLISHING") {
        m_StateColor = "green";
    } else if (a_state == "OFFLINE") {
        m_StateColor = "orange";
    } else {
        m_StateColor = "red";
    }
}

void DapNetworkStr::setErrorMessage(const QString &a_error)
{
    if (m_ErrorMessage != a_error)
        m_ErrorMessage = a_error;
}

void DapNetworkStr::setActiveLinksCount(const QString &a_count)
{
    if (m_activeLinksCount != a_count)
        m_activeLinksCount = a_count;
}

void DapNetworkStr::setLinksCount(const QString &a_linksCount)
{
    if (m_linksCount != a_linksCount)
        m_linksCount = a_linksCount;
}

void DapNetworkStr::setNodeAddress(const QString &a_nodeAddress)
{
    if (m_nodeAddress != a_nodeAddress)
        m_nodeAddress = a_nodeAddress;
}
