#include "DapWallet.h"

#include <QDebug>

DapWallet::DapWallet(QObject * parent)
    : QObject(parent)
    , m_balanceModel(new DapWalletBalanceModel(this))
{
}

DapWallet::DapWallet(const QString &a_name, QObject *parent)
    : QObject(parent)
    , m_balanceModel(new DapWalletBalanceModel(this))
    , m_sName(a_name)
{
}

DapWallet::DapWallet(const DapWallet &aWallet)
    : m_balanceModel(new DapWalletBalanceModel(this))
    , m_sName(aWallet.m_sName)
    , m_sStatus(aWallet.m_sStatus)
    , m_sIcon(aWallet.m_sIcon)
    , m_sAddress(aWallet.m_sAddress)
    , m_aNetworks(aWallet.m_aNetworks)
    , m_aAddresses(aWallet.m_aAddresses)
    , m_aChains(aWallet.m_aChains)
    , m_aTokens(aWallet.m_aTokens)
{
}

DapWallet &DapWallet::operator=(const DapWallet &aWallet)
{
    m_sName = aWallet.m_sName;
    m_sStatus = aWallet.m_sStatus;
    m_sIcon = aWallet.m_sIcon;
    m_sAddress = aWallet.m_sAddress;
    m_aNetworks = aWallet.m_aNetworks;
    m_aAddresses = aWallet.m_aAddresses;
    m_aChains = aWallet.m_aChains;
    m_aTokens = aWallet.m_aTokens;

    return (*this);
}

QString DapWallet::getName() const
{
    return m_sName;
}

void DapWallet::setName(const QString &asName)
{
    m_sName = asName;

    emit nameChanged(m_sName);
}

QString DapWallet::getStatus() const
{
    return m_sStatus;
}

void DapWallet::setStatus(const QString &asStatus)
{
    m_sStatus = asStatus;

    emit statusChanged(m_sStatus);
}

QString DapWallet::getIcon() const
{
    return m_sIcon;
}

void DapWallet::setIcon(const QString &sIcon)
{
    m_sIcon = sIcon;

    emit iconChanged(m_sIcon);
}

void DapWallet::addNetwork(const QString &asNetwork)
{
    m_aNetworks.append(asNetwork);

    emit networkAdded(asNetwork);
    emit networksChanged(m_aNetworks);
}

QStringList DapWallet::getNetworks() const
{
    return m_aNetworks;
}

void DapWallet::setAddress(const QString &asNetwork)
{
    m_sAddress = asNetwork;

    emit addressChanged(m_sAddress);
}

QString DapWallet::getAddress() const
{
    return m_sAddress;
}

void DapWallet::addAddress(const QString &asNetwork, const QString& aiAddress)
{
    m_aAddresses.insert(asNetwork, aiAddress);
}

void DapWallet::addChains(const QString &asNetwork, const QStringList &aiChains)
{
    m_aChains[asNetwork] = aiChains;
}

QString DapWallet::findAddress(const QString &asNetwork) const
{
    if (m_aAddresses.isEmpty() || asNetwork.isEmpty())
        return QString();

    auto findIter = m_aAddresses.find(asNetwork);

    return findIter != m_aAddresses.end() ? findIter.value() : QString();
}

QStringList DapWallet::getChains(const QString &asNetwork) const
{
    if (m_aChains.contains(asNetwork))
        return m_aChains.value(asNetwork);
    else
        return QStringList();
}

QMap<QString, QString> DapWallet::getAddresses() const
{
    return m_aAddresses;
}

void DapWallet::addToken(DapWalletToken *asToken)
{
    m_aTokens.append(asToken);

    emit tokenAdded(*asToken);

    QList<QObject*> tokens;
    auto begin = m_aTokens.begin();
    auto end = m_aTokens.end();
    for(;begin != end; ++begin)
    {
        tokens.append(*begin);
    }
    emit tokensChanged(tokens);
}

QList<DapWalletToken*> DapWallet::findTokens(const QString &asNetwork)
{
    QList<DapWalletToken*> tokens;
    auto begin = m_aTokens.begin();
    auto end = m_aTokens.end();
    for(;begin != end; ++begin)
    {
        if((*begin)->getNetwork() == asNetwork)
        {
            tokens.append(*begin);
        }
    }
    return tokens;
}

QList<QObject *> DapWallet::getTokens() const
{
    QList<QObject*> tokens;
    auto begin = m_aTokens.begin();
    auto end = m_aTokens.end();
    for(;begin != end; ++begin)
    {
        tokens.append(*begin);
    }
    return tokens;
}

DapWalletBalanceModel *DapWallet::getBalanceModel() const
{
    return m_balanceModel;
}

DapWallet DapWallet::fromVariant(const QVariant &aWallet)
{
    DapWallet wallet;
    QByteArray data = QByteArray::fromStdString(aWallet.toString().toStdString());
    QDataStream in(&data, QIODevice::ReadOnly);
    in >> wallet;
    return wallet;
}

void DapWallet::setBalance(QMap<const DapNetwork*, QMap<const DapToken*, balance_t>>  a_balanceMap)
{

    //Remove empty amounts:
    QMutableMapIterator<decltype (a_balanceMap)::key_type, decltype (a_balanceMap)::mapped_type> networkIt(a_balanceMap);
    while (networkIt.hasNext()) {
        networkIt.next();
        QMutableMapIterator<const DapToken *, balance_t> tokenIt(networkIt.value());
        while (tokenIt.hasNext()) {
            tokenIt.next();
            if (tokenIt.value() == 0)
                tokenIt.remove();
        }
        if (networkIt.value().isEmpty())
            networkIt.remove();
    }


    m_balanceModel->setBalance(a_balanceMap);
}

QDataStream& operator << (QDataStream& aOut, const DapWallet& aWallet)
{
    QList<DapWalletToken> tokens;
    for(int x{0}; x < aWallet.m_aTokens.size(); ++x)
    {
        tokens.append(*aWallet.m_aTokens.at(x));
    }

     aOut   << aWallet.m_sName
            << aWallet.m_sStatus
            << aWallet.m_sIcon
            << aWallet.m_sAddress
            << aWallet.m_aNetworks
            << aWallet.m_aAddresses
            << aWallet.m_aChains
            << tokens;

    return aOut;
}

QDataStream& operator >> (QDataStream& aIn, DapWallet& aWallet)
{
    QList<DapWalletToken> tokens;

        aIn >> aWallet.m_sName
            >> aWallet.m_sStatus
            >> aWallet.m_sIcon
            >> aWallet.m_sAddress
            >> aWallet.m_aNetworks
            >> aWallet.m_aAddresses
            >> aWallet.m_aChains
            >> tokens;


    auto begin = tokens.begin();
    auto end = tokens.end();
    for(;begin != end; ++begin)
        aWallet.addToken(new DapWalletToken(*begin));
    return aIn;
}

bool operator ==(const DapWallet &aWalletFirst, const DapWallet &aWalletSecond)
{
    return aWalletFirst.m_sName == aWalletSecond.m_sName;
}
