#pragma once

#include <QObject>
#include <QString>
#include <QStringList>
#include <QVector>
#include <QMap>
#include <QTimer>
#include <QMutex>
#include <QJsonArray>
#include <QDataStream>
#include <QJsonDocument>
#include "../handlers/CommandType.h"
#include "../handlers/DapCommandList.h"

class DapTransactionQueueController: public QObject
{
    Q_OBJECT



    struct DataTransaction
    {
        QString nameCommand = "";
        QString hash = "";
        QString queueHash = "";
        qint64 time = 0;
        bool isRequest = false;
        QStringList paramCommand = {};
    };



    DapTransactionQueueController(QObject *parent = nullptr);
    ~DapTransactionQueueController();
    static DapTransactionQueueController* s_instance;
public:
    using InfoCallback = std::function<QString(const QStringList&)>;
    using TestExpensesCallback = std::function<bool()>;
    using FeeInfoCallback = std::function<const QMap<QString, NetworkSpace::FeeInfoStruct>()>;

                        //   network       walletName     command and etc.
    using QueueStack = QMap<QString, QMap<QString, QVector<DataTransaction>>>;

    struct ParamsFromCommand
    {
        InfoCallback walletNameCallback = nullptr;
        InfoCallback addressCallback = nullptr;
        InfoCallback tokenCallback = nullptr;
        InfoCallback valueCallback = nullptr;
        InfoCallback feeCallback = nullptr;
        InfoCallback directionCallback = nullptr;
        TestExpensesCallback testExpenses = nullptr;
        FeeInfoCallback feeInfo = nullptr;
    };
    static DapTransactionQueueController* getTransactionController();
    static void deleteTransactionController();

    bool isReady(const QString& networkName, const QString &walletName);
    void addTransactionHash(const QString& networkName, const QString &walletName, const QString& hash);
    void addNewTransaction(const QString& nameCommand, const QStringList& paramCommand, const QString& networkName, const QString& walletName
                                    , const QString &hash ="", const QString &queueHash = "");

    void clearCurrentTransaction(const QString& networkName, const QString &walletName);

    void setParamsCommandCallback(const QString& commandName, ParamsFromCommand& callbacks);

    void addCommandWithoutInfoWallet(const QString& commandName);

    QVariant tryRemoveTransactions(const QVariant &transactions);
    QJsonObject tryRemoveTransactionsWithParams(const QVariant &args);

    QVariant isTtransactioninQueued(const QVariant& args);

    const QueueStack& getQueue() const {return m_queue;}
signals:
    void toCreateTransaction(const QString& nameCommand, const QStringList& paramCommand);
    ///                         networkName, hash
    void toGetTransactionData(const QVariant& args);
    ///                     walletName, networkName
    void toGetWalletInfo(const QVariant& args);
    ///
    void updateInfoForWallets(const QVariant& data);

    void updateHistoryForWallet(const QVariant& data);
    void newTransactionAdded(const QVariant& data);
public slots:
    void transactionDataReceived(const QVariant& requestData);
    void walletInfoReceived(const QVariant& requestData);

    void updateInfoTransaction(const QVariant& requestData);
    void onInit();
private slots:
    void updateInfoHash();
    void createTransaction(const QString &networkName, const QString &walletName);
private:
    void tryStartTransactions();
    void saveToFile();
    void loadFromFile();

    QJsonDocument getJsonQueue();
    void queueFromJson(const QJsonDocument& document);
    void tryUpdateInfo();
    void updateWalletsInfo();
    void updateHistoryInfo();

    bool isCurrentTransaction(const QString& networkName, const QString &walletName, const QString &hash);
private:
    QTimer *m_requestTimer;
    QMutex m_mutex;

    QStringList m_notChangeWalletInfoList;

    QueueStack m_queue;
    QMap<QString, ParamsFromCommand> m_dataParamsPool;

    const QRegularExpression REGULAR_HASH = QRegularExpression(R"(0x[a-fA-F0-9])");

    const QString FILE_NAME = "DapTrQueue.bin";
    const QString MEMPOOL_CHECK_COMMAND = "MempoolCheckCommand";

    const int TIMEOUT_TRANSACTION = 3600000;
    const int TIMEOUT_REQUEST = 5000;
};


