#include "HelpDictionaryController.h"
#include <QDebug>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QSet>

HelpDictionaryController::HelpDictionaryController()
    :m_treeDictionary(new TreeDictionaryBase)
{}

HelpDictionaryController::~HelpDictionaryController()
{
    delete m_treeDictionary;
}

void HelpDictionaryController::setDictionary(const QJsonObject &tree)
{
    m_treeDictionary->dictionaryFromJSON(tree);
}

bool HelpDictionaryController::isDictionary()
{
    return m_treeDictionary->isTree();
}

QStringList HelpDictionaryController::getHelpList(const QString& text, int cursorPosition)
{
    return getListVariants(text);
}

QStringList HelpDictionaryController::getListVariants(const QString& text)
{
    if(text.isEmpty())
    {
        return {};
    }

    QStringList resultList;
    QStringList list = text.split(' ');
    if(!m_treeDictionary->isTree())
    {
        return {};
    }
    TreeNode* resultNode = m_treeDictionary->getTree();

    int size = list.size();
    int maxIndex = size - 1;
    QVector<const QStringList *> variants;
    QMap<QString, int> *mainOrList = nullptr;
    QStringList currentList;

    QString wallet;
    QString network;

    auto addToCurrListAndFilterVariants = [&currentList, &variants](const QString& name)
    {
        currentList.append(name);
        int size = variants.size();
        for(int index = size - 1; index >= 0; --index)
        {
            auto& var = variants[index];
            if(!var->contains(name))
            {
                variants.remove(index);
            }
        }
    };

    auto updateResulNode = [&currentList, &resultNode, this]()
    {
        QString mainName;
        for(const QString& name: currentList)
        {
            if(!name.contains('-'))
            {
                mainName = name;
            }
            else
            {
                break;
            }
        }

        if(!m_treeDictionary)
        {
            return;
        }

        TreeNode* minNode = m_treeDictionary->getTree();
        while(resultNode->previousNode != minNode)
        {
            resultNode = resultNode->previousNode;

            if((!resultNode->variantList.isEmpty() && resultNode->name == mainName))
            {
                return;
            }
        }
    };

    bool isParametrLast = false;

    for(int i = 0; i < size; i++)
    {
        if(!resultNode) continue;

        if(variants.isEmpty() && !resultNode->variantList.isEmpty())
        {
            for(const auto& item: qAsConst(resultNode->variantList))
            {
                variants.append(&item);
            }
        }

        if(!resultNode->orList.isEmpty() && !mainOrList)
        {
            mainOrList = &resultNode->orList;
        }

        if(i < maxIndex)
        {
            if(i > 1)
            {
                if(list[i - 1] == "-w")
                {
                    wallet = list[i];
                }
                else if(list[i - 1] == "-net")
                {
                    network = list[i];
                }
            }

            if(resultNode->children.contains(list[i]))
            {
                resultNode = resultNode->children[list[i]];
                addToCurrListAndFilterVariants(resultNode->name);
                continue;
            }
            else if(!resultNode->data.isEmpty() && resultNode->children.isEmpty())
            {
                isParametrLast = false;
                updateResulNode();
                continue;
            }
            else if(!resultNode->data.isEmpty() && !resultNode->children.isEmpty())
            {
                if(!list[i].isEmpty())
                {
                    if(resultNode->children.contains(list[i]))
                    {
                        resultNode = resultNode->children[list[i]];
                        addToCurrListAndFilterVariants(resultNode->name);
                    }
                    else
                    {
                        isParametrLast = true;
                    }
                }
                continue;
            }
            else if(resultNode && resultNode->data.isEmpty() && resultNode->children.isEmpty()
                       && resultNode->previousNode != m_treeDictionary->getTree())
            {
                updateResulNode();
                resultNode = resultNode->children[list[i]];
                if(resultNode)
                {
                    addToCurrListAndFilterVariants(resultNode->name);
                }
                continue;
            }
            else if(resultNode && resultNode->data.isEmpty() && resultNode->children.isEmpty()
                       && resultNode->previousNode == m_treeDictionary->getTree())
            {
                continue;
            }
        }
        else if(i == maxIndex)
        {
            bool isFilter = true;
            if (list[i].isEmpty())
            {
                int testIndex = i - 1;
                if(testIndex >= 0 && list[testIndex] == resultNode->name
                    && resultNode->data.isEmpty() && resultNode->children.isEmpty())
                {
                    updateResulNode();
                }
                else if(testIndex >= 0 && !list[testIndex].isEmpty() &&
                           !resultNode->data.isEmpty() && !resultNode->children.isEmpty())
                {
                    //You don't have to do anything
                }
                else if(--testIndex >= 0 && list[testIndex] == resultNode->name && !resultNode->data.isEmpty())
                {
                    updateResulNode();
                }
            }
            if(resultNode->data.isEmpty() && !resultNode->children.isEmpty())
            {
                QList<QString> tmpListKeys = resultNode->children.keys();
                for(const QString& key: qAsConst(tmpListKeys))
                {
                    if(key.contains(list[i]))
                    {
                        resultList.append(key);
                    }
                }
            }
            else if (isParametrLast && !resultNode->data.isEmpty() && !resultNode->children.isEmpty())
            {
                QList<QString> tmpListKeys = resultNode->children.keys();
                for(const QString& key: qAsConst(tmpListKeys))
                {
                    if(key.contains(list[i]))
                    {
                        resultList.append(key);
                    }
                }
            }
            else if(!resultNode->data.isEmpty())
            {
                isFilter = false;
                QStringList tmpList;
                tmpList.append(getListParamrtrs(wallet, network, resultNode->data));
                if(tmpList.isEmpty())
                {
                    qDebug()<< "problems with parameter data";
                    return {};
                }
                for(const QString& item: tmpList)
                {
                    if(item.contains(list[i]))
                    {
                        resultList.append(item);
                    }
                }
            }
            else if(resultNode->name == "help" && list.size() == 2)
            {
                isFilter = false;
                resultList.append(resultNode->previousNode->children.keys());
            }

            if(!variants.isEmpty() && isFilter)
            {
                for(int index = resultList.size() - 1 ; index >= 0; index--)
                {
                    bool isContain = false;
                    if(resultList[index].contains('<'))
                    {
                        continue;
                    }

                    for(int var = 0; var < variants.size(); var++)
                    {
                        if(variants[var] && variants[var]->contains(resultList[index]))
                        {
                            isContain = true;
                        }
                    }
                    if(!isContain)
                    {
                        resultList.removeAt(index);
                    }
                }
            }

            for(const auto& item: qAsConst(list))
            {
                if(resultList.contains(item))
                {
                    resultList.removeOne(item);
                }

                if(mainOrList && !mainOrList->isEmpty() && mainOrList->contains(item))
                {
                    int number = mainOrList->value(item);
                    for(const QString& key: mainOrList->keys())
                    {
                        if(key == item || key.isEmpty())
                        {
                            continue;
                        }

                        if(mainOrList->value(key) == number)
                        {
                            resultList.removeOne(key);
                        }
                    }
                }
            }
        }
    }
    return resultList;
}

void HelpDictionaryController::variantList(const QStringList& currentList, QVector<const QStringList *>& currentVariants)
{
    int size = currentVariants.size();
    for(int index = size - 1; index >= 0; --index)
    {
        if(!isHasCorrList(currentList, currentVariants[index]))
        {
            currentVariants.remove(index);
        }
    }
}

bool HelpDictionaryController::isHasCorrList(const QStringList& currentList, const QStringList *testList) const
{
    for(const QString& item: currentList)
    {
        if(!testList->contains(item))
        {
            return false;
        }
    }
    return true;
}

void HelpDictionaryController::setData(const QJsonObject &jData)
{
    if(jData.isEmpty())
    {
        return;
    }

    auto arrayToStringList = [](const QJsonArray& list) -> QStringList
    {
        QStringList result;
        for(const auto& str: list)
        {
            result.append(str.toString());
        }
        return result;
    };

    if(jData.contains("wallets"))
    {
        m_wallets = arrayToStringList(jData["wallets"].toArray());
    }
    if(jData.contains("networks"))
    {
        m_networks = arrayToStringList(jData["networks"].toArray());
    }
    if(jData.contains("pubCertName"))
    {
        m_pubCertName = arrayToStringList(jData["pubCertName"].toArray());
    }
    if(jData.contains("privCertName"))
    {
        m_privCertName = arrayToStringList(jData["privCertName"].toArray());
    }

    auto jsonToMapStrL = [&arrayToStringList](const QJsonArray& list) -> QMap<QString, QStringList>
    {
        QMap<QString, QStringList> result;
        for(const QJsonValue& item: list)
        {
            QJsonObject tmpItem = item.toObject();
            QString key = tmpItem["key"].toString();
            QStringList data = arrayToStringList(tmpItem["data"].toArray());
            result.insert(key, data);
        }
        return result;
    };

    if(jData.contains("chains"))
    {
        m_chains = jsonToMapStrL(jData["chains"].toArray());
    }

    if(jData.contains("mempoolTokensTicker"))
    {
        m_mempoolTokensTicker = jsonToMapStrL(jData["mempoolTokensTicker"].toArray());
    }

    auto jsonToTokenStruct = [&arrayToStringList](const QJsonArray& list) -> QMap<QPair<QString, QString>, QStringList>
    {
        QMap<QPair<QString, QString>, QStringList> result;
        for(const auto& item: list)
        {
            QJsonObject tmpItem = item.toObject();
            QPair<QString, QString> key(tmpItem["wallet"].toString(), tmpItem["network"].toString());
            QStringList data = arrayToStringList(tmpItem["data"].toArray());
            result.insert(key, data);
        }
        return result;
    };

    if(jData.contains("tokensTicker"))
    {
        m_tokensTicker = jsonToTokenStruct(jData["tokensTicker"].toArray());
    }
}

QStringList HelpDictionaryController::getListParamrtrs(const QString& wallet, const QString& net, const QStringList& curData)
{
    if(curData.size() == 1)
    {
        QSet<QString> result;
        auto addToResult = [&result](const QStringList& list)
        {
            for(const QString& item: list)
            {
                result.insert(item);
            }
        };
        const QString& data = curData[0];
        if(data == "<priv_cert_name>")
        {
            return m_privCertName;
        }
        else if(data == "<pub_cert_name>")
        {
            return m_pubCertName;
        }
        else if(data == "<net_name>" || data == "<chain_net_name>")
        {
            return m_networks;
        }
        else if(data == "<wallet_name>")
        {
            return m_wallets;
        }
        else if(data == "<token_ticker>")
        {
            if(wallet.isEmpty() && net.isEmpty())
            {
                for(const QStringList& tickers: qAsConst(m_tokensTicker))
                {
                    addToResult(tickers);
                }
            }
            else if(!wallet.isEmpty() && net.isEmpty())
            {
                for(const QString& item: qAsConst(m_networks))
                {
                    QPair<QString,QString> key({wallet, item});
                    if(m_tokensTicker.contains(key))
                    {
                        addToResult(m_tokensTicker[key]);
                    }
                }
            }
            else if(wallet.isEmpty() && !net.isEmpty())
            {
                for(const QString& item: qAsConst(m_wallets))
                {
                    QPair<QString,QString> key({item, net});
                    if(m_tokensTicker.contains(key))
                    {
                        addToResult(m_tokensTicker[key]);
                    }
                }
            }
            else
            {
                QPair<QString,QString> key({wallet, net});
                if(m_tokensTicker.contains(key))
                {
                    addToResult(m_tokensTicker[key]);
                }
            }
            return result.values();
        }
        else if(data == "<mempool_token_ticker>")
        {
            if(net.isEmpty())
            {
                for(const QStringList& item: qAsConst(m_mempoolTokensTicker))
                {
                    if(!item.isEmpty())
                    {
                        addToResult(item);
                    }
                }
            }
            else
            {
                if(m_mempoolTokensTicker.contains(net))
                {
                    if(!m_mempoolTokensTicker[net].isEmpty())
                    {
                        addToResult(m_mempoolTokensTicker[net]);
                    }
                }
            }
            return result.values();
        }
        else if(data == "<chain_name>")
        {
            if(net.isEmpty())
            {
                for(const QStringList& item: qAsConst(m_chains))
                {
                    if(!item.isEmpty())
                    {
                        addToResult(item);
                    }
                }
            }
            else
            {
                if(m_chains.contains(net))
                {
                    if(!m_chains[net].isEmpty())
                    {
                        addToResult(m_chains[net]);
                    }
                }
            }
            return result.values();
        }
        else
        {
            return QStringList(curData);
        }
    }
    else
    {
        return curData;
    }
}
