#include "HelpDictionaryServiceController.h"
#include <QJsonDocument>
#include <QJsonObject>
#include <QStack>
#include <QHash>
#include <QJsonArray>
#include "../handlers/DapCommandList.h"

HelpDictionaryServiceController::HelpDictionaryServiceController(QObject *parent)
    : QObject{parent}
    , m_treeDictionary(new TreeDictionaryBase())
{
    if(DapConfigToolController::getInstance().m_statusInitConfTool)
    {
        m_sCliPath = &DapNodePathManager::getInstance().nodePaths.nodePath_cli;
        onInit();
    }
    else
    {
        connect(&DapConfigToolController::getInstance(), &DapConfigToolController::changeStatusConftoolInit, [this] (bool isReady)
        {
            if(isReady)
            {
                onInit();
                m_sCliPath = &DapNodePathManager::getInstance().nodePaths.nodePath_cli;

            }
        });
    }
}

HelpDictionaryServiceController::~HelpDictionaryServiceController()
{
    delete m_treeDictionary;
}

void HelpDictionaryServiceController::setNodeVersion(const QString& version, bool isUpdate)
{
    m_nodeVersion = version;
    if(isUpdate)
    {
        tryTestVersions();
    }
}

void HelpDictionaryServiceController::tryTestVersions()
{
    if(!m_nodeVersion.isEmpty() && m_nodeVersion != m_nodeVersionFromFile)
    {
        newDictionary();
    }
}

QJsonDocument HelpDictionaryServiceController::getJsonDictionary()
{
    if(m_treeDictionary->isTree())
    {
        return QJsonDocument(m_treeDictionary->getJSONTree());
    }

    if(loadDictionary())
    {
        return QJsonDocument(m_treeDictionary->getJSONTree());
    }
    else
    {
        return getNewJsonDictionary();
    }
    return {};
}

bool HelpDictionaryServiceController::newDictionary()
{
    if(m_nodeVersion.isEmpty())
    {
        return false;
    }

    if(auto* node = creatDictionary())
    {
        m_treeDictionary->setTree(node);
        if(!saveDictionary())
        {
            qWarning() << "Dictionory is not saved";
        }
        m_nodeVersionFromFile = m_nodeVersion;
        return true;
    }
    else
    {
        qWarning() << "couldn't create a dictionary";
    }
    return false;
}

QJsonDocument HelpDictionaryServiceController::getNewJsonDictionary()
{
    return newDictionary() ? QJsonDocument(m_treeDictionary->getJSONTree()) : QJsonDocument();
}

void HelpDictionaryServiceController::onInit()
{
    // TODO: For tests, if you need to forcibly get a new tree
    //newDictionary();
    if(!m_treeDictionary->isTree())
    {
        if(!loadDictionary())
        {
            newDictionary();
        }
        else
        {
            m_nodeVersion = m_nodeVersionFromFile;
        }
    }
}

bool HelpDictionaryServiceController::saveDictionary()
{
    QJsonObject resultObject;
    QJsonObject treeJson = m_treeDictionary->getJSONTree();
    resultObject["data"] = std::move(treeJson);
    resultObject["version"] = m_nodeVersion;
    QJsonDocument jsonDocument(resultObject);
    QFile file(FILE_NAME);
    if (!file.open(QIODevice::WriteOnly))
    {
        qDebug() << "File is not created";
        return false;
    }
    qint64 result = file.write(jsonDocument.toBinaryData());
    file.close();
    return result > -1;
}

bool HelpDictionaryServiceController::loadDictionary()
{
    QFile file(FILE_NAME);
    if (!file.open(QIODevice::ReadOnly))
    {
        qDebug() << "File not opened";
        return false;
    }    
    QByteArray jsonData = file.readAll();
    file.close();
    QJsonDocument jsonDocument = QJsonDocument::fromBinaryData(jsonData);
    if (jsonDocument.isNull())
    {
        qDebug() << "Error when reading a JSON document from a file";
        return false;
    }
    else if (!jsonDocument.isObject())
    {
        qDebug() << "A JSON document is not an object";
        return false;
    }
    QJsonObject resultObject = jsonDocument.object();
    if(!resultObject.contains("data") || !resultObject.contains("version"))
    {
        qWarning() << "Data is broken";
    }
    m_nodeVersionFromFile = resultObject["version"].toString();
    auto dataObject = resultObject["data"].toObject();
    if(dataObject.isEmpty())
    {
        m_nodeVersion = m_nodeVersionFromFile;
        return false;
    }
    m_treeDictionary->dictionaryFromJSON(dataObject);
    return true;
}

QVector<QPair<QString, int>> HelpDictionaryServiceController::findSubstrings(QString &input, const char openCurl, const char closeCurl)
{
    QStack<int> stack;
    QVector<QPair<QString, int>> substrings;
    QStack<QPair<int,int>> stackForDelete;
    QString substring;
    int open = input.count(openCurl);
    int close = input.count(closeCurl);
    if(open != close)
    {
        qWarning() << "Incorrect number of brackets, input data error. String: " << input;
    }

    for (int i = 0; i < input.length(); i++)
    {
        QChar currentChar = input[i];

        if (currentChar == openCurl)
        {
            stack.push(i);
        }

        else if (currentChar == closeCurl && !stack.isEmpty())
        {
            int startIndex = stack.pop();
            int stopIndex = i - startIndex;
            int stackSize = stack.size();
            substring = input.mid(startIndex + 1, stopIndex - 1);
            if(stackSize == 0)
            {
                stackForDelete.push_back({startIndex, stopIndex + 1});
            }

            // If there are problems with brackets,
            // then we output an empty container,
            // but we delete everything in brackets anyway.
            if(open == close)
            {
                substrings.append({substring, stackSize});
            }
        }
    }

    // Delete the extra lines
    while(!stackForDelete.isEmpty())
    {
        QPair<int, int> pair = stackForDelete.pop();
        input.remove(pair.first, pair.second);
    }

    return substrings;
}

TreeNode *HelpDictionaryServiceController::creatDictionary()
{
    QMap<QString, QString> rawData = getRawData();

    int counterIsLists = 0;

    TreeNode* resultNode = new TreeNode();

    for(const QString &key: rawData.keys())
    {
        TreeNode commandNode;
        //TODO: if you need to check the output of a specific command
        //key = "global_db";

        QStringList list = rawData[key].split(QString("\n"));

        for(int i = 1; i < list.size(); i++)
        {
            QString& str = list[i];
            if(!str.startsWith(key))
            {
                continue;
            }
            TreeNode* tmpLastNode = resultNode;

            QStringList mainLineList;
            QVector<TmpDataStruct> mainMondatoryParam;
            mainParamParce(str, mainLineList, mainMondatoryParam);

            QVector<QPair<QString, int>> unnecessExpress = findSubstrings(str);
            QVector<QPair<QString, int>> necessExpress = findSubstrings(str, '{', '}');

            int currLayerNode = 0;
            int zeroLayerNode = 0;

            auto addToNode = [&tmpLastNode, &currLayerNode]
                (QPair<QString, TreeNode*> itemNode, int layer, bool toOptional = false, int curVariantCount = 0)
            {
                itemNode.second->name = itemNode.first.trimmed();
                TreeNode *parentNode = nullptr;
                if(currLayerNode == layer)
                {
                    itemNode.second->previousNode = tmpLastNode->previousNode;
                    tmpLastNode->previousNode->children.insert(itemNode.first.trimmed(), itemNode.second);
                    parentNode = tmpLastNode->previousNode;
                    tmpLastNode = itemNode.second;
                }
                else if(currLayerNode < layer)
                {
                    currLayerNode = layer;
                    itemNode.second->previousNode = tmpLastNode;
                    tmpLastNode->children.insert(itemNode.first.trimmed(), itemNode.second);
                    parentNode = tmpLastNode;
                    tmpLastNode = itemNode.second;
                }
                else if(currLayerNode > layer)
                {
                    int dif = currLayerNode - layer;
                    currLayerNode = layer;
                    while(dif)
                    {
                        if(tmpLastNode->previousNode){
                            tmpLastNode = tmpLastNode->previousNode;
                            --dif;
                        }
                        else
                        {
                            qWarning()<< "Probably an attempt to access a non-existent value";
                            break;
                        }
                    }
                    itemNode.second->previousNode = tmpLastNode->previousNode;
                    tmpLastNode->previousNode->children.insert(itemNode.first.trimmed(), itemNode.second);
                    parentNode = tmpLastNode->previousNode;
                    tmpLastNode = itemNode.second;
                }

                if(parentNode)
                {
                    if(toOptional)
                    {
                        parentNode->optionalList.append(itemNode.first);
                    }
                    if(curVariantCount != 0)
                    {
                        parentNode->orList.insert(itemNode.first, curVariantCount);
                    }
                }
            };

            for(auto& item: mainLineList)
            {
                if(tmpLastNode->children.contains(item))
                {
                    tmpLastNode = tmpLastNode->children[item];
                    continue;
                }
                TreeNode* nodeNew = new TreeNode();
                QPair<QString, TreeNode *> tmpData = {item, nodeNew};
                addToNode(tmpData, ++zeroLayerNode);
            }
            TreeNode* lastMainNode = tmpLastNode;
            QStringList listParam;

            auto appendToMainLineList = [&mainLineList](const QString& str)
            {
                QString tmpStr = str.trimmed();
                if(!tmpStr.isEmpty())
                {
                    mainLineList.append(tmpStr);
                }
            };

            zeroLayerNode++;
            for(auto& item: mainMondatoryParam)
            {
                addToNode({item.paramName, item.node}, zeroLayerNode);
                appendToMainLineList(item.paramName);
            }

            for(int u = unnecessExpress.size() - 1; u >= 0 ; --u)
            {
                QVector<TmpDataStruct> unmondatoryParam;
                mainParamParce(unnecessExpress[u].first, mainLineList, unmondatoryParam, false);
                for(auto& item: unmondatoryParam)
                {
                    int layer = zeroLayerNode + unnecessExpress[u].second;
                    addToNode({item.paramName,item.node}, layer, true);
                    if(layer == zeroLayerNode)
                    {
                        appendToMainLineList(item.paramName);
                    }
                }
            }

            for(int n = necessExpress.size() - 1; n >= 0 ; --n)
            {
                QVector<TmpDataStruct> mondatoryParam;

                if(necessExpress[n].first.contains('{'))
                {

                    int nextCount = n - 1;
                    if(nextCount >= 0 && necessExpress[n].first.contains(necessExpress[nextCount].first))
                    {
                        int index = necessExpress[n].first.indexOf(necessExpress[nextCount].first);
                        necessExpress[n].first.remove(index, necessExpress[nextCount].first.size());
                    }
                    else
                    {
                        qWarning() << "Has a probem in help for " << key;
                    }
                }

                QStringList orParam = necessExpress[n].first.split('|');
                int localCounterIsLists = ++counterIsLists;
                for(int orCount = 0; orCount < orParam.size(); orCount++)
                {
                    if(orParam[orCount].contains('{'))
                    {
                        mainParamParce(orParam[orCount], mainLineList, mondatoryParam, false, 0, localCounterIsLists);

                        if(n - 1 >= 0)
                        {
                            n--;
                            if(necessExpress[n].first.contains('|'))
                            {
                                QStringList orParamChild = necessExpress[n].first.split('|');
                                ++counterIsLists;
                                for(auto& item: orParamChild)
                                {
                                    mainParamParce(item, mainLineList, mondatoryParam, false, 1, counterIsLists);
                                }
                            }
                            else
                            {
                                mainParamParce(necessExpress[n].first, mainLineList, mondatoryParam, false, 1, localCounterIsLists);
                            }
                        }
                    }
                    else
                    {
                        mainParamParce(orParam[orCount], mainLineList, mondatoryParam, false, 0, localCounterIsLists);
                    }
                }

                for(auto& item: mondatoryParam)
                {
                    int layer = zeroLayerNode + item.layer;
                    addToNode({item.paramName, item.node}, layer, false, item.countOrList);
                    mainLineList.append(item.paramName);
                }
            }
            TreeNode* tmpMain = lastMainNode;
            while(tmpMain->name != key)
            {
                tmpMain = tmpMain->previousNode;
            }
            if(tmpMain != lastMainNode)
            {
                tmpMain->variantList.append(mainLineList);
            }
            lastMainNode->variantList.append(std::move(mainLineList));
        }
    }
    return std::move(resultNode);
}

QMap<QString, QString> HelpDictionaryServiceController::getRawData()
{
    QMap<QString, QString> result;
    QString helpCmd = QString(CMD_HELP);
    QString dataList = requestToNode(helpCmd);
    QStringList list = dataList.split('\n');
    QRegularExpression regularHelp(R"(^(\w+):)");
    for(const auto &str: list)
    {
        QRegularExpressionMatch command = regularHelp.match(str);
        auto lst = command.capturedTexts();
        if(command.hasMatch() && !command.captured(1).isEmpty())
        {
            QString tmpResult = requestToNode(CMD_HELP + " " + command.captured(1));
            result.insert(command.captured(1), std::move(tmpResult));
        }
    }
    return result;
}

QString HelpDictionaryServiceController::requestToNode(QString cmd)
{
    QProcess process;
    process.setProgram(*m_sCliPath);
    process.setArguments(cmd.split(" "));

    process.start();

    if(process.waitForFinished(10000))
        return QString::fromLatin1(process.readAllStandardOutput());
    else{
        qWarning()<<QString("Timeout request: %1").arg(cmd);
        return QString();
    }
}

HelpDictionaryServiceController::TmpDataStruct::TmpDataStruct(const QString& name, TreeNode* node, const int layer, const int countOrList)
    : paramName(name)
    , node(node)
    , layer(layer)
    , countOrList(countOrList)
{}

void HelpDictionaryServiceController::mainParamParce(QString& str, QStringList& mainList,
                                  QVector<TmpDataStruct> &params
                                  , bool isNeedMain
                                  , int layer
                                  , int orCount)
{
    if(str.count('<') != str.count('>')
        || str.count('[') != str.count(']')
        || str.count('{') != str.count('}'))
    {
        qWarning() << "String: " << str << " has problem";
    }

    QVector<QString> list;
    QString tmpStr;
    QStack<int> triangl;
    QStack<int> rect;
    QStack<int> fig;
    int startSq = -1;
    QStack<QPair<int,int>> indexesForDel;
    QVector<QString> nonDelList;
    QVector<char> keySymbols = {' ', '<', '>', '[', ']', '{', '}'};

    auto addStrToList = [&tmpStr, &list](){
        if(!tmpStr.isEmpty() && tmpStr != ' ')
        {
            list.append(tmpStr);
            tmpStr.clear();
        }
    };

    for(int i = 0; i < str.size(); i++)
    {
        bool isSpace = str[i] == keySymbols[0];
        bool isTriangeOpen = str[i] == keySymbols[1];
        bool isTriangeClose = str[i] == keySymbols[2];
        bool isRectOpen = str[i] == keySymbols[3];
        bool isRectClose = str[i] == keySymbols[4];
        bool isBraceOpen = str[i] == keySymbols[5];
        bool isBraceClose = str[i] == keySymbols[6];

        if(!keySymbols.contains(str[i].toLatin1()))
        {
            if(rect.isEmpty())
            {
                tmpStr +=str[i];
            }

        }
        else if(isRectOpen)
        {
            rect.push_back(i);
            addStrToList();
            if(rect.size() == 1)
            {
                startSq = i;
                tmpStr = "[]";
                addStrToList();
            }
        }
        else if(isRectClose)
        {
            if(!rect.isEmpty())
            {
                rect.pop();
                indexesForDel.append({startSq, i-startSq + 1});
                startSq = -1;
            }
        }
        else if(!rect.empty())
        {
            continue;
        }
        else if(isSpace)
        {
            if(triangl.isEmpty() && fig.isEmpty() && rect.isEmpty())
            {
                addStrToList();
            }
            else if(!triangl.isEmpty() || !fig.isEmpty())
            {
                tmpStr +=str[i];
            }
        }
        else if(isTriangeOpen && !fig.empty())
        {
            tmpStr +=str[i];
        }
        else if(isTriangeClose && !fig.empty())
        {
            tmpStr +=str[i];
        }
        else if(isTriangeOpen && fig.empty())
        {
            if(triangl.isEmpty())
            {
                addStrToList();
            }
            triangl.push_back(i);
            tmpStr +=str[i];
        }
        else if(isTriangeClose && fig.empty())
        {
            tmpStr +=str[i];
            if(triangl.length())
                triangl.pop();

            if(triangl.empty())
            {
                addStrToList();
            }
        }
        else if(isBraceOpen)
        {
            if(fig.isEmpty())
            {
                addStrToList();
            }
            fig.push_back(i);
            tmpStr +=str[i];
        }
        else if(isBraceClose)
        {
            tmpStr +=str[i];
            fig.pop();
            if(fig.isEmpty())
            {
                addStrToList();
            }
        }

        if(str.size() == i + 1)
        {
            addStrToList();
        }
    }

    auto addNondelList = [&indexesForDel, &str, &nonDelList](){
        if(!indexesForDel.isEmpty())
        {
            while(!indexesForDel.isEmpty())
            {
                auto item = indexesForDel.pop();
                nonDelList.append(str.mid(item.first, item.second));
                str.remove(item.first, item.second);
            }
        }
    };

    addNondelList();

    int sizeList = list.size();
    bool isMainEnd = isNeedMain ? false : true;
    QVector<int> blackList;

    for(int i = 0; i < sizeList; i++)
    {
        if(i == 0 && !isMainEnd)
        {
            mainList.append(list[i]);
            continue;
        }

        bool isDashCur      = list[i].contains('-');
        bool isTrienglCur   = list[i].contains('<');
        bool isCurlyCur     = list[i].contains('{');
        bool isOrCur        = list[i].contains('|');
        bool isRectCur      = list[i].contains('[');

        // {-cert <cert name> | -hash <cert hash>} current value
        if(isCurlyCur && (isDashCur || isOrCur || isTrienglCur))
        {
            blackList.append(i);
            int index = str.indexOf(list[i]);
            indexesForDel.append({index, index + list[i].size()});
            continue;
        }
        else if(isRectCur)
        {
            continue;
        }
        if(i + 1 >= sizeList)
        {
            if(!isRectCur && !isCurlyCur && !isTrienglCur)
            {

                params.append({list[i], new TreeNode(), layer, orCount});
            }
            continue;
        }

        bool isDashNext         = list[i + 1].contains('-');
        bool isTrienglNext      = list[i + 1].contains('<');
        bool isCurlyNext        = list[i + 1].contains('{');
        bool isOrNext           = list[i + 1].contains('|');
        bool isRectNext         = list[i + 1].contains('[');
        bool isPointTrienglNext = list[i + 1].contains(".<");

        if(!isRectNext && i + 1 >= sizeList)
        {
                params.append(TmpDataStruct(list[i], new TreeNode(), layer, orCount));
        }

        if((!isOrCur && !isCurlyCur && !isTrienglCur) &&
            isRectNext && !isMainEnd)
        {
            mainList.append(list[i]);
            isMainEnd = true;
            continue;
        }
        if(!isMainEnd && !isOrNext && !isTrienglNext && (!isOrCur && !isCurlyCur && !isTrienglCur))
        {
            mainList.append(list[i]);
            continue;
        }
        else if(!isMainEnd && (!isOrCur && !isCurlyCur && !isTrienglCur) && isDashNext)
        {
            mainList.append(list[i]);
            isMainEnd = true;
            continue;
        }
        else if(!isMainEnd && isTrienglNext)
        {
            isMainEnd = true;
        }

        //ca del -hash
        if((!isOrCur && !isCurlyCur && !isTrienglCur) &&
            (!isOrNext && !isCurlyNext && !isTrienglNext && !isRectNext))
        {
            if(i + 2 < sizeList)
            {
                if(!list[i + 2].contains('<') && !list[i + 2].contains('{'))
                {
                    if(!isMainEnd)
                    {
                        mainList.append(list[i]);
                        i++;
                        mainList.append(list[i]);
                    }
                    else
                    {
                        params.append(TmpDataStruct(list[i], new TreeNode(), layer, orCount));
                        i++;
                        params.append(TmpDataStruct(list[i], new TreeNode(), layer, orCount));
                    }
                }
                else
                {
                    params.append(TmpDataStruct(list[i], new TreeNode(), layer, orCount));
                }
            }
            else
            {
                if(!isMainEnd)
                {
                    mainList.append(list[i]);
                }
                else
                {
                    params.append(TmpDataStruct(list[i], new TreeNode(), layer, orCount));
                    params.append(TmpDataStruct(list[i + 1], new TreeNode(), layer, orCount));
                    i++;
                }
            }
            continue;
        }
        else
        {
            if(!isMainEnd)
            {
                isMainEnd = true;
            }
        }

        // -mode {update | all}, go {online | offline | sync}
        // -from {events | events_lasts | threshold | round.new | round.<Round id in hex>}
        if((!isOrCur && !isCurlyCur && !isTrienglCur)
            && ((isOrNext && isCurlyNext && !isTrienglNext && !isDashNext && !isRectNext)
                || (isOrNext && isCurlyNext && isPointTrienglNext && !isDashNext && !isRectNext)))
        {
            TreeNode* tmpNode = new TreeNode();
            QString& secondParam = list[i+1];
            QStringList paramsList = secondParam.split('|');
            for(QString& strList: paramsList)
            {
                strList.remove('{');
                strList.remove('}');
                tmpNode->data.append(strList.trimmed());
            }
            params.append(TmpDataStruct(list[i], std::move(tmpNode), layer, orCount));
            i++;
            continue;
        }

        // {-cert <cert name> | -hash <cert hash>} next value
        if((!isOrCur && !isCurlyCur && !isTrienglCur) && isCurlyNext
            && (isDashNext || isOrNext || isTrienglNext) && !isPointTrienglNext)
        {
            if(!isMainEnd)
            {
                mainList.append(list[i]);
                isMainEnd = true;
            }
            else
            {
               params.append(TmpDataStruct(list[i], new TreeNode(), layer, orCount));
            }
            continue;
        }

        // list [-from_hash <block_hash>]
        if((!isOrCur && !isCurlyCur && !isTrienglCur) && isRectNext)
        {
            if(!isMainEnd)
            {
               mainList.append(list[i]);
               isMainEnd = true;
            }
            else
            {
               params.append(TmpDataStruct(list[i], new TreeNode(), layer, orCount));
            }
            continue;
        }

        // -n <chain net name>
        if((!isOrCur && !isCurlyCur && !isTrienglCur) &&
            (!isOrNext && !isCurlyNext && isTrienglNext && !isDashNext && !isRectNext))
        {
            TreeNode* tmpNode = new TreeNode();
            tmpNode->data.append(list[i+1]);
            params.append(TmpDataStruct(list[i], std::move(tmpNode), layer, orCount));
            i++;
            continue;
        }
    }

    addNondelList();

    for(int i = 0; i < sizeList; i++ )
    {
        if (!blackList.contains(i) && !list[i].contains('['))
        {
            int index = str.indexOf(list[i]);
            str.remove(index, list[i].size());
        }
    }
    str = str.trimmed();
    for(auto& item: nonDelList)
    {
        str += item;
    }
}

