#pragma once
#include <QObject>
#include <QProcess>
#include <QString>
#include <QStringList>
#include <QMap>
#include <QDir>
#include <QFile>
#include <QDebug>
#include <QRegularExpression>
#include <QRegularExpressionMatch>
#include <QRegularExpressionMatchIterator>

#ifdef Q_OS_ANDROID
#include <QtAndroid>
#include <QAndroidJniObject>
#endif
#ifdef Q_OS_WIN
#include "registry.h"
#endif

#include "TreeDictionaryBase.h"
#include "DapNodePathManager.h"

class HelpDictionaryServiceController : public QObject
{
    Q_OBJECT

    struct TmpDataStruct
    {
        QString paramName = "";
        TreeNode* node = nullptr;
        int layer = 0;
        int countOrList = 0;

        TmpDataStruct(const QString& name, TreeNode* node, const int layer, const int countOrList = 0);
    };

public:
    explicit HelpDictionaryServiceController(QObject *parent = nullptr);
    ~HelpDictionaryServiceController();

    QJsonDocument getJsonDictionary();
    QJsonDocument getNewJsonDictionary();

    void setNodeVersion(const QString& version, bool isUpdate = true);
    const QString& getNodeVersion() {return m_nodeVersion;}
private:
    void tryTestVersions();

    bool newDictionary();
    void onInit();
    TreeNode* creatDictionary();
    bool saveDictionary();
    bool loadDictionary();

    QMap<QString, QString> getRawData();

    QString requestToNode(QString cmd);
    QVector<QPair<QString, int>> findSubstrings(QString& input, const char openCurl = '[', const char closeCurl = ']');

    void mainParamParce(QString& str, QStringList &mainList
                        , QVector<TmpDataStruct> &params
                        , bool isNeedMain = true, int layer = 0, int orCount = 0);

private:
    QString *m_sCliPath;

    TreeDictionaryBase* m_treeDictionary = nullptr;
    QString m_nodeVersion = "";
    QString m_nodeVersionFromFile = "";

private: //const property
    const QString CMD_HELP = "help";
    QString test = QString(DAP_BRAND).toLower();

#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
    const QString FILE_NAME = QString("/opt/%1/data/HelpDictionary.bin").arg(QString(DAP_BRAND).toLower());
#elif defined (Q_OS_MACOS)
    const QString FILE_NAME = QString("/tmp/HelpDictionary.bin");
#elif defined (Q_OS_WIN)
    const QString FILE_NAME = QString("%1/%2/data/HelpDictionary.bin").arg(regWGetUsrPath()).arg(DAP_BRAND);
#elif defined (Q_OS_ANDROID)
    static QAndroidJniObject l_pathObj = QtAndroid::androidContext().callObjectMethod(
                "getExternalFilesDir"
                , "(Ljava/lang/String;)Ljava/io/File;"
                , QAndroidJniObject::fromString(QString("")).object());
    const QString FILE_NAME = QString("%1/data/HelpDictionary.bin").arg(l_pathObj.toString());
#else
    const QString FILE_NAME = "/tmp/HelpDictionary.bin";
#endif

};

