#include "TreeDictionaryBase.h"
#include <QJsonObject>
#include <QJsonArray>

TreeNode::~TreeNode()
{
    for(TreeNode* item: qAsConst(children))
    {
        delete item;
    }
}

TreeDictionaryBase::TreeDictionaryBase()
{}

TreeDictionaryBase::~TreeDictionaryBase()
{
    if(m_tree)
    {
        delete m_tree;
    }
}

QJsonObject TreeDictionaryBase::intMapToJson(const QMap<QString, int>& map)
{
    QJsonObject json;
    for (auto it = map.begin(); it != map.end(); ++it) {
        json[it.key()] = it.value();
    }
    return json;
}

QMap<QString, int> TreeDictionaryBase::intMapFromJson(const QJsonObject& json)
{
    QMap<QString, int> map;
    for (auto it = json.begin(); it != json.end(); ++it) {
        map[it.key()] = it.value().toInt();
    }
    return map;
}

QJsonArray TreeDictionaryBase::stringListToJson(const QStringList& list)
{
    QJsonArray json;
    for (const auto& str : list) {
        json.append(str);
    }
    return json;
}

QStringList TreeDictionaryBase::stringListFromJson(const QJsonArray& json)
{
    QStringList list;
    for (const auto& val : json) {
        list.append(val.toString());
    }
    return list;
}

QJsonArray TreeDictionaryBase::vectorStringListToJson(const QVector<QStringList>& vec)
{
    QJsonArray json;
    for (const auto& list : vec) {
        json.append(stringListToJson(list));
    }
    return json;
}

QVector<QStringList> TreeDictionaryBase::vectorStringListFromJson(const QJsonArray& json)
{
    QVector<QStringList> vec;
    for (const auto& val : json) {
        vec.append(stringListFromJson(val.toArray()));
    }
    return vec;
}

QJsonObject TreeDictionaryBase::getJSONTree()
{
    if(!m_tree)
    {
        qDebug() << "it is not possible to convert to json because a tree has not been created";
        return {};
    }
    QJsonObject result;
    for(auto& key: m_tree->children.keys())
    {
        result.insert(key, recursiveGetJSONTree(m_tree->children.value(key)));
    }
    return result;
}

QJsonObject TreeDictionaryBase::recursiveGetJSONTree(TreeNode* node)
{
    QJsonObject result;
    result.insert("name", node->name);

    if(!node->orList.isEmpty())
    {
        result.insert("orList", intMapToJson(node->orList));
    }
    if(!node->optionalList.isEmpty())
    {
        result.insert("optionalList", QJsonArray::fromStringList(node->optionalList));
    }
    if(!node->data.isEmpty())
    {
        result.insert("data", QJsonArray::fromStringList(node->data));
    }
    if(!node->variantList.isEmpty())
    {
        result.insert("variantList", vectorStringListToJson(node->variantList));
    }
    if(!node->children.isEmpty())
    {
        QJsonObject children;
        for(auto& key: node->children.keys())
        {
            children.insert(key, recursiveGetJSONTree(node->children.value(key)));
        }
        result.insert("children", children);
    }
    return result;
}

void TreeDictionaryBase::dictionaryFromJSON(const QJsonObject& tree)
{
    if(tree.isEmpty())
    {
        return;
    }

    TreeNode* node = new TreeNode();

    auto keys = tree.keys();
    for(auto key: keys)
    {
        TreeNode* tmpNode = new TreeNode();
        recursiveTreeNodeFromJson(tmpNode, tree[key].toObject());
        tmpNode->previousNode = node;
        node->children.insert(key, tmpNode);
    }

    if(m_tree)
    {
        delete m_tree;
    }

    m_tree = std::move(node);
}

void TreeDictionaryBase::recursiveTreeNodeFromJson(TreeNode* node, const QJsonObject& tree)
{
    if(tree.contains("name"))
    {
        node->name = tree["name"].toString();

    }
    if(tree.contains("orList"))
    {

//--------TODO: for local win build.--------//
//        for (auto it = tree["orList"].toObject().begin(); it != tree["orList"].toObject().end(); ++it) {
//            QString key = it.key();
//            int val = it.value().toInt();}
//-----------------------------------------//

        node->orList.insert(intMapFromJson(tree["orList"].toObject()));
    }
    if(tree.contains("optionalList"))
    {
        QStringList list = stringListFromJson(tree["optionalList"].toArray());
        node->optionalList.append(list);
    }
    if(tree.contains("data"))
    {
        QStringList list = stringListFromJson(tree["data"].toArray());
        node->data.append(list);
    }
    if(tree.contains("variantList"))
    {
        QVector<QStringList> list = vectorStringListFromJson(tree["variantList"].toArray());
        node->variantList.append(list);
    }

    if(tree.contains("children"))
    {
        auto keys = tree["children"].toObject().keys();
        for(auto key: keys)
        {
            TreeNode* tmpNode = new TreeNode();
            recursiveTreeNodeFromJson(tmpNode, tree["children"][key].toObject());
            tmpNode->previousNode = node;
            node->children.insert(key, tmpNode);
        }
    }
}
