#ifndef TREEDICTIONARYBASE_H
#define TREEDICTIONARYBASE_H

#include <QObject>
#include <QMap>
#include <QHash>
#include <QString>
#include <QStringList>
#include <QVector>
#include <QDebug>

/// The structure of the tree node.
/// children - Branching is carried out. here are the main parameters.
/// data - data for the specified parameter.
/// variantList - the branches of variants are recorded.
///  To filter out the data in the list.
/// orList - to filter out the list of parameters that should not be together.
/// optionalLast - parameters that are not required to execute the command.
/// previousNode - is the address of the previous note for maneuvering through the list.
///  Creates a bidirectional graph.
/// name - the name of the node to clarify the position.
struct TreeNode
{
    QString name = "";
    QHash<QString, TreeNode*> children;
    TreeNode* previousNode = nullptr;
    QMap<QString, int> orList;
    QStringList optionalList;
    QStringList data;
    QVector<QStringList> variantList;

    ~TreeNode();
};

class TreeDictionaryBase
{
public:
    TreeDictionaryBase();
    ~TreeDictionaryBase();

    TreeNode* getTree() const {return m_tree;}
    void setTree(TreeNode* tree) { m_tree = tree;}

    bool isTree() {return m_tree != nullptr;}

    QJsonObject getJSONTree();
    void dictionaryFromJSON(const QJsonObject &tree);
private:
    void recursiveTreeNodeFromJson(TreeNode* node, const QJsonObject &tree);
    QJsonObject recursiveGetJSONTree(TreeNode* node);
    QVector<QStringList> vectorStringListFromJson(const QJsonArray& json);
    QJsonArray vectorStringListToJson(const QVector<QStringList>& vec);
    QStringList stringListFromJson(const QJsonArray& json);
    QJsonArray stringListToJson(const QStringList& list);
    QMap<QString, int> intMapFromJson(const QJsonObject& json);
    QJsonObject intMapToJson(const QMap<QString, int>& map);

private:
    TreeNode* m_tree = nullptr;
};

#endif // TREEDICTIONARYBASE_H
