#include "dapconfigreader.h"

#include <QDebug>

#if defined (Q_OS_MACOS)
#include "dap_common.h"
#endif

#ifdef Q_OS_WIN
#include "registry.h"
#endif

DapConfigReader::DapConfigReader() : config(NULL)
{
    QString config_path = getConfigPath();
    if (!dap_config_init(config_path.toLocal8Bit()))
    {
        config = dap_config_open("cellframe-node");
        if (config == NULL)
            qDebug() << "Error in dap_config_open!";
        else
            status = true;
    }
    else
        qDebug() << "Error in dap_config_init!" << config_path;
}

bool DapConfigReader::getConfigStatus()
{
    return status;
}

bool DapConfigReader::getItemBool(const char *a_section_path, const char *a_item_name, bool a_default)
{
    if (config != NULL)
        return dap_config_get_item_bool_default(
                    config, a_section_path, a_item_name, a_default);
    else
        return a_default;
}

QString DapConfigReader::getItemString(const char *a_section_path, const char *a_item_name, const char * a_default)
{
    if (config != NULL)
        return QString::fromLatin1(dap_config_get_item_str_default(
                    config, a_section_path, a_item_name, a_default));
    else
        return a_default;
}

int DapConfigReader::getItemInt(const char *a_section_path, const char *a_item_name, int a_default)
{
    if (config != NULL)
        return dap_config_get_item_int32_default(
                    config, a_section_path, a_item_name, a_default);
    else
        return a_default;
}

QString DapConfigReader::getConfigPath()
{
#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
    return QString("/opt/%1-node/etc").arg(DAP_BRAND_BASE_LO);
#elif defined (Q_OS_MACOS)
    char * l_username = NULL;
    exec_with_ret(&l_username,"whoami|tr -d '\n'");
    if (!l_username)
    {
        qWarning() << "Fatal Error: Can't obtain username";
        return QString();
    }
    QDir dirOld(QString("/Users/%1/Applications/Cellframe.app/Contents/Resources/etc").arg(l_username));
    QDir dirNew("/Applications/CellframeNode.app/Contents/Resources/etc");
    QDir dirNewNew("/Library/Application\ Support/CellframeNode/etc/");

    if(dirNewNew.exists())
    {
        QFileInfo cfg(QString(dirNewNew.absolutePath() + "/cellframe-node.cfg"));
        if(cfg.exists())
            return dirNewNew.absolutePath();
        else
            return "";
    }
    if(dirNew.exists())
    {
        QFileInfo cfg(QString(dirNew.absolutePath() + "/cellframe-node.cfg"));
        if(cfg.exists())
            return dirNew.absolutePath();
        else
            return "";
    }
    else if(dirOld.exists())
    {
        QFileInfo cfg(QString(dirOld.absolutePath() + "/cellframe-node.cfg"));
        if(cfg.exists())
            return dirOld.absolutePath();
        else
            return "";
    }
    else
        return "";
#elif defined (Q_OS_WIN)
    return QString("%1/cellframe-node/etc").arg(regWGetUsrPath());
#elif defined Q_OS_ANDROID
    return QString("/sdcard/cellframe-node/etc");
#else
    return QString();
#endif
}
