#include "DapCertificateOperation.h"
#include <QString>

#include "DapCertificateType.h"
#include "DapCertificateCommands.h"
#include "CellframeNodeConfig.h"
#include <QJsonArray>
#include <QJsonObject>
#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QDebug>
#include <QProcess>


#include "dap_cert_file.h"
#include "dap_binary_tree.h"
#include "dap_common.h"
#include "dap_config.h"
#include "dap_string.h"
#include "dap_strfuncs.h"



bool DapCertificateOperation::addMetaDataToCert(const QString &certName, const QVariantMap& metaData, const QString &s_toolPath)
{
    Q_UNUSED(certName)
    Q_UNUSED(s_toolPath)

    auto iterator = metaData.constBegin();
    while (iterator != metaData.constEnd()) {
        QString value = iterator.value().toString();
        QProcess process;

        QString command = QString("cert add_metadata %1 %2")
                                   .arg(certName).arg(QString("%1:%2:%3:%4")    //  key:type:length:value
                                                                          .arg(iterator.key())
                                                                          .arg(dap_cert_metadata_type::DAP_CERT_META_STRING)
                                                                          .arg(value.size())
                                                                          .arg(value)
                                                                       );

        qDebug() << "DapCertificateOperation::addMetaDataToCert";
        command = command.toLatin1();
        qInfo() << "command:" << command;

        QString result = requestToNode(s_toolPath, command);

        ++iterator;
    }


    return true;
}


//bool DapCertificateOperation::addMetaDataToCert(dap_cert_t *certFile, const QVariantMap &metaData)
//{
//    auto iterator = metaData.constBegin();
//    while (iterator != metaData.constEnd()) {
////        QString value = iterator.value().toString();

////        QString(":%1:%2:%3")    //  :%3:%4"  key:type:length:value
////                .arg(dap_cert_metadata_type::DAP_CERT_META_STRING)
////                .arg(value.size())
////                .arg(value)
////                ;

//        ++iterator;
//    }   //

//    return true;
//}


QJsonObject DapCertificateOperation::createCertificate(const QString &certName, const QString &signatureType, const QVariantMap &metaData
                                                       , QString &status, QString &errorMessage, const QString& s_toolPath)
{
    QString path = CellframeNodeConfig::instance()->getDefaultCADir() + QString("/%1.dcert").arg(certName);
    qInfo() << "[DapCertificateOperation][createCertificate] path = " << path;
    QFileInfo info(path);

    if (info.exists()){
        status = DapCertificateCommands::statusFAIL();
        ///TODO: It should be replaced with an error code and the text should be transferred to qml.
        errorMessage = tr("Certificate already exists");

        return QJsonObject({
                               { "fileName", certName }
                          });
    }


    static const QMap<QString, dap_enc_key_type_t> convertSignatureType = QMap<QString, dap_enc_key_type_t>({
                                                                          { "dilithium", DAP_ENC_KEY_TYPE_SIG_DILITHIUM }
                                                                        , { "picnic", DAP_ENC_KEY_TYPE_SIG_PICNIC }
                                                                        , { "bliss", DAP_ENC_KEY_TYPE_SIG_BLISS }
                                                                        , { "tesla", DAP_ENC_KEY_TYPE_SIG_TESLA }
                                                                        , { "falcon", DAP_ENC_KEY_TYPE_SIG_FALCON }
                                                                    });

    qDebug() << "DapCertificateOperation::createCertificate";

    auto command = QString("cert create %1 %2").arg(certName).arg(signatureType);
    qInfo() << "command:" << command;

    QString result = requestToNode(s_toolPath, command);
    qInfo() << "result:" << result.simplified();


    info.refresh();

    if (info.exists()) {       //existsCertificate(certName, s_toolPath)
        qDebug() << "add cert metaData" << metaData.size();
        addMetaDataToCert(certName, metaData, s_toolPath);

        //TODO check error add metadata

        return QJsonObject({
                               { "fileName", info.fileName() }
                             , { "completeBaseName", info.completeBaseName() }
                             , { "filePath", info.filePath() }
                             , { "dirType", DapCertificateType::DefaultDir }
                             , { "accessKeyType", DapCertificateType::PublicAndPrivate  }     //getAccessKeyType(newCert)
                         });
    } else {
        status = DapCertificateCommands::statusFAIL();
        ///TODO: It should be replaced with an error code and the text should be transferred to qml.
        errorMessage = tr("Certificate not created");

        return QJsonObject({
                               { "fileName", certName }
                          });
    }

}   //createCertificate


QJsonObject DapCertificateOperation::deleteCertificate(const QString& filePath, QString& status, QString& errorMessage)
{
    Q_UNUSED(status)
    Q_UNUSED(errorMessage)

    if (!QFile::remove(filePath)) {
        status = DapCertificateCommands::statusFAIL();
        ///TODO: It should be replaced with an error code and the text should be transferred to qml.
        errorMessage = tr("File not deleted.");
    }

    return QJsonObject({
                          { "deletedFilePath", filePath }
                      });
}

QJsonObject DapCertificateOperation::pkeyShow(const QString& certName, QString &status, QString &errorMessage, const QString& s_toolPath)
{
    QString command = QString("cert pkey show %1").arg(certName).toLatin1();

    qDebug() << "DapCertificateOperation::pkeyShow";
    QString result = requestToNode(s_toolPath, command);
    qInfo() << "certName pkey fo cert " << certName << "result:" << result.simplified();

    if(result.contains("0x"))
    {
        status = DapCertificateCommands::statusOK();

        return QJsonObject({
            { "fileName", certName },
            { "hash", result }
        });
    }

    status = DapCertificateCommands::statusFAIL();
    ///TODO: It should be replaced with an error code and the text should be transferred to qml.
    errorMessage = tr("No such certificate was found");

    return QJsonObject({{"fileName", certName } });
}

QJsonObject DapCertificateOperation::moveCertificate(const QStringList& params, const QString& certPath, QString &status, QString &errorMessage)
{
    QString error;
    auto moveFile = [&](const QString& sourceFilePath, const QString& destinationDirPath) -> bool
    {

        auto list = sourceFilePath.split('/');
        QString destPath = destinationDirPath + "/" + list.back();
        QFile destFile(destPath);
        if(destFile.exists())
        {
            qDebug() << "[DapCertificateOperation] [moveCertificate] The file already exists.";
            return true;
        }

        QFile sourceFile(sourceFilePath);
        if (sourceFile.exists())
        {
            QDir destinationDir(destinationDirPath);

            auto tmp = sourceFile.fileName();
            if (QFile::copy(sourceFilePath, destPath)) {
                qDebug() << "[DapCertificateOperation] [moveCertificate]Certificate copyed.";
                return true;
            } else {
                error = "Certificate not copyed.";
                qDebug() << "[DapCertificateOperation] [moveCertificate]Certificate not copyed.";
                return false;
            }
        }
        else
        {
            error = "File not found.";
            qDebug() << "[DapCertificateOperation] [moveCertificate]File not found.";
            return false;
        }
        return false;
    };

    if(moveFile(params[2], certPath))
    {
        status = DapCertificateCommands::statusOK();
    }
    else
    {
        status = DapCertificateCommands::statusFAIL();

        if(error.isEmpty())
        {
            errorMessage = "The certificate could not be transferred";
        }
        else
        {
            errorMessage = error;
        }
    }

    return QJsonObject({{"fileName", params[1] } });
}

QJsonObject DapCertificateOperation::dumpCertificate(const QString &certName, QString &status, QString &errorMessage
                                                     , const QString &s_toolPath)
{
    auto command = QString("cert dump %1").arg(certName);

    QString processResult = requestToNode(s_toolPath, command);
    QJsonObject resultObject;
    if (processResult.isEmpty() || !processResult.contains("Certificate name") ) {
        status = DapCertificateCommands::statusFAIL();
        ///TODO: It should be replaced with an error code and the text should be transferred to qml.
        errorMessage = tr("Certificate not find");
    }
    else
    {
        auto list = processResult.split("\n");

        for(auto& item : list)
        {
            auto itemList = item.split(":");
            if(itemList.size() != 2)
            {
                continue;
            }
            if(itemList[0].contains("Certificate name")) resultObject.insert("name", itemList[1].trimmed());
            else if(itemList[0].contains("Signature type")) resultObject.insert("signature", itemList[1].trimmed());
            else if(itemList[0].contains("Private key size")) resultObject.insert("privateKeySize", itemList[1].trimmed());
            else if(itemList[0].contains("Public key size")) resultObject.insert("publicKeySize", itemList[1].trimmed());
            else if(itemList[0].contains("Metadata section count")) resultObject.insert("metaSectionCount", itemList[1].trimmed());
            else if(itemList[0].contains("Certificates signatures chain size")) resultObject.insert("signatureChainSize", itemList[1].trimmed());
        }
    }

    return resultObject;
}


QJsonObject DapCertificateOperation::dumpCertificateFile(const QString &certName, const QString &filePath, QString &status
                                                         , QString &errorMessage)
{
    QJsonObject result;
    result.insert("filePath", filePath);

    dap_cert_t* certFile = dap_cert_file_load(qPrintable(filePath));

    if (certFile == nullptr) {
        status = DapCertificateCommands::statusFAIL();
        ///TODO: It should be replaced with an error code and the text should be transferred to qml.
        errorMessage = tr("Certificate does not exist");
        result.insert("name", certName);

        return result;
    }

    auto parseCertFile{
        [&result](dap_cert_t * a_cert) {    // -> bool
            result.insert("name", QJsonValue::fromVariant(QString(a_cert->name)));
            result.insert("signature", QJsonValue::fromVariant(QString(dap_sign_type_to_str( dap_sign_type_from_key_type(a_cert->enc_key->type) ))));
            result.insert("privateKeySize", QJsonValue::fromVariant(QVariant::fromValue(a_cert->enc_key->priv_key_data_size)));
            result.insert("publicKeySize", QJsonValue::fromVariant(QVariant::fromValue(a_cert->enc_key->pub_key_data_size)));
            size_t l_meta_items_cnt = dap_binary_tree_count(a_cert->metadata);
            result.insert("metaSectionCount", QJsonValue::fromVariant(QVariant::fromValue(l_meta_items_cnt)));
            result.insert("signatureChainSize", QJsonValue::fromVariant(QVariant::fromValue(dap_cert_count_cert_sign (a_cert))));

            QJsonArray metadata;

            if (l_meta_items_cnt) {
                printf ("Metadata sections\n");
                dap_list_t *l_meta_list = dap_binary_tree_inorder_list(a_cert->metadata);
                dap_list_t *l_meta_list_item = dap_list_first(l_meta_list);
                while (l_meta_list_item) {
                    dap_cert_metadata_t *l_meta_item = (dap_cert_metadata_t *)l_meta_list_item->data;
                    char *l_str;
                    QString key, value;
                    int type = l_meta_item->type;
                    int length = 0;

                    switch (l_meta_item->type) {
                        case DAP_CERT_META_STRING:
                            l_str = strndup((char *)l_meta_item->value, l_meta_item->length);

                            key = l_meta_item->key;
                            length = l_meta_item->length;
                            value = l_str;

                            free(l_str);
                            break;
                        case DAP_CERT_META_INT:
                        case DAP_CERT_META_BOOL:
                            key = l_meta_item->key;
                            length = l_meta_item->length;
                            value = l_str;
                            break;
                        default:
                            l_str = l_meta_item->length ? DAP_NEW_Z_SIZE(char, l_meta_item->length * 2 + 1) : NULL;
                            dap_bin2hex(l_str, l_meta_item->value, l_meta_item->length);
                            key = l_meta_item->key;
                            length = l_meta_item->length;
                            value = l_str;
//                            printf("%s\t%u\t%u\t%s\n", l_meta_item->key, l_meta_item->type, l_meta_item->length, l_str);
                            DAP_DELETE(l_str);
                            break;
                    }  //switch

                    metadata.append(QJsonObject({
                                                    { "key", key }
                                                  , { "type", type }
                                                  , { "length", length }
                                                  , { "value", value }
                                                }));

                    l_meta_list_item = l_meta_list_item->next;
                }
                dap_list_free(l_meta_list);
            }  //if meta_item_cnt

            result.insert("metadata", metadata);
        }
    };


    parseCertFile(certFile);

    dap_cert_delete(certFile);

    return result;
}


bool DapCertificateOperation::existsCertificate(const QString &certName, const QString &s_toolPath)
{
    qDebug() << "DapCertificateOperation::existsCertificate";

    QString command = QString("cert dump %1").arg(certName);
    QString result = requestToNode(s_toolPath, command);
    qInfo() << "result:" << result.simplified();
    //qDebug() << "DapCertificateOperation::existsCertificate" << certName << s_toolPath << result;
    return !result.isEmpty();
}


QJsonObject DapCertificateOperation::exportPublicCertificateToFile(const QString &certName, const QString &newCertName, QString &status
                                                                   , QString &errorMessage, const QString& s_toolPath)
{
    //WARNING сертификат с публичным ключом создается в папке обычных сертификатов
    QString filePath = QString("%1/%2.dcert").arg(CellframeNodeConfig::instance()->getDefaultCADir()).arg(newCertName);
    QFileInfo info(filePath);

    if (info.exists()) {
        status = DapCertificateCommands::statusFAIL();
        ///TODO: It should be replaced with an error code and the text should be transferred to qml.
        errorMessage = tr("Certificate already exists");
        return QJsonObject({
                        { "fileName", newCertName + ".dcert" }
                      , { "completeBaseName", newCertName }
                      , { "filePath", filePath }
                      , { "dirType", DapCertificateType::DefaultDir }
                      , { "accessKeyType", getAccessKeyType(filePath) }     //getAccessKeyType(newCert)
//                      , { "accessKeyType", DapCertificateType::PublicAndPrivate }     //getAccessKeyType(newCert)
                  });
    }

    qDebug() << "DapCertificateOperation::exportPublicCertificateToFile";

    QString command(QString("cert create_cert_pkey %1 %2").arg(certName).arg(newCertName));
    QString result = requestToNode(s_toolPath, command);

    qInfo() << "result:" << result.simplified();
    //WARNING в результате всегда пусто!
    if (!result.isEmpty()) {
        //TODO check process result
    }

    //qDebug() << args << "\nexportPublicCertificateToFile" << filePath << ", result:" << result;
    info.refresh();
    return getSimpleCertificateInfo(info, DapCertificateType::DefaultDir);
}


QJsonObject DapCertificateOperation::exportPublicCertificateToMempool(const QString &network, const QString &certName, QString &status
                                                                      , QString &errorMessage, const QString& s_cliPath)
{
    QString command(QString("mempool_add_ca -net %1 -ca_name %2").arg(network).arg(certName));   // newCertName

    QString result = requestToNode(s_cliPath, command);//WARNING тут тоже ответ пустой

    if (result.isEmpty()) {

    } else {

    }

    //qDebug() << args << "\nexportPublicCertificateToMempool" << certName << network << ", result:" << result;

    return QJsonObject({
                    { "certName", certName }
                  , { "network", network }
              });
}


DapCertificateType::AccessKeyType DapCertificateOperation::getAccessKeyType(const QString &certFilePath)
{
    dap_cert_t* certFile = dap_cert_file_load(qPrintable(certFilePath));
    if (certFile == nullptr){
        qCritical() << "file not open" << certFilePath;
        return DapCertificateType::Error;
        //WARNING нужна более продвинутая обработка ошибок, но по умолчанию вызов этой функции проиходит сразу после получение списка файлов
    }

    auto result = getAccessKeyType(certFile);
    dap_cert_delete(certFile);
    return result;
}


DapCertificateType::AccessKeyType DapCertificateOperation::getAccessKeyType(dap_cert_t *certFile)
{
    int privateKeySize = certFile->enc_key->priv_key_data_size;
    int publicKeySize = certFile->enc_key->pub_key_data_size;

    qDebug() << "DapCertificateOperation::getAccessKeyType"
             << "name" << certFile->name
             << "privateKeySize" << privateKeySize
             << "publicKeySize" << publicKeySize;

    if (privateKeySize == 0 && publicKeySize > 0)
        return DapCertificateType::Public;
    else
    if (privateKeySize > 0 && publicKeySize > 0)
        return DapCertificateType::PublicAndPrivate;
    else
    {
        qWarning() << "Wrong Certificate!" << "privateKeySize" << privateKeySize
                   << "publicKeySize" << publicKeySize;
        return DapCertificateType::Error;
    }
}


QJsonArray DapCertificateOperation::getCertificateList(QString& status, QString& errorMessage)
{
    Q_UNUSED(status)
    Q_UNUSED(errorMessage)

    QJsonArray result;

    auto parseDir =
        [&result](const QString& dirPath, const DapCertificateType::DirType& dirType) -> bool
        {
            QDir dir(dirPath);

            if (!dir.exists()) {
                qWarning() << "The directory does not exist:" << dirPath;
                return false;
            }

            dir.setFilter(QDir::Files | QDir::Hidden);      //only files in derictory
            dir.setSorting(QDir::Name);      //set sort by name

            QFileInfoList list = dir.entryInfoList();   // get all QFileInfo for files in dir

            foreach (const QFileInfo& info, list) {
                if (info.suffix() == "dcert") {       //выбираем только файлы сертификатов с расширением dcert
                    result.append(getSimpleCertificateInfo(info, dirType));
                }

            }

            return true;
        };


    //предполагаем что в разных папках лежат, разные ключи.
    QString privateCa, publicCa;
    privateCa = CellframeNodeConfig::instance()->getDefaultCADir();
    publicCa = CellframeNodeConfig::instance()->getShareCADir();

    parseDir(privateCa, DapCertificateType::DefaultDir );
    parseDir(publicCa,  DapCertificateType::ShareDir   );

    qDebug() << "[CellframeNodeConfig::instance()->getDefaultCADir()] " << privateCa;
    qDebug() << "[CellframeNodeConfig::instance()->getShareCADir()] "   << publicCa;

    for(int i = 0; i < result.count(); i++)
    {
        for(int j = 0; j < result.count(); j++)
        {

            if(result.at(i).toObject().value("fileName").toString() == result.at(j).toObject().value("fileName").toString())
            {
                if(result.at(i).toObject().value("dirType").toInt() != result.at(j).toObject().value("dirType").toInt())
                {
                    if(result.at(i).toObject().value("dirType").toInt())
                        result.removeAt(i);
                    else
                        result.removeAt(j);
                    i--;
                    j--;
                }

                QString names = result.at(j).toObject().value("fileName").toString();
                QRegExp exp("[А-Яа-я]*");
                exp.indexIn(names);
                auto check = exp.cap(0);

                if(check.isEmpty())
                    continue;
                else
                    result.removeAt(i);
                i--;
                j--;
            }
        }
    }


    return result;
}


QJsonObject DapCertificateOperation::getSimpleCertificateInfo(const QFileInfo &info, const DapCertificateType::DirType& dirType)
{
    return QJsonObject({
                            { "fileName", info.fileName() }
                          , { "completeBaseName", info.completeBaseName() }
                          , { "filePath", info.filePath() }
                          , { "dirType", dirType }
                          , { "accessKeyType", getAccessKeyType(info.filePath()) }
                      });
}

QString DapCertificateOperation::requestToNode(const QString& path, const QString& command)
{
    QProcess process;
    process.setProgram(path);
    process.setArguments(command.split(" "));
    process.start();

    if(process.waitForFinished()) {
        auto result = QString::fromLatin1(process.readAll());
        qInfo() << "result:" << result.simplified();
        return result;
    }

    qWarning() << QString("Timeout request: %1").arg(command);
    return QString();
}
