#include "DapAddWalletCommand.h"

const QString DapAddWalletCommand::SUCCESS = "success";
const QString DapAddWalletCommand::MESSAGE = "message";
const QString DapAddWalletCommand::WALLET_NAME = "walletName";

const QString DapAddWalletCommand::SIGNATURE_TYPE = "signatureType";
const QString DapAddWalletCommand::USE_EXISTING = "useExisting";



/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
DapAddWalletCommand::DapAddWalletCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapAddWalletCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QJsonObject resultObj;

    // if string
    QString resultStr = cmdList->addWallet(args).toString();
    if(resultStr.contains("successfully created"))
    {
        QString walletName = args.toStringList().first();

        resultObj.insert(SUCCESS,QJsonValue(true));
        resultObj.insert(MESSAGE,QJsonValue("Wallet successfully created"));
        resultObj.insert(WALLET_NAME,QJsonValue(walletName));
    }
    else
    {
        QString failureMessage;
        if(resultStr.contains("message:"))
            failureMessage = resultStr.split("message:").at(1).split("\n").at(0);
        else
            failureMessage = resultStr;

        resultObj.insert(SUCCESS,QJsonValue(false));
        resultObj.insert(MESSAGE,QJsonValue(failureMessage));
    }


    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultObj}}));
    return resultDoc.toJson();
}

void DapAddWalletCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}
