#include "DapCertificateManagerCommands.h"

#include <QJsonDocument>
#include <QDir>
#include "CertificateManager/DapCertificateOperation.h"

#ifdef Q_OS_WIN
#include "registry.h"
#define PRIV_CERT_PATH QString("%1/cellframe-node/var/lib/ca/").arg(regGetUsrPath())
#define PUB_CERT_PATH QString("%1/cellframe-node/share/ca/").arg(regGetUsrPath())
#endif

#ifdef Q_OS_MAC
#include <sys/stat.h>
#define PUB_CERT_PATH QString("/Users/%1/Applications/Cellframe.app/Contents/Resources/share/ca/").arg(getenv("USER"))
#define PRIV_CERT_PATH QString("/Users/%1/Applications/Cellframe.app/Contents/Resources/var/lib/ca/").arg(getenv("USER"))
#endif

#ifdef Q_OS_LINUX
#include <sys/stat.h>
#define PUB_CERT_PATH QString("/opt/cellframe-node/share/ca")
#define PRIV_CERT_PATH QString("/opt/cellframe-node/var/lib/ca")
#endif

DapCertificateManagerCommands::DapCertificateManagerCommands(const QString &asServicename, QObject *parent
                                                             , const QString &asCliPath, const QString &asToolPath)
    : DapAbstractCommand(asServicename, parent, asCliPath, asToolPath)
{

}


//    result = {
//                  command: <enumcommand>
//                  status: "OK" | "FAIL",
//                  errorMessage: "",            //optional when error
//                  data: ...                    //empty or object or array
//             }
//arg1 DapCertificateCommands, arg2...arg10  - аргументы для конкретной команды
QVariant DapCertificateManagerCommands::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList params = args.toStringList();

    qDebug() << "respondToClient, " << DapCertificateCommands::instance()->commandToString(
                    DapCertificateCommands::Commands(params[0].toInt()));

    bool arg1ConvertSuccess = true;
    int command = params[0].toInt(&arg1ConvertSuccess);

    if (!arg1ConvertSuccess) {
        qCritical() << "command not responsed";

        QJsonObject failObj {
            { "command", DapCertificateCommands::UnknownCommand }
            , { "status", "FAIL" }
            , { "errorMessage", "command not responsed" }
        };

        QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, failObj}}));
        return resultDoc.toJson();
    };


    QString status = DapCertificateCommands::statusOK();
    QString errorMessage;
    QVariant result;


    switch (command) {
        case DapCertificateCommands::GetSertificateList:
            result = DapCertificateOperation::getCertificateList(status, errorMessage);
            break;
        case DapCertificateCommands::CreateCertificate: {
                QVariantMap metaData = QJsonDocument::fromJson(params[3].toUtf8()).toVariant().value<QVariantMap>();
                result = DapCertificateOperation::createCertificate(params[1], params[2], metaData
                                                                    , status, errorMessage, *m_sToolPath);
            }
            break;
        case DapCertificateCommands::MoveCertificate:
            result = DapCertificateOperation::moveCertificate(params, PRIV_CERT_PATH, status, errorMessage);
            break;
        case DapCertificateCommands::DumpCertifiacate:
        {
            QString path;
            if(!params.contains("from"))
            {
                path = params[2];
            }
            else
            {
                if(params.contains("type"))
                {
                    int index = params.indexOf("type");
                    if(index + 1 >= params.size())
                    {
                        path = PRIV_CERT_PATH + "/" + params[1] + ".dcert";
                    }
                    else if(params[index + 1] == "private")
                    {
                        path = PRIV_CERT_PATH + "/" + params[1] + ".dcert";
                    }
                    else
                    {
                        path = params[2];
                    }
                }
                else
                {
                    path = PRIV_CERT_PATH + "/" + params[1] + ".dcert";
                }
            }
            result = DapCertificateOperation::dumpCertificateFile(params[1], path, status, errorMessage);   //this cliPath to tools_path
        }
            break;
        case DapCertificateCommands::ImportCertificate:
            //TODO
            break;
        case DapCertificateCommands::ExportPublicCertificateToFile:
            result = DapCertificateOperation::exportPublicCertificateToFile(params[1], params[2], status, errorMessage, *m_sToolPath);
            break;
        case DapCertificateCommands::ExportPublicCertificateToMempool:
            result = DapCertificateOperation::exportPublicCertificateToMempool(params[1], params[2], status, errorMessage, *m_sCliPath);
            break;
        case DapCertificateCommands::AddSignatureToCertificate:
            //TODO
            break;
        case DapCertificateCommands::DeleteCertificate:
            result = DapCertificateOperation::deleteCertificate(params[1], status, errorMessage);
            break;
        case DapCertificateCommands::UpdateCertificateList:
            //вообще команда только для клиента, если произошел запрос к сервисы то выполняем тоже что и при получении списка
            result = DapCertificateOperation::getCertificateList(status, errorMessage);
            break;
        case DapCertificateCommands::PkeyShow:
            result = DapCertificateOperation::pkeyShow(params[1], status, errorMessage, *m_sToolPath);
            break;
        default:
            command = command < DapCertificateCommands::UnknownCommand && command > DapCertificateCommands::UpdateCertificateList
                      ? DapCertificateCommands::UnknownCommand : command;
            break;
    }

    QJsonObject resultObj;
    if(params.contains(WEB3_KEY))
    {
        if(command == DapCertificateCommands::CreateCertificate)
        {
            // categoryCert
            QString newCertName;
            if(errorMessage.isEmpty() && params[3] == "public")
            {
                newCertName = params[1] + "_public";
                result = DapCertificateOperation::exportPublicCertificateToFile(params[1], newCertName, status, errorMessage, *m_sToolPath);
            }

            if(errorMessage.isEmpty())
            {
                QJsonObject data;
                data.insert("name", params[1]);
                data.insert("type", params[2]);
                if(params[3] == "public")
                    data.insert("public_name", newCertName);
                resultObj.insert(RESULT_KEY, data);
            }
        }
        else if(command == DapCertificateCommands::UpdateCertificateList)
        {
            if(errorMessage.isEmpty())
            {
                // categoryCert
                int accessKeyType = -1;
                if(params[1] == "public")
                    accessKeyType = 0;
                else if(params[1] == "private")
                    accessKeyType = 1;

                QJsonArray resultArr = result.toJsonArray();
                QJsonArray data;
                QJsonObject item;

                for(int i=0; i<resultArr.count(); ++i)
                {
                    item = resultArr[i].toObject();
                    if(accessKeyType >= 0)
                    {
                        if(item["accessKeyType"].toInt() != accessKeyType) continue;
                    }
                    data.append(item["completeBaseName"].toString());
                }
                resultObj.insert(RESULT_KEY, data);
            }
        }

        if(!errorMessage.isEmpty())
            resultObj.insert(ERROR_KEY, errorMessage);
        addWeb3Result(resultObj, args);
    }
    else
    {
        QJsonObject resultOK {
            { "command", command }
            , { "status", status }
            , { "errorMessage", errorMessage }
            , { "data", QJsonValue::fromVariant(result) }
        };

        resultObj.insert(RESULT_KEY, resultOK);
    }

    QJsonDocument resultDoc(resultObj);
    return resultDoc.toJson();
}

void DapCertificateManagerCommands::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}

void DapCertificateManagerCommands::notifedFromService(const QVariant &args)
{
    QStringList params = args.toStringList();

    qInfo() << "DapCertificateManagerCommands::notifedFromService" << params[0] << params[1] << params[2] << params[3];


    bool arg1ConvertSuccess = true;
    int command = params[0].toInt(&arg1ConvertSuccess);

    if (!arg1ConvertSuccess) {
        qCritical() << "command not responsed";
        return ;
    }


    QString status = params[1];
    if (!(status == DapCertificateCommands::statusOK()
          || status == DapCertificateCommands::statusFAIL()))  //правильней будет валидировать статус
    {
        status = DapCertificateCommands::statusFAIL();
    }

    QString errorMessage = params[2];
    const QVariant& response = params[3];
    QVariant result;

    switch (command) {
        case DapCertificateCommands::UpdateCertificateList:
            result = response;
            break;
    }


    emit clientNotifed(QVariant(QJsonObject({
                                                { "command", command }
                                              , { "status", status }
                                              , { "errorMessage", errorMessage }
                                              , { "data", QJsonValue::fromVariant(result) }
                                            })
                                ));
}
