#include "DapCheckQueueTransactionCommand.h"
#include "TransactionQueue/DapTransactionQueueController.h"
#include <QString>

DapCheckQueueTransactionCommand::DapCheckQueueTransactionCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

QVariant DapCheckQueueTransactionCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    DapTransactionQueueController* controller = DapTransactionQueueController::getTransactionController();

    QStringList params = args.toStringList();
    /// 0 - queueHash
    /// 1 - walletName
    /// 2 - networkName

    Q_ASSERT_X(!params[0].isNull(), "DapCheckQueueTransactionCommand", QString("There is no queue hash").toStdString().c_str());
    Q_ASSERT_X(!params[1].isNull(), "DapCheckQueueTransactionCommand", QString("There is no wallet name").toStdString().c_str());
    Q_ASSERT_X(!params[2].isNull(), "DapCheckQueueTransactionCommand", QString("There is no network name").toStdString().c_str());

    QString queueHash = params[0];
    QString walletName = params[1];
    QString networkName = params[2];
    QString txHash;
    QString state;
    int count = -1;
    const auto& queue = controller->getQueue();
    if(queue.contains(networkName))
    {
        const auto& waletsQueue = queue[networkName];
        if(waletsQueue.contains(walletName))
        {
            const auto& listTransaction = waletsQueue[walletName];
            for(int i = 0; i < listTransaction.size(); i++)
            {
                if(listTransaction[i].queueHash == queueHash)
                {
                    txHash = listTransaction[i].hash;
                    count = i;
                    break;
                }
            }
        }
    }

    if(count == 0)
    {
        state = "mempool";
    }
    else if(count > 0)
    {
        state = "queue";
    }
    else
    {
        state = "notFound";
    }

    QJsonObject resultObject, infoObject;
    infoObject.insert("state", state);
    infoObject.insert("txHash", txHash);
    infoObject.insert("queueHash", queueHash);
    infoObject.insert("walletName", walletName);
    infoObject.insert("networkName", networkName);

    resultObject.insert(RESULT_KEY, infoObject);
    addWeb3Result(resultObject, args);
    QJsonDocument resultDoc(resultObject);
    return resultDoc.toJson();
}
