#ifndef DAPCOMMANDLIST_H
#define DAPCOMMANDLIST_H
#include <QString>
#include <QDebug>
#include <QProcess>
#include <QObject>
#include <QMap>
#include <QJsonValue>
#include <QJsonArray>
#include <QJsonObject>
#include <QJsonDocument>
#include <QRegularExpression>
#include <QDateTime>
#include <QPair>

#include <QIODevice>
#include <QLocalSocket>
#include <QTcpSocket>

#include "DapNodePathManager.h"

namespace DapWalletSpace
{
    struct BalanceType
    {
        QString coins = "";
        QString datoshi = "";
        QString token = "";
        QString description = "";
    };

    struct WalletInfoType
    {
        QString walletName = "";
        QString address = "";
        QString networkName = "";
        QList<BalanceType> tokenList;
    };
}

namespace NetworkSpace
{
    struct NetworkStatus
    {
        QString network = "";
        QString currentAddress = "";
        QString activeLinks = "";
        QString totalLinks = "";
        QString currantState = "";
        QString targetState = "";
        QString syncPercent = "";
    };

    struct ValidatorItem
    {
        QString coin = "0.0";
        QString datoshi = "0";
    };

    struct FeeInfoStruct
    {
        QString netCoins = "0.0";
        QString netDatoshi = "0";
        QString netTicker = "";
        QString netAddr = "";
        QString valTicker = "";
        QMap<QString, ValidatorItem> validatorFeeList;
    };
}
Q_DECLARE_METATYPE(NetworkSpace::FeeInfoStruct);
namespace LadgerSpace
{
    struct StatusToken
    {
        QString hash = "";
        QString signType = "";
        QString bytes = "";
        QMap<QString, QString*> respontStruct = {{"hash", &hash}
                                                  ,{"pkey_type", &signType}
                                                  ,{"bytes", &bytes}};
    };

    struct TokensData{

        QString tokenName = "";
        QString type = "";
        QString flags = "";
        QString supplyCurrent = "";
        QString supplyTotal = "";
        QString decimals = "";
        QString tickerTokenFrom = "";
        QString emissionRate = "";
        QString totalEmission = "";
        QString authValid = "";
        QList<StatusToken> statusList;

        QMap<QString, QString*> respontStruct = {{"Token name", &tokenName}
                                                ,{"type", &type}
                                                ,{"flags", &flags}
                                                ,{"Supply current", &supplyCurrent}
                                                ,{"Supply total", &supplyTotal}
                                                ,{"Decimals", &decimals}
                                                ,{"ticker_token_from", &tickerTokenFrom}
                                                ,{"emission_rate", &emissionRate}
                                                ,{"Total emissions", &totalEmission}
                                                ,{"Auth signs valid", &authValid}};
    };
}

namespace Dap {
    template<typename T>
    void getValue(T& data, const QString& str)
    {
        auto tmpList = str.split(":");
        if(tmpList.size() != 2)
        {
            return;
        }

        QString head = tmpList[0].trimmed();
        if(!data.respontStruct.contains(head))
        {
            return;
        }
        *(data.respontStruct[head]) = tmpList[1].trimmed();
    }

    enum RequestType
    {
        CLI = 0,
        JSON
    };

    struct JsonCommandsRespond
    {
        QJsonValue result;
        QJsonValue error;
    };

    const QString RESULT_TEXT_JSON = "result";
    const QString ERROR_TEXT_JSON = "errors";
}

class DapCommandList:public QObject
{
public:
    DapCommandList(QObject *parent=nullptr);

    QVariant getListNetworks();
    QVariant addWallet(const QVariant &args);
    QVariant createPassForWallet(const QVariant &args);
    QVariant getFeeComand(const QString &net);
    NetworkSpace::FeeInfoStruct getFee(const QString &net);
    QVariant getListOrders(const QVariant &args, QString &error);
    QVariant getListTokensCommand(const QString &net);
    QMap<QString, LadgerSpace::TokensData> getListTokens(const QString &net);
    QVariant getLedgerTxHashCommand(const QString &net, const QString &hash, QString &error);
    QVariant getRequestListWallets();
    QVariant getNetworkStateCommand(const QString &net);
    QVariant getNetworkIdCommand(const QString &net);
    NetworkSpace::NetworkStatus getNetworkState(const QString &net, QString &error);
    QVariant getWalletInfoCommand(const QString &net, const QString &wallet, QString &error);
    DapWalletSpace::WalletInfoType getWalletInfo(const QString &net, const QString &wallet, QString *error = nullptr);
    QVariant getTXHistory(const QString &net, const QString &walletName, const QString &walletAddr);
    QVariant createTransaction(const QVariant &args);
    QVariant createJsonTransaction(const QVariant &args);
    QVariant getXchangeOrdersList(const QString &net);
    QVariant getXchangeTokenPairsList(const QString &net);
    QVariant getXchangeAverageCommand(const QVariant &args);
    QPair<qint64, QString> getXchangeAverage(const QVariant &args);
    QVariant getXchangeHistory(const QVariant &args);
    QVariant getXchangeList(const QVariant &args);
    QVariant mempoolProcess(const QVariant &args);
    QVariant networkGoTo(const QVariant &args);
    QVariant networkSingleSync(const QString &net);
    QVariant removeChainsOrGdb(const QVariant &args);
    QVariant runCmd(const QVariant &args);
    QVariant tokenDecl(const QVariant &args);
    QVariant tokenEmission(const QVariant &args);
    QString getVersion();
    QVariant nodeDumpCommand(const QString &net);
    QVariant nodeListCommand(const QString &net);
    QVariant walletActivateOrDeactivate(const QVariant &args);
    QVariant xchangeOrderCreate(const QVariant &args);
    QVariant txCondCreate(const QVariant &args);
    QVariant stakeLockHold(const QVariant &args);
    QVariant stakeLockTake(const QVariant &args);
    QVariant historyInfoByHash(const QVariant &args);
    QVariant xchangeOrderPurchase(const QVariant &args);
    QVariant xchangeOrderRemove(const QVariant &args);
    QVariant getListMempool(const QString &net, const QString &addr, const QString &chain);
    QVariant getMempoolHashInfo(const QVariant &args);
    QVariant getWalletAddress(const QString &net, const QString &wallet);
    QVariant createVPNOrder(const QVariant &args);
    QVariant createStakeOrder(const QVariant &args);
    QVariant createOrderStaker(const QVariant &args);
    QVariant createOrderValidator(const QVariant &args);
    QVariant getListKeysCommand(const QString &net);
    QVariant getListChains();
    QVariant addNode(const QVariant &args);
    QVariant addNodeCommand(const QVariant &args);
    QVariant delNode(const QVariant &args);
    QVariant getServiceLimits(const QVariant &args);
    QVariant srvStakeDelegate(const QVariant &args);
    QVariant srvStakeInvalidate(const QVariant &args);
    QVariant votingCreate(const QVariant &args);
    QVariant votingVote(const QVariant &args);
    QVariant votingList(const QVariant &args);
    QVariant votingDump(const QVariant &args);

    Dap::JsonCommandsRespond getJsonResult(const QVariant& resultVar, const QString &command);

    QString getError(const QJsonValue &json);

    void restartNode();

    bool getParamForPairList(const QStringList &list, const QString &paramName, QString& param);
    bool testJsonBlock(bool expression, const QString& message, const QString& commentForLog);
private:
    QString convertCliToJson(const QString &cmd);
    QVariant wrapJsonError(const QString& msg);
    QVariant requestToNodeJSON(QPair<QString*, QString> cmd);
    QVariant requestToNodeCLI(QPair<QString*, QString> cmd);
    QVariant requestToNode(QPair<QString*, QString>, Dap::RequestType type = Dap::RequestType::CLI);
    bool checkContentLenght(QString body);
    void updateConfigParams();

private slots:
    void initPaths();

private:
    QString *m_cliPath;
    QString *m_toolPath;
    QString *m_nodePath;

    QString m_socketPath = "";
    uint16_t m_socketPort = 0;
};

#endif // DAPCOMMANDLIST_H
