#include "DapCreateJsonTransactionCommand.h"
#include <QFile>

#ifdef Q_OS_WIN
#include "registry.h"
#endif

DapCreateJsonTransactionCommand::DapCreateJsonTransactionCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

QVariant DapCreateJsonTransactionCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList listData = args.toStringList();
    QRegularExpressionMatch match = REGULAR_JSON.match(listData.at(0));

    QString result, error;

    if (!match.hasMatch())
    {
        error = "Incorrect json data";
        qWarning() << "[DapCreateJsonTransactionCommand] Incorrect json data";
    }
    else
    {
        QJsonDocument jsonCommand = QJsonDocument::fromJson(match.captured(0).toStdString().data());

#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
        QString pathJsonCmd =  QString("/opt/%1/data/").arg(DAP_BRAND_LO);
#elif defined(Q_OS_MACOS)
        QString pathJsonCmd =  QString("/tmp/");
#elif defined (Q_OS_WIN)
        QString pathJsonCmd =  QString("%1/%2/data/").arg(regWGetUsrPath()).arg(DAP_BRAND);
#endif

        QString path = pathJsonCmd + "tx_json.json";
        QFile file(path);

        if(file.open(QIODevice::WriteOnly))
        {
            file.write(jsonCommand.toJson());
            file.close();
        }

        result = cmdList->createJsonTransaction(QStringList() << path).toString();
    }

    QJsonObject resultObject;

    QJsonObject mainResult;

    if(result.contains("successfully") || result.contains("tx_create: true"))
    {
        mainResult.insert(SUCCESS,QJsonValue(true));
        mainResult.insert(MESSAGE,QJsonValue(result));

        QRegularExpressionMatch match = REGULAR_HASH.match(result);
        if(match.hasMatch())
        {
            mainResult.insert(HASH, QJsonValue(match.captured(1)));
        }
    }
    else
    {
        mainResult.insert(SUCCESS,QJsonValue(false));
        mainResult.insert(MESSAGE,QJsonValue(result));
        resultObject.insert(ERROR_KEY, error.isEmpty() ? result : error);
    }
    resultObject.insert(RESULT_KEY, mainResult);
    addWeb3Result(resultObject, args);
    QJsonDocument resultDoc;

    resultDoc.setObject(resultObject);

    return resultDoc.toJson();
}
