#include "DapCreateOrderValidatorCommand.h"

DapCreateOrderValidatorCommand::DapCreateOrderValidatorCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapCreateOrderValidatorCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    auto result = cmdList->createOrderValidator(args);
    QJsonObject reply = result.toJsonObject();

    auto returnError = [this, &args](const QJsonObject& object) -> QVariant
    {
        QJsonObject errorObject;
        errorObject.insert(ERROR_KEY, object);
        addWeb3Result(errorObject, args);

        QJsonDocument resultDoc;
        resultDoc.setObject(errorObject);
        return resultDoc.toJson();
    };

    QJsonObject resultObject;
    if(reply.isEmpty())
    {
        return returnError({{"message", "Error. Node gave an empty answer."}});
    }
    if(reply.contains("errors"))
    {
        return returnError({{"message", reply["errors"].toString()}});
    }
    if(!reply["result"].isArray())
    {
        return returnError({{"message", "Error. Answer is does't array."}});
    }
    auto replyResult = reply["result"].toArray();
    if(replyResult.isEmpty() || !replyResult.first().isObject())
    {
        return returnError({{"message", "Error. Answer is does't array."}});
    }
    QJsonObject replyObject = replyResult.first().toObject();
    QJsonObject respondObject;
    if(replyObject.contains("order_hash") && replyObject.contains("status"))
    {
        respondObject.insert("hash", replyObject["order_hash"].toString());
        respondObject.insert("status", replyObject["status"].toString());
    }
    else if(replyObject.contains("errors"))
    {
        auto errorArray = replyObject.value("errors").toArray();
        if(errorArray.isEmpty())
        {
            return returnError({{"message", "Error. The structure of the json output for displaying the error has changed."}});
        }
        auto errorObject = errorArray.first().toObject();
        if(!errorObject.contains("message") && !errorObject.contains("code"))
        {
            return returnError({{"message", "Error. The message or/and error code items were not found."}});
        }

        return returnError({{"message", errorObject.value("message")}
                           ,{"code", errorObject.value("code")}});
    }
    else
    {
        return returnError({{"message", "Error. Unknown error."}});
    }

    resultObject.insert(RESULT_KEY, std::move(respondObject));
    addWeb3Result(resultObject, args);

    QJsonDocument resultDoc;
    resultDoc.setObject(resultObject);
    return resultDoc.toJson();
}
