#include "DapCreateStakeOrder.h"

DapCreateStakeOrder::DapCreateStakeOrder(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapCreateStakeOrder::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList params = args.toStringList();
    QJsonObject stakeOrderObject;

    int index_from = params.indexOf("from");
    if(index_from >= 0 && params.size() > index_from + 1)
    {
        stakeOrderObject.insert("from", params[index_from + 1]);
    }

    QString errorMsg;

    QString status = "";
    QString hash = "";

    auto requestDoc = cmdList->createStakeOrder(args);
    auto request = requestDoc.toJsonObject();

    qDebug() << "[DapCreateStakeOrder] result: " << request;
    if(!request["errors"].isNull())
    {
        QJsonArray errArr = request["errors"].toArray();
        QJsonObject errObj = errArr[0].toObject()["error"].toObject();
        errorMsg = errObj["message"].toString();
        qDebug() << "[DapCreateStakeOrder] ERROR REQUEST: " << errorMsg;
    }
    else if(!request.contains("result"))
    {
        qWarning() << "[DapCreateStakeOrder][respondToClient] The result block was not found, the node output may have changed.";
        errorMsg = "The result block was not found, the node output may have changed.";
    }
    else
    {
        QJsonArray result = request["result"].toArray();

        if(result.isEmpty())
        {
            errorMsg = "It is possible to change the output of the node. there is no result.";
        }
        else
        {
            QJsonObject firstValue = result[0].toObject();
            if(firstValue.contains(ERRORS_KEY))
            {
                qDebug() << "[DapCreateStakeOrder][respondToClient] error: " << firstValue;
                errorMsg = firstValue.value(ERRORS_KEY).toString();
            }
            status = firstValue.value(STUTUS_KEY).toString();
            hash = firstValue.value(ORDER_HASH_KEY).toString();
        }
    }

    if(status == "success")
    {
        stakeOrderObject.insert("success",QJsonValue(true));
        stakeOrderObject.insert("message",QJsonValue("Stake order successfully created"));
        stakeOrderObject.insert(TX_HASH, hash);
    }
    else
    {
        stakeOrderObject.insert("success",QJsonValue(false));
        stakeOrderObject.insert("message",QJsonValue(request));
    }

    QJsonObject resultObject;
    addWeb3Result(resultObject, args);
    resultObject.insert(RESULT_KEY, stakeOrderObject);

    if(!errorMsg.isEmpty())
    {
        resultObject.insert(ERROR_KEY, errorMsg);
    }

    QJsonDocument resultDoc;
    resultDoc.setObject(resultObject);
    return resultDoc.toJson();
}

void DapCreateStakeOrder::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}
