#include "DapCreateTransactionCommand.h"

const QString DapCreateTransactionCommand::SUCCESS = "success";
const QString DapCreateTransactionCommand::TO_QUEUE = "toQueue";
const QString DapCreateTransactionCommand::TX_HASH = "tx_hash";
const QString DapCreateTransactionCommand::ERROR_MESSAGE = "errorMessage";

const QString DapCreateTransactionCommand::WALLET_NAME = "walletName";
const QString DapCreateTransactionCommand::NETWORK = "network";
const QString DapCreateTransactionCommand::RECIEVER_ADDRESS = "recieverAddress";
const QString DapCreateTransactionCommand::AMOUNT = "amount";
const QString DapCreateTransactionCommand::TOKEN = "tokenName";

const QString DapCreateTransactionCommand::TIME = "time";
const QString DapCreateTransactionCommand::ADDRESS = "address";
const QString DapCreateTransactionCommand::HASH = "hash";
const QString DapCreateTransactionCommand::STATE = "state";
const QString DapCreateTransactionCommand::INCOME_TYPE = "income";

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapCreateTransactionCommand::DapCreateTransactionCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1 Network.
/// @param arg2 Chain.
/// @param arg3 Wallet sender.
/// @param arg4 Recipient's wallet address.
/// @param arg5 Token.
/// @param arg6 Transfer amount.
/// @return Reply to client.
QVariant DapCreateTransactionCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

    qInfo() << QString("Creat transaction. network: %1 wallet: %2 toAdress: %3 ticker: %4 datosi_value: %5 datoshi_fee: %6")
        .arg(params[0])
        .arg(params[1])
        .arg(params[2])
        .arg(params[3])
        .arg(params[4])
        .arg(params[5]);

    QString wallet_name = params[1];
    QString reciever_address = params[2];
    QString token = params[3];
    QString value = params[4];

    QJsonObject resultObject, infoObject;

    auto request = cmdList->getJsonResult(cmdList->createTransaction(args), "createTransaction");
    QString error = request.error.toString();

    if(!error.isEmpty())
    {
        error = request.error.toString();
    }
    QString hash, transfer;
    if(error.isEmpty())
    {
        if(cmdList->testJsonBlock(request.result.isNull(), "the result block was not found", "DapCreateTransactionCommand"))
        {
            auto tmpArray = request.result.toArray();
            if(!cmdList->testJsonBlock(tmpArray.isEmpty(), "the array is empty", "DapCreateTransactionCommand")) error = "result is not an array";
            else
            {
                auto tmpFObject = tmpArray[0].toObject();
                if(!cmdList->testJsonBlock(!tmpFObject.contains("hash"), "the error hash block is missing", "DapCreateTransactionCommand")) error = "The output of the node has changed.";
                else
                {
                    hash = tmpFObject["hash"].toString();
                }
                if(!cmdList->testJsonBlock(!tmpFObject.contains("transfer"), "the error transfer block is missing", "DapCreateTransactionCommand")) error = "The output of the node has changed.";
                else
                {
                    transfer = tmpFObject["transfer"].toString();
                }
            }
        }
    }
    if(transfer == "Ok")
    {
        QSettings settings(DAP_BRAND + QString(".ini"), QSettings::IniFormat);
        settings.beginGroup(wallet_name);
        settings.beginGroup(hash);
        settings.setValue(DapCreateTransactionCommand::TIME,       "");
        settings.setValue(DapCreateTransactionCommand::AMOUNT,     "-" + value);
        settings.setValue(DapCreateTransactionCommand::TOKEN,      token);
        settings.setValue(DapCreateTransactionCommand::ADDRESS,    reciever_address);
        settings.endGroup();
        settings.endGroup();

        infoObject.insert(SUCCESS,true);
        infoObject.insert("message", "Transaction created.");
    }
    else
    {
        infoObject.insert(SUCCESS,false);
        infoObject.insert("message", "Some kind of error occurred when creating the transaction.");
    }

    if(!hash.isEmpty())
    {
        infoObject.insert(TX_HASH, hash);
    }

    if(!error.isEmpty())
    {
        resultObject.insert(ERROR_KEY, error);
        infoObject.insert(ERROR_MESSAGE, error);
    }
    else
    {
        infoObject.insert(ERROR_MESSAGE,DapErrors::Error::NOT_ENOUGH_FUNDS_FOR_TRANSFER);
    }

    addWeb3Result(resultObject, args);
    resultObject.insert(RESULT_KEY, infoObject);

    QJsonDocument resultDoc;
    resultDoc.setObject(resultObject);
    return resultDoc.toJson();
}

void DapCreateTransactionCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}
