#ifndef DAPCREATETRANSACTIONCOMMAND_H
#define DAPCREATETRANSACTIONCOMMAND_H

#include <QProcess>
#include "DapAbstractCommand.h"
#include <QDateTime>
#include <QSettings>
#include "DapErrors.h"

class DapCreateTransactionCommand : public DapAbstractCommand
{
public:
    static const QString SUCCESS;
    static const QString TO_QUEUE;
    static const QString TX_HASH;
    static const QString ERROR_MESSAGE;

    static const QString WALLET_NAME;
    static const QString NETWORK;
    static const QString RECIEVER_ADDRESS;
    static const QString AMOUNT;
    static const QString TOKEN;

    static const QString TIME;
    static const QString ADDRESS;
    static const QString HASH;
    static const QString STATE;
    static const QString INCOME_TYPE;

    /// Overloaded constructor.
    /// @param asServiceName Service name.
    /// @param parent Parent.
    /// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
    /// @param asCliPath The path to cli nodes.
    DapCreateTransactionCommand(const QString &asServicename, QObject *parent = nullptr, const QString &asCliPath = QString());

public slots:
    /// Send a response to the client.
    /// @details Performed on the service side.
    /// @param arg1 Network.
    /// @param arg2 Chain.
    /// @param arg3 Wallet sender.
    /// @param arg4 Recipient's wallet address.
    /// @param arg5 Token.
    /// @param arg6 Transfer amount.
    /// @return Reply to client.
    QVariant respondToClient(const QVariant &args = QVariant()) override;
    /// Reply from service.
    /// @details Performed on the service side.
    /// @return Service reply.
    void replyFromService() override;
};

#endif // DAPCREATETRANSACTIONCOMMAND_H
