#include "DapCreateVPNOrder.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
DapCreateVPNOrder::DapCreateVPNOrder(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapCreateVPNOrder::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QVariant v_res = cmdList->createVPNOrder(args);
    QJsonObject resultObject;
    QString result = v_res.toString();
    QJsonObject mainResult;

    if(result.contains("Created order"))
    {
        mainResult.insert("success",QJsonValue(true));
        mainResult.insert("message",QJsonValue("VPN order successfully created"));
        QRegularExpressionMatch match = REGULAR_HASH.match(result);
        if(match.hasMatch())
        {
            mainResult.insert(HASH, QJsonValue(match.captured(1)));
        }
    }
    else
    {
        mainResult.insert("success",QJsonValue(false));
        mainResult.insert("message",QJsonValue(result));
        resultObject.insert(ERROR_KEY, result);
    }
    resultObject.insert(RESULT_KEY, mainResult);
    addWeb3Result(resultObject, args);

    QJsonDocument resultDoc;

    resultDoc.setObject(resultObject);

    return resultDoc.toJson();
}

void DapCreateVPNOrder::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();
    emit serviceResponded(resultObj);
}
