#ifndef DAPDICTIONARYCOMMAND_H
#define DAPDICTIONARYCOMMAND_H

#include "DapAbstractCommand.h"
#include "../autocomplete/HelpDictionaryServiceController.h"

#ifdef Q_OS_WIN
#include "registry.h"
#define PRIV_CERT_PATH QString("%1/cellframe-node/var/lib/ca/").arg(regGetUsrPath())
#define PUB_CERT_PATH QString("%1/cellframe-node/share/ca/").arg(regGetUsrPath())
#define NETWORKS_PATH QString("%1/cellframe-node/etc/network/").arg(regGetUsrPath())
#endif

#ifdef Q_OS_MAC
#include <sys/stat.h>
#define PUB_CERT_PATH QString("/Users/%1/Applications/Cellframe.app/Contents/Resources/share/ca/").arg(getenv("USER"))
#define PRIV_CERT_PATH QString("/Users/%1/Applications/Cellframe.app/Contents/Resources/var/lib/ca/").arg(getenv("USER"))
#define NETWORKS_PATH QString("/Users/%1/Applications/Cellframe.app/Contents/Resources/etc/network/").arg(getenv("USER"))
#endif

#ifdef Q_OS_LINUX
#include <sys/stat.h>
#define PUB_CERT_PATH QString("/opt/cellframe-node/share/ca")
#define PRIV_CERT_PATH QString("/opt/cellframe-node/var/lib/ca")
#define NETWORKS_PATH QString("/opt/cellframe-node/etc/network/")
#endif

class DapDictionaryCommand : public DapAbstractCommand
{
    Q_OBJECT
public:
    explicit DapDictionaryCommand(const QString &asServiceName, QObject *parent = nullptr, const QString &asCliPath = QString());

    ~DapDictionaryCommand();
public slots:

    QVariant respondToClient(const QVariant &args = QVariant()) override;
protected:
    void initRegularRequestController() override;
private:
    QJsonDocument getData();

    void updateCerts();
    void updateTokens();
    void updateMempoolTokens();
    void updateChains();

    void updateData();
    void updateVersionNode(bool isUpdate = true);
private slots:
    void slotUpdateVersionNode();

private:
    HelpDictionaryServiceController* m_dictionary = nullptr;

    bool m_walletsReceived = false;
    bool m_networksReceived = false;

    QStringList m_pubCertName;
    QStringList m_privCertName;
    // network chains
    QMap<QString, QStringList> m_chains;
    // <wallet, network>, tickers
    QMap<QPair<QString, QString>, QStringList> m_tokensTicker;
    // net, tickers
    QMap<QString, QStringList> m_mempoolTokensTicker;
};

#endif // DAPDICTIONARYCOMMAND_H
