#include "DapGetAllWalletHistoryCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetAllWalletHistoryCommand::DapGetAllWalletHistoryCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapGetAllWalletHistoryCommandBase(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1 Network array. Format "<network name 1>:<chain 1>:<address 1>/<network name 2>:<chain 2>:<address 2>/..."
/// @return Reply to client.
QVariant DapGetAllWalletHistoryCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    m_argsCash = args;

    QVariantList params = args.toList();
    qInfo() << "Get history for wallet: " << params[0];
    QJsonArray txHistory;

    QString walletName = params[0].toString();
    bool update = params[1].toBool();
    bool isLastActions = params[2].toBool();

    QSet<QString> hashSet;

    if(isWalletActivated(walletName))
    {
        QJsonArray arrayQueue = getQueueTransactions(walletName);
        QMap<QString, NetCommissionInfo> comissions;

        if(!arrayQueue.isEmpty())
        {
            for(const QJsonValue &item: qAsConst(arrayQueue))
            {
                QJsonObject itemObject = item.toObject();
                QString net = itemObject[NETWORK_KEY].toString();
                if(!comissions.contains(net))
                {
                    comissions.insert(net,parseNetCommissionInfo(getFee()[net]));
                }
                itemObject.insert(FEE_NET_KEY, comissions[net].value);
                itemObject.insert(FEE_TOKEN_KEY, comissions[net].tokenName);
                txHistory.append(std::move(itemObject));
            }
        }
    }

    QStringList netList = getListNetworks();
    for (const QString &net : qAsConst(netList))
    {
        QString walletAddr = "";
        if (!walletName.isEmpty())
        {
            auto netInfo = parseNetCommissionInfo(getFee()[net]);
            QString historyResult = getHistory(net, walletName, walletAddr, netInfo,  hashSet, txHistory);
            if(!historyResult.isEmpty())
            {
                qWarning() << "getHistory() Error. " << historyResult;
            }

            if(walletAddr.isEmpty())
                continue;

            QString mempoolResult = getMempool(net, walletName, walletAddr, "", netInfo, hashSet, txHistory);
            if(!mempoolResult.isEmpty())
            {
                qWarning() << "getMempool() Error. " << mempoolResult;
            }
        }
    }

    QJsonObject resultObject;
    resultObject.insert("history", txHistory);
    resultObject.insert("walletName", walletName);
    resultObject.insert("isLastActions", isLastActions);

    QJsonDocument docResult;
    docResult.setObject(resultObject);
    QByteArray result = docResult.toJson();

    if(_lastResult != result || update)
    {
        _lastResult = result;
        return result;
    }

    return QVariant(QString("isEqual"));
}


bool DapGetAllWalletHistoryCommand::isWalletActivated(QString wallet)
{
    auto walletsList = getListWallets();
    if(walletsList.contains(wallet))
    {
        if(walletsList[wallet] != "non-Active")
        {
            return true;
        }
    }
    return false;
}

void DapGetAllWalletHistoryCommand::transactionFromQueudAdded(const QVariant& args)
{
    QStringList params = args.toStringList();
    /// 0 - network
    /// 1 - wallet
    /// 2 - hash
    ///
    if(params.size() != 3)
    {
        qWarning() << "incorrect number of parameters";
    }

    QString network = params[0];
    QString walletName = params[1];
    QString hash = params[2];
    QPair<QString, bool> newHash = {hash, true};
    if(m_transactionFromQueued.contains(network))
    {
        if(m_transactionFromQueued[network].contains(walletName))
        {
            m_transactionFromQueued[network][walletName] = newHash;

            return;
        }
        else
        {
            m_transactionFromQueued[network].insert(walletName, std::move(newHash));
        }
    }
    else
    {
        QMap<QString, QPair<QString, bool>> newNet;
        newNet.insert(walletName, newHash);
        m_transactionFromQueued.insert(network, std::move(newNet));
    }
}

void DapGetAllWalletHistoryCommand::mempoolTransactionMoved(const QString& network, const QString& walletName, const QString& hash, const QString& ladgerTs)
{
    if(m_transactionFromQueued.contains(network))
    {
        if(m_transactionFromQueued[network].contains(walletName))
        {
            if(m_transactionFromQueued[network][walletName].first == hash)
            {
                if((ladgerTs.isEmpty() || ladgerTs == "DAP_LEDGER_TX_ALREADY_CACHED")
                    && m_transactionFromQueued[network][walletName].second)
                {
                    emit transactionMoved(QStringList() << network << walletName << hash);
                    m_transactionFromQueued[network][walletName].second = false;
                }
            }
        }
    }
}
