#include "DapGetAllWalletHistoryCommandBase.h"
#include <QElapsedTimer>

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetAllWalletHistoryCommandBase::DapGetAllWalletHistoryCommandBase(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QString DapGetAllWalletHistoryCommandBase::getHistory(const QString& net, const QString& walletName, QString& walletAddr,
                                                      DapGetAllWalletHistoryCommandBase::NetCommissionInfo& netInfo,
                                                      QSet<QString>& hashSet,
                                                      QJsonArray& txHistory)
{
    ///TODO: history request time test
    QElapsedTimer timer;
    timer.start();

    QVariant resultDoc;
    if(walletName.isEmpty())
        resultDoc = cmdList->getTXHistory(net, "", walletAddr);
    else
        resultDoc = cmdList->getTXHistory(net, walletName, "");

    auto historyJson = cmdList->getJsonResult(resultDoc, "DapGetAllWalletHistoryCommandBase get history");


    if(!historyJson.error.toString().isEmpty())
        return historyJson.error.toString();

    // 0 - tx data
    // 1 - statistics tx data
    QJsonArray dataRes = historyJson.result.toArray();

    // 0     - address
    // 1     - limit
    // 2...N - tx
    QJsonArray transactions = dataRes.first().toArray();

    ///TODO: history request time test
    qint64 elapsedTime = timer.elapsed();
    qDebug() << QString("[getHistory] The history request for the  %1 network took %2 milliseconds").arg(net).arg(elapsedTime);

    if(transactions.count() > 2)
    {
        for (const auto &tx : qAsConst(transactions))
        {
            QJsonObject txObj = tx.toObject();

            if(txObj.contains("address"))
                walletAddr = txObj["address"].toString();
            else if(txObj.contains("limit"))
                continue;
            else if(txObj.contains("status") && txObj["status"].toString() == "empty")
                continue;
            else
            {
                if(hashSet.contains(txObj["hash"].toString()))
                {
                    continue;
                }
                else
                {
                    hashSet.insert(txObj["hash"].toString());
                }

                QJsonObject txHeader;               
                QMap<typeHistoryData, HistoryData> dataTransaction;

                auto getStatus = [](const QString& tx_status) -> QString
                {
                    return tx_status == "send" ? "Sent" : "Received";
                };

                auto setData = [&txHeader, &dataTransaction, &txHistory, getStatus, this](){

                    // Using these keys, we set options for filling in the history
                    bool isStateLock = dataTransaction.contains(typeHistoryData::STATE_LOCK);
                    bool isValidator = dataTransaction.contains(typeHistoryData::VALIDATOR);
                    bool isNet = dataTransaction.contains(typeHistoryData::NET);
                    bool isEmission = dataTransaction.contains(typeHistoryData::EMISSION);
                    bool isNullAddr = dataTransaction.contains(typeHistoryData::NULL_ADDR);
                    bool isBase = dataTransaction.contains(typeHistoryData::BASE);
                    bool isServicePay = dataTransaction.contains(typeHistoryData::SOME_SERVICE_PAY);
                    bool isReward = dataTransaction.contains(typeHistoryData::REWARD);
                    bool isRewardFee = dataTransaction.contains(typeHistoryData::FEE_REWARD);
                    bool isXChange = dataTransaction.contains(typeHistoryData::XCHANGE);
                    bool isVote = dataTransaction.contains(typeHistoryData::VOTE);

                    if(isReward)
                    {
                        txHeader.insert(VALUE_KEY, dataTransaction[typeHistoryData::REWARD].value);
                        txHeader.insert(TOKEN_KEY, dataTransaction[typeHistoryData::REWARD].token);
                        txHeader.insert(ADDRESS_KEY, "Reward");
                        txHeader.insert(STATUS_KEY, getStatus(dataTransaction[typeHistoryData::REWARD].status));
                        txHeader.insert(DIRECTION_KEY, dataTransaction[typeHistoryData::REWARD].direction);
                    }

                    if(isRewardFee)
                    {
                        txHeader.insert(FEE_KEY, dataTransaction[typeHistoryData::FEE_REWARD].value);
                        txHeader.insert(FEE_TOKEN_KEY, dataTransaction[typeHistoryData::REWARD].token);
                    }

                    if(isStateLock)
                    {
                        txHeader.insert(VALUE_KEY, dataTransaction[typeHistoryData::STATE_LOCK].value);
                        txHeader.insert(TOKEN_KEY, dataTransaction[typeHistoryData::STATE_LOCK].token);
                        txHeader.insert(ADDRESS_KEY, "Stake lock");
                        txHeader.insert(STATUS_KEY, getStatus(dataTransaction[typeHistoryData::STATE_LOCK].status));
                        txHeader.insert(DIRECTION_KEY, dataTransaction[typeHistoryData::STATE_LOCK].direction);
                    }

                    if(isNet)
                    {
                        txHeader.insert(FEE_NET_KEY, dataTransaction[typeHistoryData::NET].value);
                    }

                    if(isValidator)
                    {
                        txHeader.insert(FEE_TOKEN_KEY, dataTransaction[typeHistoryData::VALIDATOR].token);
                        txHeader.insert(FEE_KEY, dataTransaction[typeHistoryData::VALIDATOR].value);
                    }

                    if(isVote)
                    {
                        txHeader.insert(STATUS_KEY, "Vote");
                    }

                    if(isValidator && !isStateLock && !isEmission && !isNullAddr && !isBase && !isServicePay && !isReward && !isVote)
                    {
                        txHeader.insert(STATUS_KEY, "Unknown");
                    }
                    else if(isEmission && isStateLock)
                    {
                        txHeader.insert(M_VALUE_KEY, dataTransaction[typeHistoryData::EMISSION].value);
                        txHeader.insert(M_DIRECTION_KEY, dataTransaction[typeHistoryData::EMISSION].direction);
                        txHeader.insert(M_TOKEN, dataTransaction[typeHistoryData::EMISSION].token);
                    }
                    else if (!isBase && isEmission)
                    {
                        txHeader.insert(VALUE_KEY, dataTransaction[typeHistoryData::EMISSION].value);
                        txHeader.insert(DIRECTION_KEY, dataTransaction[typeHistoryData::EMISSION].direction);
                        txHeader.insert(ADDRESS_KEY, "emission");
                        txHeader.insert(TOKEN_KEY, dataTransaction[typeHistoryData::EMISSION].token);
                        txHeader.insert(STATUS_KEY, getStatus(dataTransaction[typeHistoryData::EMISSION].status));
                    }

                    if(isServicePay)
                    {
                        txHeader.insert(VALUE_KEY, dataTransaction[typeHistoryData::SOME_SERVICE_PAY].value);
                        txHeader.insert(TOKEN_KEY, dataTransaction[typeHistoryData::SOME_SERVICE_PAY].token);
                        txHeader.insert(ADDRESS_KEY, TYPES_SERVICE_TRANSACTION[dataTransaction[typeHistoryData::SOME_SERVICE_PAY].address]);
                        txHeader.insert(STATUS_KEY, getStatus(dataTransaction[typeHistoryData::SOME_SERVICE_PAY].status));
                        txHeader.insert(DIRECTION_KEY, dataTransaction[typeHistoryData::SOME_SERVICE_PAY].direction);
                    }

                    if(!isBase && !isEmission && !isStateLock && isNullAddr)
                    {
                        txHeader.insert(VALUE_KEY, dataTransaction[typeHistoryData::NULL_ADDR].value);
                        txHeader.insert(DIRECTION_KEY, dataTransaction[typeHistoryData::NULL_ADDR].direction);
                        txHeader.insert(TOKEN_KEY, dataTransaction[typeHistoryData::NULL_ADDR].token);
                        txHeader.insert(STATUS_KEY, getStatus(dataTransaction[typeHistoryData::NULL_ADDR].status));
                        txHeader.insert(ADDRESS_KEY, "null");
                    }
                    if(!isBase && !isEmission && isStateLock && isNullAddr)
                    {
                        txHeader.insert(M_VALUE_KEY, dataTransaction[typeHistoryData::NULL_ADDR].value);
                        txHeader.insert(M_DIRECTION_KEY, dataTransaction[typeHistoryData::NULL_ADDR].direction);
                        txHeader.insert(M_TOKEN, dataTransaction[typeHistoryData::NULL_ADDR].token);
                        txHeader.insert(ADDRESS_KEY, "null");
                    }

                    if(isBase)
                    {
                        txHeader.insert(STATUS_KEY, getStatus(dataTransaction[typeHistoryData::BASE].status));
                        txHeader.insert(VALUE_KEY, dataTransaction[typeHistoryData::BASE].value);
                        txHeader.insert(ADDRESS_KEY, dataTransaction[typeHistoryData::BASE].address);
                        txHeader.insert(DIRECTION_KEY, dataTransaction[typeHistoryData::BASE].direction);
                        txHeader.insert(TOKEN_KEY, dataTransaction[typeHistoryData::BASE].token);
                    }

                    if(!isBase && isXChange)
                    {
                        txHeader.insert(STATUS_KEY, getStatus(dataTransaction[typeHistoryData::XCHANGE].status));
                        txHeader.insert(VALUE_KEY, dataTransaction[typeHistoryData::XCHANGE].value);
                        txHeader.insert(ADDRESS_KEY, "SRV_XCHANGE");
                        txHeader.insert(DIRECTION_KEY, dataTransaction[typeHistoryData::XCHANGE].direction);
                        txHeader.insert(TOKEN_KEY, dataTransaction[typeHistoryData::XCHANGE].token);
                    }

                    if(isBase && isXChange)
                    {
                        txHeader.insert(XCHANGE_VALUE_KEY, dataTransaction[typeHistoryData::XCHANGE].value);
                        txHeader.insert(XCHANGE_TOKEN_KEY, dataTransaction[typeHistoryData::XCHANGE].token);
                        txHeader.insert(XCHANGE_DIRECTION_KEY, dataTransaction[typeHistoryData::XCHANGE].direction);
                        txHeader.insert(STATUS_KEY, "Exchange");
                    }

                    dataTransaction.clear();

                    if(txHeader.contains(TX_HASH_KEY) && !txHeader[TX_HASH_KEY].toString().isEmpty())
                    {
                        txHistory.append(txHeader);
                    }
                };

                enum MainTypeHistoryData
                {
                    HASH = 0,
                    ATOM_HASH,
                    TX_CREATED,
                    NULL_MAIN_DATA
                };

                enum TypeHistoryData
                {
                    TX_TYPE = 0,
                    COINS,
                    TOKEN,
                    ADDRESS,
                    NULL_DATA
                };

                auto getHistoryData = [&](const QJsonObject& itemTx) -> HistoryData
                {
                    HistoryData result;
                    QString direction = itemTx["tx_type"].toString();
                    result.status     = direction;
                    result.direction  = direction == "send" ? "to" : "from";

                    if(direction == "send")
                    {
                        result.value         = itemTx["send_coins"].toString();
                        result.value_datoshi = itemTx["send_datoshi"].toString();
                        result.token         = itemTx["token"].toString();
                        result.address       = itemTx["destination_address"].toString();
                    }
                    else
                    {
                        result.value         = itemTx["recv_coins"].toString();
                        result.value_datoshi = itemTx["recv_datoshi"].toString();
                        result.token         = itemTx["token"].toString();
                        result.address       = itemTx["source_address"].toString();
                    }

                    return result;
                };

                QDateTime date = QDateTime::fromString(txObj["tx_created"].toString(), Qt::RFC2822Date).toLocalTime();
                date.setTimeSpec(Qt::LocalTime);
                qint64 dateToSecs = date.toSecsSinceEpoch();

                txHeader.insert(DATA_KEY,         date.toString("yyyy-MM-dd"));
                txHeader.insert(DATE_TO_SECS_KEY, QString::number(dateToSecs));
                txHeader.insert(TX_STATUS_KEY,    txObj["status"].toString());
                txHeader.insert(TX_HASH_KEY,      txObj["hash"].toString());
                txHeader.insert(ATOM_KEY,         txObj["atom_hash"].toString());
                txHeader.insert(NETWORK_KEY,      net);
                txHeader.insert(WALLET_NAME_KEY, walletName);
                txHeader.insert(VALUE_KEY, "0.0");
                txHeader.insert(TOKEN_KEY, netInfo.tokenName);
                txHeader.insert(FEE_TOKEN_KEY, netInfo.tokenName);
                txHeader.insert(FEE_KEY, "0.0");
                txHeader.insert(FEE_NET_KEY, "0.0");
                txHeader.insert(M_VALUE_KEY, "0.0");

                QString action = txObj["action"].toString();

                QJsonArray txItemsArray = txObj["data"].toArray();
                for(const auto &arrVal: txItemsArray)
                {
                    QJsonObject singeItemTx = arrVal.toObject();
                    HistoryData data = getHistoryData(singeItemTx);
                    if(data.address == REWARD_HISTORY)
                    {
                        if(!dataTransaction.contains(typeHistoryData::REWARD))
                        {
                            dataTransaction.insert(typeHistoryData::REWARD, data);
                        }
                    }
                    else if(data.address == TYPE_COMMISSION)
                    {
                        if(!dataTransaction.contains(typeHistoryData::VALIDATOR))
                        {
                            dataTransaction.insert(typeHistoryData::VALIDATOR, data);
                        }

                        if(action == "use")// vote //todo optimize
                        {
                            if(!dataTransaction.contains(typeHistoryData::VOTE))
                            {
                                dataTransaction.insert(typeHistoryData::VOTE, HistoryData({}));
                            }
                        }
                    }
                    else if(data.address == SRV_XCHANGE)
                    {
                        if(!dataTransaction.contains(typeHistoryData::XCHANGE))
                        {
                            dataTransaction.insert(typeHistoryData::XCHANGE, data);
                        }
                    }
                    else if (data.address == TYPE_M_TRANSACTION)
                    {
                        if(!dataTransaction.contains(typeHistoryData::STATE_LOCK))
                        {

                            dataTransaction.insert(typeHistoryData::STATE_LOCK, data);
                        }
                    }
                    else if (data.address.contains(REGULAR_WALLET_ADDRESS))
                    {
                        // Network commission
                        if(data.address == netInfo.address)
                        {
                            if(!dataTransaction.contains(typeHistoryData::NET))
                            {
                                dataTransaction.insert(typeHistoryData::NET, data);
                            }
                        }
                        else
                        {
                            if(!dataTransaction.contains(typeHistoryData::BASE))
                            {
                                dataTransaction.insert(typeHistoryData::BASE, data);
                            }
                        }
                    }
                    else if (data.address == "null")
                    {
                        if(!dataTransaction.contains(typeHistoryData::NULL_ADDR))
                        {
                            dataTransaction.insert(typeHistoryData::NULL_ADDR, data);
                        }
                    }
                    else if (data.address ==  "emission")
                    {
                        if(!dataTransaction.contains(typeHistoryData::EMISSION))
                        {
                            dataTransaction.insert(typeHistoryData::EMISSION, data);
                        }
                    }
                    else
                    {
                        for(const QString& key: TYPES_SERVICE_TRANSACTION.keys())
                        {
                            if(data.address == key)
                            {
                                if(!dataTransaction.contains(typeHistoryData::SOME_SERVICE_PAY))
                                {
                                    dataTransaction.insert(typeHistoryData::SOME_SERVICE_PAY, data);
                                }
                            }
                        }
                    }
                }
                setData();
            }
        }
    }
    return QString();
}

QString DapGetAllWalletHistoryCommandBase::getMempool(const QString& net, const QString &walletName, QString &walletAddr, const QString &chain,
                                                       NetCommissionInfo &netInfo,
                                                       QSet<QString> &hashSet,
                                                       QJsonArray &txHistory)
{
    QVariant mempoolVariant;
    if(!walletAddr.isEmpty())
        mempoolVariant = cmdList->getListMempool(net, walletAddr, chain);
    else if(!walletName.isEmpty())
        mempoolVariant = cmdList->getListMempool(net, walletName, chain);
    else
        mempoolVariant = cmdList->getListMempool(net, "", "");

    auto mempoolJson = cmdList->getJsonResult(mempoolVariant, "DapGetAllWalletHistoryCommandBase get mempool");

    if(!mempoolJson.result.isArray())
    {
        return QString("The result is not a list.");
    }
    QJsonArray resultArray = mempoolJson.result.toArray();
    if(resultArray.isEmpty())
    {
        return QString("The result is empty..");
    }
    QJsonObject resultObject = resultArray[0].toObject();
    if(!resultObject.contains("chains"))
    {
        return QString("There are no chains in the object.");
    }
    QJsonArray chainsArray = resultObject["chains"].toArray();
    if(chainsArray.isEmpty())
    {
        return QString("No chains found.");
    }
    QJsonObject chainObject;
    for(const auto& itemValue: chainsArray)
    {
        QJsonObject item = itemValue.toObject();
        if(item.contains("name"))
        {
            QString chaineName = item["name"].toString();
            if((chain.isEmpty() && chaineName == "main") || (!chain.isEmpty() && chaineName == chain) )
            {
                chainObject = item;
            }
        }
    }
    if(chainObject.isEmpty())
    {
        return "";
    }
    if(!chainObject.contains("datums"))
    {
        return QString("No datums found.");
    }
    if(chainObject["datums"].isNull())
    {
        return QString();
    }
    QJsonArray datumsArray = chainObject["datums"].toArray();
    if(datumsArray.isEmpty())
    {
        return QString();
    }

   // QStringList stringsMempool = mempoolStr.split("\n");

    bool hasAddressSelf = false;
    bool isAnstake = false;

    QString addressWalletFrom, tx_hash, timeStamp, time;
    QMap<MemoolTypes, QList<MempoolItem>> typesInfo;

    auto getTicker = [](const QString& tiker) -> QString
    {
        return tiker.contains("null") ? "UNKNOWN" : tiker;
    };

    auto setMempoolData = [&]()
    {
        QJsonObject txHeader;

        txHeader.insert(DATA_KEY, time);
        txHeader.insert(DATE_TO_SECS_KEY, timeStamp);
        txHeader.insert(TX_HASH_KEY, tx_hash);
        txHeader.insert(TX_STATUS_KEY, "PROCESSING");
        txHeader.insert(STATUS_KEY, "Pending");
        txHeader.insert(NETWORK_KEY, net);
        txHeader.insert(WALLET_NAME_KEY, walletName);
        txHeader.insert(ATOM_KEY, "");
        txHeader.insert(FEE_KEY, "0.0");
        txHeader.insert(FEE_NET_KEY, "0.0");
        txHeader.insert(TOKEN_KEY, getTicker(netInfo.tokenName));
        txHeader.insert(FEE_TOKEN_KEY, getTicker(netInfo.tokenName));
        txHeader.insert(M_VALUE_KEY, "0.0");
        txHeader.insert(VALUE_KEY, "0.0");

        bool direction = walletAddr == addressWalletFrom;
        txHeader.insert(DIRECTION_KEY, (walletAddr.isEmpty() && walletName.isEmpty()) ? "" :
                                           !direction ? "from" : "to");

        if(typesInfo[MemoolTypes::SECTION_TO].size() == 1 && typesInfo[MemoolTypes::SECTION_TO][0].coins.isEmpty())
        {
            typesInfo[MemoolTypes::SECTION_TO].clear();
        }

        for(const auto& item: typesInfo[MemoolTypes::SECTION_TO])
        {
            if(netInfo.address == item.addr)
            {
                if(direction)
                {
                    txHeader.insert(FEE_NET_KEY, item.coins);
                }
                if(typesInfo[MemoolTypes::SECTION_TO].size() == 1)
                {
                    typesInfo[MemoolTypes::SECTION_TO].clear();
                }

            }
            else
            {
                txHeader.insert(VALUE_KEY, item.coins);
                txHeader.insert(TOKEN_KEY, item.token);

                txHeader.insert(WALLET_NAME_KEY, !direction ? walletName : item.addr);
                txHeader.insert(ADDRESS_KEY, !direction ? item.addr : walletName);
            }
        }

        if(typesInfo[MemoolTypes::SECTION_TO].isEmpty() && typesInfo[MemoolTypes::SECTION_CHANGE].size() > 1)
        {
            QMap<QString, int> testMap;
            for(const auto& item: typesInfo[MemoolTypes::SECTION_CHANGE])
            {
                if(testMap.contains(item.token))
                {
                    testMap[item.token]++;
                }
                else
                {
                    testMap.insert(item.token, 1);
                }
            }
            QString maxKeyStr = "";
            int max = 0;
            for(const QString& key: testMap.keys())
            {
                if(testMap[key] > max)
                {
                    max = testMap[key];
                    maxKeyStr = key;
                }
            }
            txHeader.insert(TOKEN_KEY, maxKeyStr.isEmpty() ? getTicker(netInfo.tokenName) : maxKeyStr);
            txHeader.insert(DIRECTION_KEY, "from");
            txHeader.insert(ADDRESS_KEY, walletName);
        }

        if(!typesInfo[MemoolTypes::SECTION_FEE].isEmpty())
        {
            if(direction)
            {
                const auto& item = typesInfo[MemoolTypes::SECTION_FEE][0];
                txHeader.insert(FEE_KEY, item.coins);
                txHeader.insert(FEE_TOKEN_KEY, item.token);
            }
        }

        if(!typesInfo[MemoolTypes::SECTION_SRV_XCHANGE].isEmpty())
        {
            if(typesInfo[MemoolTypes::SECTION_TO].isEmpty())
            {
                auto& item = typesInfo[MemoolTypes::SECTION_SRV_XCHANGE][0];
                txHeader.insert(VALUE_KEY, item.coins);
                txHeader.insert(TOKEN_KEY, item.token);
                txHeader.insert(DIRECTION_KEY, !direction ? "from" : "to");
            }
        }

        if(!typesInfo[MemoolTypes::SECTION_SRV].isEmpty())
        {
            const auto& item = typesInfo[MemoolTypes::SECTION_SRV][0];
            txHeader.insert(DIRECTION_KEY, "to");
            txHeader.insert(VALUE_KEY, item.coins);
            txHeader.insert(TOKEN_KEY, item.token);
        }

        if(!typesInfo[MemoolTypes::SECTION_SRV_STAKE_LOCK].isEmpty())
        {

        }

        if(!typesInfo[MemoolTypes::SECTION_SRV_STAKE_POS_DELEGATE].isEmpty())
        {
            const auto& item = typesInfo[MemoolTypes::SECTION_SRV_STAKE_POS_DELEGATE][0];
            txHeader.insert(DIRECTION_KEY, "to");
            txHeader.insert(VALUE_KEY, item.coins);
            txHeader.insert(TOKEN_KEY, item.token);
        }        

        if(!typesInfo[MemoolTypes::SECTION_FROM_EMMISSION].isEmpty())
        {
            auto& item = typesInfo[MemoolTypes::SECTION_FROM_EMMISSION][0];
            if(item.token[0] == "m")
            {
                txHeader.insert(M_VALUE_KEY, item.coins);
                txHeader.insert(M_TOKEN, item.token);
                txHeader.insert(M_DIRECTION_KEY, "from");
            }
        }
        txHistory.append(std::move(txHeader));

        hasAddressSelf = false;

        isAnstake = false;
        addressWalletFrom.clear();
        tx_hash.clear();
        timeStamp.clear();
        time.clear();;
        typesInfo.clear();
    };


    auto getValue = [](const QJsonObject& object, const QString& name) -> QString
    {
        if(object.contains(name))
        {
            return object[name].toString();
        }
        return QString();
    };

    auto getTypeInfo = [this, &hasAddressSelf, &walletAddr, &getValue](const QJsonValue& value) -> QList<MempoolItem>
    {
        QList<MempoolItem> result;

        for(const auto& itemValue: value.toArray())
        {
            QJsonObject item = itemValue.toObject();
            MempoolItem itemResult;
            itemResult.addr = getValue(item, ADDR_MEMPOOL);
            if(itemResult.addr == walletAddr)
            {
                hasAddressSelf = true;
            }

            if(item.contains(MONEY_MEMPOOL))
            {
                item = item[MONEY_MEMPOOL].toObject();
            }
            itemResult.value = getValue(item, VALUE_MEMPOOL);
            itemResult.coins = getValue(item, COINS_MEMPOOL);
            itemResult.token = getValue(item, TOKEN_MEMPOOL);
            result.append(itemResult);
        }
        return result;
    };

    for(const auto& itemValue: datumsArray)
    {
        QJsonObject item = itemValue.toObject();
        QString hash = getValue(item, HASH_MEMPOOL);
        if(hashSet.contains(hash))
        {
            mempoolTransactionMoved(net, walletName, hash);
            continue;
        }

        tx_hash = hash;
        QString type = getValue(item, TYPE_MEMPOOL);
        if(type != "DATUM_TX")
        {
            continue;
        }
        if(item.contains(CREATED_MEMPOOL))
        {
            QJsonObject timeObject = item[CREATED_MEMPOOL].toObject();
            if(timeObject.contains(TIME_MEMPOOL))
            {
                qint64 timestamp = timeObject[TIME_MEMPOOL].toInt();
                QDateTime datetime = QDateTime::fromSecsSinceEpoch(timestamp);

                qint64 date_to_secs = datetime.toSecsSinceEpoch();

                timeStamp = QString::number(date_to_secs);
                time = datetime.toString("yyyy-MM-dd");
            }
            else
            {
                qWarning() << "The timestamp is not specified";
            }
        }
        else
        {
            qWarning() << "The time of the transaction creation is not specified.";
        }
        addressWalletFrom = getValue(item, FROM_MEMPOOL);
        if(addressWalletFrom == walletAddr)
        {
            hasAddressSelf = true;
        }
        mempoolTransactionMoved(net, walletName, tx_hash, getValue(item, LADGER_RC_MEMPOOL));

        for(const auto& key: MATCHING_TYPE_MEMPOOL.keys())
        {
            if(item.contains(key))
            {
                typesInfo.insert(MATCHING_TYPE_MEMPOOL[key], getTypeInfo(item[key]));
            }
        }
    }
    if(hasAddressSelf || (walletAddr.isEmpty() && walletName.isEmpty()))
    {
        setMempoolData();
    }
    return QString();
}

void DapGetAllWalletHistoryCommandBase::queueDataUpdate(const QVariant &data)
{
    m_queueObject = data.value<QMap<QString,QJsonArray>>();

    if(!m_argsCash.isNull())
    {
        QVariant result = respondToClient(m_argsCash);
        emit queuedUpdated(result);
    }
}

QJsonArray DapGetAllWalletHistoryCommandBase::getQueueTransactions(const QString& walletName)
{
    if(m_queueObject.contains(walletName))
    {
        return m_queueObject[walletName];
    }
    return QJsonArray();
}

DapGetAllWalletHistoryCommandBase::NetCommissionInfo DapGetAllWalletHistoryCommandBase::parseNetCommissionInfo(const NetworkSpace::FeeInfoStruct& fee)
{
    NetCommissionInfo result;
    result.value = fee.netCoins;
    result.tokenName = fee.netTicker;
    result.address = fee.netAddr;
    return result;
}
DapGetAllWalletHistoryCommandBase::HistoryData::HistoryData(const QString& status, const QString& value, const QString& value_datoshi, const QString& token, const QString& direction, const QString& address)
    : status(status), value(value), value_datoshi(value_datoshi), token(token), direction(direction), address(address)
{}
