#ifndef DAPGETALLWALLETHISTORYCOMMANDBASE_H
#define DAPGETALLWALLETHISTORYCOMMANDBASE_H

#include <QObject>
#include <QProcess>
#include <QRegExp>
#include <QRegularExpression>
#include <QDate>
#include <QJsonArray>
#include <QJsonObject>
#include <QJsonDocument>
#include <QStringList>
#include <QList>

#include "DapAbstractCommand.h"

#include "dap_chain_common.h"

class DapGetAllWalletHistoryCommandBase : public DapAbstractCommand
{
    Q_OBJECT
public:
    using ItemInfo = QMap<QString,QString>;

    struct NetCommissionInfo
    {
        QString address = "";
        QString value = "";
        QString tokenName = "";
    };

    /// Overloaded constructor.
    /// @param asServiceName Service name.
    /// @param parent Parent.
    /// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
    /// @param asCliPath The path to cli nodes.
    DapGetAllWalletHistoryCommandBase(const QString &asServicename, QObject *parent = nullptr, const QString &asCliPath = QString());

    QString getHistory(const QString& net, const QString& walletName, QString &walletAddr,
                       NetCommissionInfo &netInfo,
                       QSet<QString> &hashSet,
                       QJsonArray &txHistory);

    QString getMempool(const QString& net, const QString &walletName, QString& walletAddr, const QString &chain,
                        DapGetAllWalletHistoryCommandBase::NetCommissionInfo& netInfo,
                        QSet<QString>& hashSet,
                        QJsonArray& txHistory);

    QJsonArray getQueueTransactions(const QString& walletName);
    NetCommissionInfo parseNetCommissionInfo(const NetworkSpace::FeeInfoStruct& fee);

signals:
    void queuedUpdated(const QVariant& doc);

public slots:

    void queueDataUpdate(const QVariant &data);

protected:
    virtual void mempoolTransactionMoved(const QString& network, const QString& walletName, const QString& hash, const QString& ladgerTs = "") {}

protected:

    struct MempoolInfo
    {
        QString hash = "";
        QString ts_created = "";
        ItemInfo mainInfo;
        QList<ItemInfo> items;
    };

    struct HistoryData
    {
        HistoryData() = default;
        HistoryData(const QString& status, const QString& value, const QString& value_datoshi, const QString& token, const QString& direction, const QString& address);
        QString status = ""; //tx_type
        QString value = "";
        QString value_datoshi = "";
        QString token = "";
        QString direction = "";
        QString address = "";
    };

    struct MempoolItem
    {
        QString value = QString();
        QString coins = QString();
        QString token = QString();
        QString addr = QString();
    };

    enum MemoolTypes
    {
        SECTION_TO = 0,
        SECTION_CHANGE,
        SECTION_FEE,
        SECTION_SRV,
        SECTION_SRV_XCHANGE,
        SECTION_SRV_STAKE_LOCK,
        SECTION_FROM_EMMISSION,
        SECTION_SRV_STAKE_POS_DELEGATE
    };
    QVariant m_argsCash;

    QMap<QString,QJsonArray> m_queueObject;

    const QRegularExpression REGULAR_WALLET_ADDRESS = QRegularExpression(R"([a-zA-Z0-9]{64})");

    const QString VALUE_MEMPOOL = "value";
    const QString COINS_MEMPOOL = "coins";
    const QString TOKEN_MEMPOOL = "token";
    const QString ADDR_MEMPOOL = "addr";
    const QString MONEY_MEMPOOL = "money";

    const QString LADGER_RC_MEMPOOL = "ledger_rc";
    const QString FROM_MEMPOOL = "from";
    const QString TIME_MEMPOOL = "time_stamp";
    const QString CREATED_MEMPOOL = "created";
    const QString TYPE_MEMPOOL = "type";
    const QString HASH_MEMPOOL = "hash";
    const QString SECTION_TO_MEMPOOL = "to";
    const QString SECTION_CHANGE_MEMPOOL = "change";
    const QString SECTION_FEE_MEMPOOL = "fee";
    const QString SECTION_SRV_MEMPOOL = "srv_pay";
    const QString SECTION_SRV_XCHANGE_MEMPOOL = "srv_xchange";
    const QString SECTION_SRV_STAKE_LOCK_MEMPOOL = "srv_stake_lock";
    const QString SECTION_FROM_EMMISSION_MEMPOOL = "from_emission";
    const QString SECTION_SRV_STAKE_POS_DELEGATE_MEMPOOL = "srv_stake_pos_delegate";

    const QMap<QString, MemoolTypes> MATCHING_TYPE_MEMPOOL = {{SECTION_TO_MEMPOOL, MemoolTypes::SECTION_TO}
                                                             ,{SECTION_CHANGE_MEMPOOL, MemoolTypes::SECTION_CHANGE}
                                                             ,{SECTION_FEE_MEMPOOL, MemoolTypes::SECTION_FEE}
                                                             ,{SECTION_SRV_MEMPOOL, MemoolTypes::SECTION_SRV}
                                                             ,{SECTION_SRV_XCHANGE_MEMPOOL, MemoolTypes::SECTION_SRV_XCHANGE}
                                                             ,{SECTION_SRV_STAKE_LOCK_MEMPOOL, MemoolTypes::SECTION_SRV_STAKE_LOCK}
                                                             ,{SECTION_FROM_EMMISSION_MEMPOOL, MemoolTypes::SECTION_FROM_EMMISSION}
                                                             ,{SECTION_SRV_STAKE_POS_DELEGATE_MEMPOOL, MemoolTypes::SECTION_SRV_STAKE_POS_DELEGATE}};

    const QString TYPE_COMMISSION = "DAP_CHAIN_TX_OUT_COND_SUBTYPE_FEE";
    const QString SRV_XCHANGE = "DAP_CHAIN_TX_OUT_COND_SUBTYPE_SRV_XCHANGE";
    const QString REWARD_HISTORY = "reward collecting";
    const QString TYPE_M_TRANSACTION = "DAP_CHAIN_TX_OUT_COND_SUBTYPE_SRV_STAKE_LOCK";
    const QMap<QString, QString> TYPES_SERVICE_TRANSACTION = {{"DAP_CHAIN_TX_OUT_COND_SUBTYPE_SRV_STAKE_POS_DELEGATE", " Service stake pos delegate"}
                                                              ,{"DAP_CHAIN_TX_OUT_COND_SUBTYPE_SRV_XCHANGE", "Service exchange,"}
                                                              ,{"DAP_CHAIN_TX_OUT_COND_SUBTYPE_SRV_PAY", "Service pay"}};

    const QString DATA_KEY = "date";                    // - date transaction
    const QString DATE_TO_SECS_KEY = "date_to_secs";    // - time transaction
    const QString TX_HASH_KEY = "tx_hash";
    const QString TX_STATUS_KEY = "tx_status";
    const QString STATUS_KEY = "status";
    const QString WALLET_NAME_KEY = "wallet_name";
    const QString ATOM_KEY = "atom";
    const QString FEE_KEY = "fee";                      // - - commission validator
    const QString FEE_TOKEN_KEY = "fee_token";          // - token type for fee
    const QString M_VALUE_KEY = "m_value";
    const QString M_TOKEN = "m_token";
    const QString DIRECTION_KEY = "direction";
    const QString VALUE_KEY = "value";                  // - main value
    const QString TOKEN_KEY = "token";                  // - - token type
    const QString ADDRESS_KEY = "address";
    const QString M_DIRECTION_KEY = "m_direction";
    const QString XCHANGE_DIRECTION_KEY = "x_direction";
    const QString XCHANGE_VALUE_KEY = "x_value";
    const QString XCHANGE_TOKEN_KEY = "x_token";
    const QString FEE_NET_KEY = "fee_net";              // - commission for network
    const QString NETWORK_KEY = "network";

    enum typeHistoryData
    {
        STATE_LOCK = 0,
        VALIDATOR,
        NET,
        EMISSION,
        NULL_ADDR,
        SOME_SERVICE_PAY,
        BASE,
        REWARD,
        FEE_REWARD,
        XCHANGE,
        SRV_PAY,
        VOTE
    };
};

#endif // DAPGETALLWALLETHISTORYCOMMANDBASE_H
