#include "DapGetFeeCommand.h"

DapGetFeeCommand::DapGetFeeCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetFeeCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();
    QString netParam = params.first();
    bool isWeb3 = params.contains(WEB3_KEY);
    QString errorMsg = "";

    QStringList netList;

    if(!netParam.isEmpty())
    {
        if(netParam == "all")
        {
            netList = getListNetworks();
        }
        else
            netList.append(netParam);
    }
    else
    {
        errorMsg = "Error. No have argument Network";
    }

    QJsonObject feeNetworks;

    for(const QString &net : qAsConst(netList))
    {
        auto result = getFee()[net];

        QJsonObject objRes;
        QJsonObject objNet;

        if(isWeb3) objRes.insert("network", net);

        objNet.insert("fee_coins", result.netCoins);
        objNet.insert("fee_datoshi", result.netDatoshi);
        objNet.insert("fee_ticker", result.netTicker);
        objNet.insert("fee_addr", result.netAddr);

        objRes.insert("network_fee", objNet);

        QJsonObject objValidator;

        if(!result.validatorFeeList.contains("min") || !result.validatorFeeList.contains("max")
            || !result.validatorFeeList.contains("average") || !result.validatorFeeList.contains("median"))
        {
            errorMsg = "Error";
            continue;
        }
        const auto& valList = result.validatorFeeList;
        objValidator.insert("min_fee_coins", valList["min"].coin);
        objValidator.insert("min_fee_datoshi", valList["min"].datoshi);
        objValidator.insert("max_fee_coins", valList["max"].coin);
        objValidator.insert("max_fee_datoshi", valList["max"].datoshi);

        QString average_fee_coins = valList["average"].coin;
        objValidator.insert("average_fee_coins", average_fee_coins == "0.0" ? "0.05" : average_fee_coins);
        objValidator.insert("average_fee_datoshi", average_fee_coins == "0.0" ? "50000000000000000" : valList["average"].datoshi);

        QString median_fee_coins = valList["median"].coin;
        objValidator.insert("median_fee_coins", median_fee_coins == "0.0" ? "0.05" : median_fee_coins);
        objValidator.insert("median_fee_datoshi", median_fee_coins == "0.0" ? "50000000000000000" : valList["median"].datoshi);

        objValidator.insert("fee_ticker", result.valTicker);
        objRes.insert("validator_fee", objValidator);

        feeNetworks.insert(net, objRes);

    }

    QJsonObject resultObject;
    QJsonDocument resultDoc;

    if(!feeNetworks.isEmpty() && errorMsg.isEmpty())
    {
        resultObject.insert(RESULT_KEY, feeNetworks);

        if(isWeb3)
            addWeb3Result(resultObject, args);
    }
    else
    {
        if(isWeb3)
        {
            resultObject.insert(ERROR_KEY, errorMsg);
            resultObject.insert(RESULT_KEY, feeNetworks);
            addWeb3Result(resultObject, args);
        }
        else
        {
            resultObject.insert(RESULT_KEY, QString("Error"));
        }
    }

    resultDoc.setObject(resultObject);
    return resultDoc.toJson();
}

void DapGetFeeCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();

    if(replyObj[RESULT_KEY].isObject())
    {
        QJsonObject resultObj = replyObj[RESULT_KEY].toObject();
        QJsonDocument result(resultObj);
        emit serviceResponded(result.toJson());
    }
    else
    {
        QString resultStr = replyObj[RESULT_KEY].toString();
        emit serviceResponded(resultStr);
    }
}
