#include "DapGetHistoryExecutedCmdCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetHistoryExecutedCmdCommand::DapGetHistoryExecutedCmdCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
    QDir().mkpath(QFileInfo(cmdHistory).path());
    m_File = new QFile(cmdHistory, this);
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetHistoryExecutedCmdCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QJsonArray historyExecutedCmd;

    if(!m_File->open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, historyExecutedCmd}}));
        return resultDoc.toJson();
    }

    while (!m_File->atEnd())
    {
        historyExecutedCmd.append(QJsonValue(QString::fromLatin1(m_File->readLine()).trimmed()));
    }
    m_File->close();

    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, historyExecutedCmd}}));
    return resultDoc.toJson();
}

void DapGetHistoryExecutedCmdCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonArray resultArray = replyObj[RESULT_KEY].toArray();

    emit serviceResponded(resultArray);
}
