#ifndef DAPGETHISTORYEXECUTEDCMDCOMMAND_H
#define DAPGETHISTORYEXECUTEDCMDCOMMAND_H

#include <QProcess>
#include <QFile>
#include <QDir>

#include "DapAbstractCommand.h"

#ifdef Q_OS_WIN
#include "registry.h"
#endif

class DapGetHistoryExecutedCmdCommand : public DapAbstractCommand
{
    /// Command history file.
    QFile * m_File {nullptr};

private:

#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
    const QString cmdHistory = QString("/opt/%1/data/cmd_history.txt").arg(QString(DAP_BRAND).toLower());
#elif defined (Q_OS_MACOS)
    const QString cmdHistory = QString("/tmp/cmd_history.txt");
#elif defined (Q_OS_WIN)
    const QString cmdHistory = QString("%1/%2/data/cmd_history.txt").arg(regWGetUsrPath()).arg(DAP_BRAND);
#elif defined (Q_OS_ANDROID)
    static QAndroidJniObject l_pathObj = QtAndroid::androidContext().callObjectMethod(
        "getExternalFilesDir"
        , "(Ljava/lang/String;)Ljava/io/File;"
        , QAndroidJniObject::fromString(QString("")).object());
    const QString cmdHistory = QString("%1/data/cmd_history.txt").arg(l_pathObj.toString());
#else
    const QString cmdHistory = "/tmp/cmd_history.txt";
#endif

public:
    /// Overloaded constructor.
    /// @param asServiceName Service name.
    /// @param parent Parent.
    /// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
    /// @param asCliPath The path to cli nodes.
    DapGetHistoryExecutedCmdCommand(const QString &asServicename, QObject *parent = nullptr, const QString &asCliPath = QString());

public slots:
    /// Send a response to the client.
    /// @details Performed on the service side.
    /// @param arg1...arg10 Parameters.
    /// @return Reply to client.
    QVariant respondToClient(const QVariant &args = QVariant()) override;
    /// Reply from service.
    /// @details Performed on the service side.
    /// @return Service reply.
    void replyFromService() override;
};

#endif // DAPGETHISTORYEXECUTEDCMDCOMMAND_H
