#include "DapGetListKeysCommand.h"

DapGetListKeysCommand::DapGetListKeysCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}


/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetListKeysCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

    auto net = params[0];
    QString errorMsg;
    QJsonObject resultObject;

    QJsonObject data;

    auto requestDoc = cmdList->getListKeysCommand(net);
    auto request = requestDoc.toJsonObject();
    if(!request["errors"].isNull())
    {
        QJsonArray errArr = request["errors"].toArray();
        QJsonObject errObj = errArr[0].toObject()["error"].toObject();
        errorMsg = errObj["message"].toString();
        qDebug() << "[DapGetListKeysCommand] ERROR REQUEST: " << errorMsg;
    }
    else if(!request.contains("result"))
    {
        qWarning() << "[DapGetListKeysCommand][respondToClient] The result block was not found, the node output may have changed.";
        errorMsg = "The result block was not found, the node output may have changed.";
    }
    else
    {
        QJsonArray resultArray;
        if(request.value("result").toArray().isEmpty())
        {
            qWarning() << "[DapGetListKeysCommand][respondToClient] The structure of the command output may have been changed.";
            errorMsg = "The structure of the command output may have been changed.";
        }
        else
        {
            auto resultObj = request.value("result").toArray()[0];
            if (resultObj.isObject())
            {
                auto errArr = resultObj.toObject().value("errors").toArray();
                if (!errArr.isEmpty())
                {
                    errorMsg = errArr[0].toObject().value("message").toString();
                }
                else
                {
                    errorMsg = "not found";
                }
            }
            else
                resultArray = resultObj.toArray();
        }
        QJsonArray keyArray;
        for(const auto& itemValue: resultArray)
        {
            QJsonObject itemObject = itemValue.toObject();

            if(itemObject.contains("total_keys"))
            {
                data.insert("total keys",                        itemObject.value("total_keys").toInt());
                data.insert("total weight coins",                itemObject.value("total_weight_coins").toString());
                data.insert("total weight str",                  itemObject.value("total_weight_str").toString());
                data.insert("total effective weight coins",      itemObject.value("total_effective_weight_coins").toString());
                data.insert("total effective weight",            itemObject.value("total_effective_weight").toString());
                data.insert("key delegating min value",          itemObject.value("key_delegating_min_value").toString());
                data.insert("key delegating min value ticker",   itemObject.value("key_delegating_min_value_ticker").toString());
                data.insert("each validator max related weight", itemObject.value("each_validator_max_related_weight").toString());
            }
            else
            {
                QJsonObject object;
                object.insert("pKey hash",       itemObject.value("pkey_hash").toString());
                object.insert("stake value",     itemObject.value("stake_value").toString());
                object.insert("effective value", itemObject.value("effective_value").toString());
                object.insert("related weight",  itemObject.value("related_weight").toString());
                object.insert("tx hash",         itemObject.value("tx_hash").toString());
                object.insert("node addr",       itemObject.value("node_addr").toString());
                object.insert("sovereign addr",  itemObject.value("sovereign_addr").toString());
                object.insert("sovereign tax",   itemObject.value("sovereign_tax").toString());
                keyArray.append(object);
            }
        }
        data.insert("keys", keyArray);
    }
    data.insert("networkName", net);
    resultObject.insert(RESULT_KEY, data);
    if(!errorMsg.isEmpty())
    {
        resultObject.insert(ERROR_KEY, errorMsg);
    }
    addWeb3Result(resultObject, args);
    QJsonDocument resultDoc(resultObject);
    return resultDoc.toJson();
}
