#include "DapGetListNetworksCommand.h"

#include "dapconfigreader.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetListNetworksCommand::DapGetListNetworksCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetListNetworksCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList list = getListNetworks();

    QJsonArray resultArr;
    for(auto net: list)
    {
        resultArr.append(QJsonValue({net}));
    }
    QJsonDocument resultDoc;
    QJsonObject resultObject;
    resultObject.insert(RESULT_KEY, resultArr);
    addWeb3Result(resultObject, args);
    resultDoc.setObject(resultObject);

    return resultDoc.toJson();
}

void DapGetListNetworksCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonArray resultArr = replyObj[RESULT_KEY].toArray();

    QStringList resultList;
    for(int i = 0; i < resultArr.size(); i++)
    {
        resultList.append(resultArr.at(i).toString());
    }

    emit serviceResponded(resultList);
}
