#include "DapGetListOrdersCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetListOrdersCommand::DapGetListOrdersCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetListOrdersCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();
    QJsonObject objOrders;
    QString errMsg;

    bool isWeb3 = params.contains(WEB3_KEY);
    QStringList listNetworks;

    QStringList results;

    if(isWeb3)
    {
        listNetworks.append(params[1]);  // net
    }
    else
    {
        listNetworks = getListNetworks();
    }

    for(const QString &net : listNetworks)
    {
        QVariant resultJson;
        QJsonArray arrOrders;

        if(isWeb3)
        {
            resultJson = cmdList->getListOrders(args, errMsg);
        }
        else
        {
            resultJson = cmdList->getListOrders(QStringList({"net", net}), errMsg);
        }

        // auto resultJson = cmdList->getXchangeTokenPairsList(net);
        auto resultStruct = cmdList->getJsonResult(resultJson, "DapGetListOrdersCommand");

        auto result = resultStruct.result;
        if(result.isNull())
        {
            qWarning() << "Result is null";
            continue;
        }
        QJsonArray level1 = result.toArray();

        if(level1.isEmpty())
        {
            continue;
        }
        QJsonObject ordersObject = level1[0].toObject();
        if(!ordersObject.contains("orders"))
        {
            qWarning() << "[DapGetListOrdersCommand] orders not found.";
            continue;
        }
        QJsonArray orders = ordersObject.value("orders").toArray();

        for(const auto& itemValue: orders)
        {
            QJsonObject resultItem;
            QJsonObject item = itemValue.toObject();

            if(!item.contains("order") || !item.contains("node_addr"))
            {
                continue;
            }

            QString hash = item.value("order").toString();
            resultItem.insert("hash", hash);
            for(const auto& key: item.keys())
            {
                if(key == "hash")
                {
                    continue;
                }
                if(!isWeb3)
                {
                    if(key == "created")
                    {
                        QString date_str = item.value(key).toString();

                        QDateTime date = QDateTime::fromString(date_str, Qt::RFC2822Date).toLocalTime();
                        date.setTimeSpec(Qt::LocalTime);

                        resultItem.insert("created", date.toString("yyyy-MM-dd hh:mm:ss"));
                        continue;
                    }
                    else if(key == "srv_uid")
                    {
                        QString value = item.value(key).toString();
                        value = value == uids.vpn   ? "VPN"   :
                                    value == uids.stake ? "Stake" : "Other";
                        resultItem.insert(key, value);
                        continue;
                    }
                }
                QJsonValue value = item.value(key);
                if(value.isString())
                {
                    resultItem.insert(key, value.toString());
                }
                else
                {
                    resultItem.insert(key, QString::number(value.toInt()));
                }
            }

            arrOrders.append(resultItem);
        }
        objOrders.insert(net, arrOrders);
    }

    QJsonObject resultObj;
    QJsonDocument resultDoc;

    if(isWeb3)
    {
        if(!errMsg.isEmpty())
            resultObj.insert(ERROR_KEY, errMsg);

        if(objOrders.isEmpty())
            resultObj.insert(ERROR_KEY, results.first());

        resultObj.insert(RESULT_KEY, objOrders[params[1]].toArray());
        addWeb3Result(resultObj, args);
        resultDoc.setObject(resultObj);
    }
    else
    {
        if(!errMsg.isEmpty())
            qWarning() << errMsg;

        resultDoc.setObject(objOrders);
    }

    return resultDoc.toJson();
}
