#include "DapGetListTokensCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetListTokensCommand::DapGetListTokensCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetListTokensCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

    QJsonArray array;
    QStringList netlist = getListNetworks();
    QString tempResult;

    bool update = params[0] == "update";

    for (const QString &net : qAsConst(netlist))
    {
        QJsonObject obj;
        obj.insert("network", net);
//        qDebug()<<net;
        QJsonArray arrTokens;

        auto result = cmdList->getListTokens(net);

        /*
             * 1 - token name
             * 2 - type
             * 3 - flags
             * 4 - supply
             * 5 - decimals
             * 6 - auth signs
             */

        for(const auto& item: result)
        {
            QJsonObject tokenObject;
            tokenObject.insert("name", item.tokenName);
            tokenObject.insert("type", item.type);
            tokenObject.insert("flags", item.flags);

            tokenObject.insert("current_supply", item.supplyCurrent);
            tokenObject.insert("total_supply", item.supplyTotal);
            tokenObject.insert("decimals", item.decimals);
            tokenObject.insert("auth_signs", item.authValid);

            tokenObject.insert("total_emissions", item.totalEmission);
            QJsonObject tsdObj;

            tsdObj.insert("tsd_ticker_from", item.tickerTokenFrom);
            tsdObj.insert("tsd_rate", item.emissionRate);
            tokenObject.insert("tsd", tsdObj);
            QJsonArray signaturesArray;
            for(auto& status: item.statusList)
            {
                QJsonObject signatureObj;

                signatureObj.insert("sign_key"  , status.hash);
                signatureObj.insert("sign_type" , status.signType);
                signatureObj.insert("sign_size" , status.bytes);

                signaturesArray.append(signatureObj);
            }
            tokenObject.insert("signatures", signaturesArray);

            arrTokens.append(tokenObject);
        }

        obj.insert("tokens", arrTokens);
        array.append(obj);
    }

    QJsonDocument resultDoc;
    if (tempResult != lastResult || !update)
    {
        lastResult = tempResult;
        resultDoc.setObject(QJsonObject({{RESULT_KEY, array}}));
    }
    else
    {
        resultDoc.setObject(QJsonObject({{RESULT_KEY, QString("isEqual")}}));
    }

    return resultDoc.toJson();
}

void DapGetListTokensCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();

    if(replyObj[RESULT_KEY].isArray())
    {
        QJsonArray resultArr = replyObj[RESULT_KEY].toArray();
        QJsonDocument resultDoc(resultArr);
        emit serviceResponded(resultDoc.toJson());
    }
    else
    {
        QString resultStr = replyObj[RESULT_KEY].toString();
        emit serviceResponded(resultStr);
    }
}

QString DapGetListTokensCommand::deleteZeros(QString balance)
{
    if(balance == "0") return balance;
    if(balance.length() > 19)
        balance.insert(balance.length()-18,'.');
    else{
        int size = balance.length();
        for(int i = 0; i < 18-size; i++)
            balance.insert(0,'0');
        balance.insert(0,'.');
    }

    while (balance.back() == '0')
        balance.remove(balance.length()-1, 1);
    if (balance.back() == '.')
        balance.remove(balance.length()-1, 1);

    balance.remove( QRegExp("^[0]*") );
    if(balance.at(0) == '.')
        balance.insert(0,'0');

    return balance;
}
