#include "DapGetListWalletsCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetListWalletsCommand::DapGetListWalletsCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetListWalletsCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QJsonDocument walletsDoc;
    QJsonArray walletsArray;
    auto resources = getListWallets();

    for(auto key: resources.keys())
    {
        QJsonObject walletObj;
        walletObj.insert("name", key);
        walletObj.insert("status", resources[key]);
        walletsArray.append(walletObj);
    }

    QJsonObject resultObject;
    resultObject.insert(RESULT_KEY, walletsArray);
    addWeb3Result(resultObject, args);
    walletsDoc.setObject(resultObject);
    return walletsDoc.toJson();
}

void DapGetListWalletsCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonArray resultArr = replyObj[RESULT_KEY].toArray();

    QJsonDocument resultDoc(resultArr);
    emit serviceResponded(resultDoc.toJson());
}
