#include "DapGetNetworkStatusCommand.h"

#include <QRegularExpression>

const QString DapGetNetworkStatusCommand::NAME = "name";
const QString DapGetNetworkStatusCommand::STATE = "state";
const QString DapGetNetworkStatusCommand::TARGET_STATE = "targetState";
const QString DapGetNetworkStatusCommand::ACTIVE_LINKS_COUNT = "activeLinksCount";
const QString DapGetNetworkStatusCommand::LINKS_COUNT = "linksCount";
const QString DapGetNetworkStatusCommand::NODE_ADDRESS = "nodeAddress";

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetNetworkStatusCommand::DapGetNetworkStatusCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetNetworkStatusCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();
    QString errorMsg;

    auto result = cmdList->getNetworkState(params.first(), errorMsg);

    QJsonObject resultObj({
                                {NAME               , result.network},
                                {STATE              , result.currantState},
                                {TARGET_STATE       , result.targetState},
                                {NODE_ADDRESS       , result.currentAddress}
                            });

    resultObj.insert(ACTIVE_LINKS_COUNT, result.activeLinks);
    resultObj.insert(LINKS_COUNT, result.totalLinks);


    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultObj}}));
    return resultDoc.toJson();
}

void DapGetNetworkStatusCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();

    if(replyObj[RESULT_KEY].isObject())
    {
        QJsonObject resultObj = replyObj[RESULT_KEY].toObject();
        emit serviceResponded(resultObj);
    }
    else
    {
        emit serviceResponded({});
    }
}
