#ifndef DAPGETNETWORKSSTATECOMMAND_H
#define DAPGETNETWORKSSTATECOMMAND_H

#include <QProcess>
#include <QRegExp>
#include <QRegularExpression>
#include <QByteArray>
#include <QDataStream>
#include <QBuffer>
#include <QTextCodec>

#include "DapAbstractCommand.h"

class DapGetNetworksStateCommand : public DapAbstractCommand
{
public:
    /// Overloaded constructor.
    /// @param asServiceName Service name.
    /// @param parent Parent.
    /// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
    /// @param asCliPath The path to cli nodes.
    DapGetNetworksStateCommand(const QString &asServicename, QObject *parent = nullptr, const QString &asCliPath = QString());

    static const QMap<QString, QString> s_stateStrings;

public slots:
    /// Send a response to the client.
    /// @details Performed on the service side.
    /// @param arg1...arg10 Parameters.
    /// @return Reply to client.
    QVariant respondToClient(const QVariant &args = QVariant()) override;
    /// Reply from service.
    /// @details Performed on the service side.
    /// @return Service reply.
    void replyFromService() override;

private:
    QString convertState(const QString &state);
};

#endif // DAPGETNETWORKSSTATECOMMAND_H
