#include "DapGetNodeIPCommand.h"

DapGetNodeIPCommand::DapGetNodeIPCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}


/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetNodeIPCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

    auto net = params[0];
    auto nodeAddr = params[1];

    QJsonObject resultObj;

    QJsonArray nodeArray;
    if(!nodeAddr.isEmpty())
    {
        nodeArray.append(nodeAddr);
    }
    else
    {
        QJsonDocument doc = QJsonDocument::fromJson(params[2].toLatin1());
        if(doc.isArray())
        {
            nodeArray = doc.array();
        }
    }

    QString error;
    if(!nodeArray.isEmpty())
    {
        auto result = cmdList->nodeListCommand(net).toString();
        QMap<QString,QString> dumpInfo;
        QRegularExpression regular(
            R"(^(\b(?:[0-9A-Fa-f]{4}::){3}[0-9A-Fa-f]{4})(\s*)(\S*)(\s*)(\S*)(\s*)(.+) )", QRegularExpression::MultilineOption);
        QRegularExpressionMatchIterator matchItr = regular.globalMatch(result);

        if(matchItr.hasNext())
        {
            while (matchItr.hasNext())
            {
                QRegularExpressionMatch match = matchItr.next();
                dumpInfo.insert(match.captured(1), match.captured(3));
            }
        }

        if(dumpInfo.isEmpty())
        {
            error = result;
        }

        if(nodeArray.size() == 1)
        {
            QString nodeAddr = nodeArray[0].toString();
            if(!nodeAddr.contains(QRegularExpression(R"(^([0-9A-Fa-f\:]{22}))")) || nodeAddr.size() != 22)
            {
                error = "Address node error: " + nodeAddr + ".";
            }
            else
            {
                if(dumpInfo.contains(nodeAddr))
                {
                    resultObj.insert(RESULT_KEY, QJsonObject({{nodeAddr, dumpInfo[nodeAddr]}}));
                }
                else
                {
                    resultObj.insert(RESULT_KEY, QJsonObject({{nodeAddr, "unknown ip"}}));
                }
            }
        }
        else
        {
            QJsonArray arrResult;
            for(const auto& value: nodeArray)
            {
                QString item = value.toString();
                if(dumpInfo.contains(item))
                {
                    arrResult.append(QJsonObject({{item, dumpInfo[item]}}));
                }
                else
                {
                    arrResult.append(QJsonObject({{item, "unknown ip"}}));
                }
            }
            resultObj.insert(RESULT_KEY, arrResult);
        }
    }
    else
    {
        error = "No nodes are specified.";
    }

    if(!error.isEmpty())
    {
        resultObj.insert(ERROR_KEY, error);
    }

    addWeb3Result(resultObj, args);
    QJsonDocument resultDoc(resultObj);
    return resultDoc.toJson();
}
