#include "DapGetOnceWalletInfoCommand.h"

DapGetOnceWalletInfoCommand::DapGetOnceWalletInfoCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapGetOnceWalletInfoCommand::respondToClient(const QVariant &args)
{
    QStringList params = args.toStringList();
    if(params.isEmpty() && params.size() < 2)
    {
        qWarning() << "incoming data does not match the required";
        QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, {}}}));
        return resultDoc.toJson();
    }
    QString walletName = params[0];
    QString networkName = params[1];

    auto result = cmdList->getWalletInfo(networkName, walletName);

    QJsonObject resultObj;
    QJsonArray tokenArray;
    for(const auto& token: result.tokenList)
    {
        QJsonObject tokenObj;

        tokenObj.insert("name", token.token );
        tokenObj.insert("coins", token.coins);

        tokenObj.insert("datoshi", token.datoshi);

        tokenArray.append(tokenObj);
    }
    resultObj.insert("tokens", tokenArray);
    resultObj.insert("address", result.address);
    resultObj.insert("walletName", walletName);
    resultObj.insert("networkName", networkName);

    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultObj}}));
    return resultDoc.toJson();
}



void DapGetOnceWalletInfoCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();

    if(replyObj[RESULT_KEY].isObject())
    {
        QJsonObject resultObj = replyObj[RESULT_KEY].toObject();
        QJsonDocument resultDoc(resultObj);
        emit serviceResponded(resultDoc.toJson());
    }
    else if(replyObj[RESULT_KEY].isString())
    {
        QString resultStr = replyObj[RESULT_KEY].toString();
        emit serviceResponded(resultStr);
    }
    else
    {
        emit serviceResponded({});
    }
}
