#include "DapGetWalletAddressesCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
DapGetWalletAddressesCommand::DapGetWalletAddressesCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send request to service.
/// @details Performed on the client side.
/// @param arg1...arg10 Parameters.
void DapGetWalletAddressesCommand::requestToService(const QVariant &args)
{

    DapAbstractCommand::requestToService(args);
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetWalletAddressesCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

    QStringList walletAddress;
    QStringList networkList = params[1].split("");
    if(!networkList.isEmpty())
    {
        for(const QString &net : networkList)
        {
            auto result = cmdList->getWalletInfo(net, params.first());
            walletAddress.append(net);
            walletAddress.append(result.address);
        }
    }

    QJsonArray resultArr;
    for(auto addr: walletAddress)
    {
        resultArr.append(QJsonValue({addr}));
    }
    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultArr}}));
    return resultDoc.toJson();
}

void DapGetWalletAddressesCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonArray resultArr = replyObj[RESULT_KEY].toArray();

    QStringList resultList;
    for(int i = 0; i < resultArr.size(); i++)
    {
        resultList.append(resultArr.at(i).toString());
    }

    emit serviceResponded(resultList);
}

