#ifndef DAPGETWALLETINFOCOMMAND_H
#define DAPGETWALLETINFOCOMMAND_H

#include <QProcess>
#include <QRegExp>
#include <QRegularExpression>
#include <QByteArray>
#include <QDataStream>
#include <QBuffer>
#include <QTextCodec>

#include "DapWallet.h"
#include "CommandType.h"
#include "DapAbstractCommand.h"

class DapGetWalletInfoCommand : public DapAbstractCommand
{
public:
    /// Overloaded constructor.
    /// @param asServiceName Service name.
    /// @param parent Parent.
    /// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
    /// @param asCliPath The path to cli nodes.
    DapGetWalletInfoCommand(const QString &asServicename, QObject *parent = nullptr, const QString &asCliPath = QString());
public slots:
    /// Send a response to the client.
    /// @details Performed on the service side.
    /// @param arg1...arg10 Parameters.
    /// @return Reply to client.
    QVariant respondToClient(const QVariant &args = QVariant()) override;
    /// Reply from service.
    /// @details Performed on the service side.
    /// @return Service reply.
    void replyFromService() override;

    void queueDataUpdate(const QVariant& data);
private:
    bool getAvailableBalance(const QString& walletName, const QString& datoshi, const QString& networkName, const QString& ticker,
                             QString& availableDatoshi, QString& availableCoins);
private:
    QVector<Command::QueueInfoConteiner> m_queueData;
};

#endif // DAPGETWALLETINFOCOMMAND_H
