#include "DapGetWalletTokenInfoCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
DapGetWalletTokenInfoCommand::DapGetWalletTokenInfoCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetWalletTokenInfoCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

    QString walletName = params[0];
    QString net = params[1];

    QStringList token;
    if(!(net.isEmpty() || net.isNull()
         || walletName.isEmpty() || walletName.isNull()))
    {
        auto result = cmdList->getWalletInfo(net, walletName);

        QRegExp regex("wallet: (.+)\\s+addr:\\s+(.+)\\s+(balance)|(\\d+.\\d+)\\s\\((\\d+)\\)\\s(\\w+)");

        for(const auto& item: result.tokenList)
        {
            token.append(item.token);
            token.append(item.coins);
            token.append(item.datoshi);
        }
    }

    QJsonArray resultArr;
    for(auto item: token) resultArr.append(QJsonValue({item}));
    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultArr}}));
    return resultDoc.toJson();

}

void DapGetWalletTokenInfoCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonArray resultArr = replyObj[RESULT_KEY].toArray();

    QStringList resultList;
    for(int i = 0; i < resultArr.size(); i++) resultList.append(resultArr.at(i).toString());

    emit serviceResponded(resultList);
}

