#include <QtDebug>
#include "DapGetWalletsInfoCommand.h"
#include "../DapTypes/DapCoin.h"
#include "dapconfigreader.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapGetWalletsInfoCommand::DapGetWalletsInfoCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetWalletsInfoCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

    bool update = params[0] == "true";

    QJsonArray walletsArray;

    // get list networks
    QStringList netlist = getListNetworks();

    // get list wallets
    auto res = getListWallets();

    for (const auto& key: res.keys())
    {
        QJsonObject walletObj;

        QString walletName = key;
        QString status =res[key];

        if(BACKUP == status) continue;

        walletObj.insert("name", walletName);
        walletObj.insert("status", status);

        if(status == "Active" || status.isEmpty())
        {
            QJsonArray networkArray;

            for (const QString &net: netlist)
            {
                auto result = cmdList->getWalletInfo(net, walletName);


                QJsonArray tokenArray;

                for(const auto& token: result.tokenList)
                {
                    QJsonObject tokenObj;

                    tokenObj.insert("name", token.token );
                    tokenObj.insert("coins", token.coins);

                    tokenObj.insert("datoshi", token.datoshi);
                    tokenObj.insert("availableCoins", "");
                    tokenObj.insert("availableDatoshi", "");
                    QString availableDatoshi, availableCoins;

                    if(getAvailableBalance(walletName, token.datoshi, net,
                                            token.token, availableDatoshi, availableCoins))
                    {
                        tokenObj.insert("availableCoins", availableCoins);
                        tokenObj.insert("availableDatoshi", availableDatoshi);
                    }

                    tokenArray.append(tokenObj);
                }
                QJsonObject networkObj;
                networkObj.insert("tokens", tokenArray);
                networkObj.insert("address", result.address);
                networkObj.insert("name", net);

                networkArray.append(networkObj);
            }

            walletObj.insert("networks", networkArray);
        }
        walletsArray.append(walletObj);
    }

    QJsonDocument docResult;
    docResult.setArray(walletsArray);

    if(buffer != docResult.toJson() || update)
    {
        buffer = docResult.toJson();
        QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, walletsArray}}));
        return resultDoc.toJson();
    } else {
        QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, QString("isEqual")}}));
        return resultDoc.toJson();
    }
}

/// Reply from service.
/// @details Performed on the service side.
/// @return Service reply.
void DapGetWalletsInfoCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();

    if(replyObj[RESULT_KEY].isArray())
    {
        QJsonArray resultArr = replyObj[RESULT_KEY].toArray();
        QJsonDocument resultDoc(resultArr);
        emit serviceResponded(resultDoc.toJson());
    }
    else
    {
        QString resultStr = replyObj[RESULT_KEY].toVariant().toString();
        emit serviceResponded(QVariant(resultStr));
    }
}

void DapGetWalletsInfoCommand::queueDataUpdate(const QVariant &data)
{
    m_queueData = data.value<QVector<Command::QueueInfoConteiner>>();
    QVariant result = respondToClient(QStringList() << "true");
    emit queuedUpdated(result);
}

bool DapGetWalletsInfoCommand::getAvailableBalance(const QString& walletName, const QString& datoshi, const QString& networkName, const QString& ticker,
                                                   QString& availableDatoshi, QString& availableCoins)
{
    auto feeInfo = getFee()[networkName];
    Dap::Coin res = dap_uint256_scan_uninteger(datoshi.toStdString().data());

    bool isHasDatoshi = false;

    for(const auto& item: m_queueData)
    {
        if(item.walletName == walletName && item.networkName == networkName && ticker == feeInfo.valTicker)
        {
            Dap::Coin value = dap_uint256_scan_uninteger(item.fee.toStdString().data());
            res = res - value;
            isHasDatoshi = true;
        }
        if(item.walletName == walletName && item.networkName == networkName && ticker == feeInfo.netTicker)
        {
            Dap::Coin value = feeInfo.netCoins;
            res = res - value;
            isHasDatoshi = true;
        }
        if(walletName == item.walletName && networkName == item.networkName && ticker == item.ticker)
        {
            Dap::Coin value = dap_uint256_scan_uninteger(item.value.toStdString().data());
            res = res - value;
            isHasDatoshi = true;
        }
    }
    if(!isHasDatoshi)
    {
        return false;
    }
    availableDatoshi = res.toDatoshiString();
    availableCoins = res.toCoinsString();
    return true;
}
