#ifndef DAPGETWALLETSINFOCOMMAND_H
#define DAPGETWALLETSINFOCOMMAND_H

#include <QProcess>
#include <QRegExp>
#include <QRegularExpression>
#include <QByteArray>
#include <QDataStream>
#include <QBuffer>
#include <QTextCodec>

#include "DapAbstractCommand.h"
#include "CommandType.h"

class DapGetWalletsInfoCommand : public DapAbstractCommand
{
    Q_OBJECT
public:
    /// Overloaded constructor.
    /// @param asServiceName Service name.
    /// @param parent Parent.
    /// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
    /// @param asCliPath The path to cli nodes.
    DapGetWalletsInfoCommand(const QString &asServicename, QObject *parent = nullptr, const QString &asCliPath = QString());

    QByteArray buffer;
signals:
    void queuedUpdated(const QVariant& doc);
public slots:
    /// Send a response to the client.
    /// @details Performed on the service side.
    /// @param arg1...arg10 Parameters.
    /// @return Reply to client.
    QVariant respondToClient(const QVariant &args = QVariant()) override;
    void replyFromService() override;

    void queueDataUpdate(const QVariant& data);
private:
    bool getAvailableBalance(const QString& walletName, const QString& datoshi, const QString& networkName, const QString& ticker,
                                QString& availableDatoshi, QString& availableCoins);

private:
    const QString BACKUP = "Backup";
    const QString REGULAR_WALLET_ACTIVE = R"(^Wallet: (\S+)( \((\S+)\))?)";

    QVector<Command::QueueInfoConteiner> m_queueData;
};

#endif // DAPGETWALLETSINFOCOMMAND_H
