#include "DapGetXchangeOrdersList.h"
#include <QRandomGenerator>

DapGetXchangeOrdersList::DapGetXchangeOrdersList(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetXchangeOrdersList::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QJsonObject obj;

    // get list networks

    QStringList netlist = getListNetworks();

    for (const QString &net : qAsConst(netlist))
    {
        QJsonArray arrOrders;

        auto resultJson = cmdList->getXchangeOrdersList(net);
        auto resultStruct = cmdList->getJsonResult(resultJson, "DapGetXchangeOrdersList get orders");

        auto result = resultStruct.result;
        if(!result.isArray())
        {
            qWarning() << "[DapGetXchangeTokenPriceHistory] Unknow problem";
            continue;
        }

        for(const auto& itemValue: result.toArray())
        {
            QJsonObject item = itemValue.toObject();
            if(!item.contains("token_buy") || !item.contains("token_sell"))
            {
                continue;
            }
            if(!item.contains("amount_coins") || !item.contains("amount_datoshi") ||
                !item.contains("filled_percent") || !item.contains("net") ||
                !item.contains("order_hash") || !item.contains("rate") ||
                !item.contains("status") || !item.contains("ts_created"))
            {
                qWarning() << "[DapGetXchangeOrdersList] Either the structure or the names of the keys have changed.";
                continue;
            }
            QJsonObject orderObject;
            orderObject.insert("order_hash", item.value("order_hash").toString());
            QDateTime date = QDateTime::fromString(item.value("ts_created").toString(), Qt::RFC2822Date).toLocalTime();
            date.setTimeSpec(Qt::LocalTime);
            orderObject.insert("created", date.toString("yyyy-MM-dd hh:mm:ss")); //D M dd hh:mm:ss YYYY
            orderObject.insert("created_unix", QString::number(date.toSecsSinceEpoch())); //unix
            orderObject.insert("status", item.value("status").toString());
            orderObject.insert("amount", item.value("amount_coins").toString());
            orderObject.insert("amount_datoshi", item.value("amount_datoshi").toString());
            orderObject.insert("amount_token", item.value("token_sell").toString());
            orderObject.insert("sell_token", item.value("token_sell").toString());
            orderObject.insert("buy_token", item.value("token_buy").toString());
            orderObject.insert("filled", QString::number(item.value("filled_percent").toInt()));
            orderObject.insert("rate", item.value("rate").toString());
            orderObject.insert("network", item.value("net").toString());

            arrOrders.append(orderObject);
        }
        obj.insert(net, arrOrders);
    }

    QJsonObject resultObject;
    resultObject.insert(RESULT_KEY, obj);
    addWeb3Result(resultObject, args);

    QJsonDocument resultDoc(resultObject);

    return resultDoc.toJson();
}

void DapGetXchangeOrdersList::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    QJsonDocument resultDoc(resultObj);

    emit serviceResponded(resultDoc.toJson());
}
