#include "DapGetXchangeTokenPair.h"
#include "../DapTypes/DapCoin.h"

DapGetXchangeTokenPair::DapGetXchangeTokenPair(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetXchangeTokenPair::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QJsonArray arrPairs;
    // get list of networks

    QStringList netlist = getListNetworks();

    for (const QString &net : netlist)
    {
        auto resultJson = cmdList->getXchangeTokenPairsList(net);
        auto resultStruct = cmdList->getJsonResult(resultJson, "DapGetXchangeTokenPair get pair list");

        auto result = resultStruct.result;
        if(result.isNull())
        {
            qWarning() << "Result is null";
            continue;
        }
        QJsonArray level1 = result.toArray();

        if(level1.isEmpty())
        {
            continue;
        }

        QJsonObject pairObject = level1[0].toObject();
        if(pairObject.isEmpty())
        {
            continue;
        }

        if(!pairObject.contains("TICKERS PAIR"))
        {
            continue;
        }

        QJsonArray pairArray = pairObject["TICKERS PAIR"].toArray();

        for(const auto& itemValue: pairArray)
        {
            QJsonObject item = itemValue.toObject();
            if(item.count() != 2)
            {
                continue;
            }
            if(!item.contains("ticker_1") || !item.contains("ticker_2"))
            {
                qWarning() << "[DapGetXchangeTokenPair] The keys for the tokens have been changed.";
                continue;
            }
            QJsonObject pair;

            QString token_1 = item.value("ticker_1").toString();
            QString token_2 = item.value("ticker_2").toString();
            pair.insert("token1", token_1);
            pair.insert("token2", token_2);
            pair.insert("network",  net);
            arrPairs.append(pair);
        }
    }

    QJsonObject resultObject;
    resultObject.insert(RESULT_KEY, arrPairs);
    addWeb3Result(resultObject, args);

    QJsonDocument resultDoc(resultObject);
    return resultDoc.toJson();
}

void DapGetXchangeTokenPair::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonArray resultArr = replyObj[RESULT_KEY].toArray();

    QJsonDocument resultDoc(std::move(resultArr));

    emit serviceResponded(resultDoc.toJson());
}
