#include "DapGetXchangeTokenPriceAverage.h"
#include <QRandomGenerator>
#include <QPair>
#include "../DapTypes/DapCoin.h"

DapGetXchangeTokenPriceAverage::DapGetXchangeTokenPriceAverage(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapGetXchangeTokenPriceAverage::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

//    auto getData = [&](const QVariant &args) ->QPair<QString, QString>
//    {
//        QVariant v_result = cmdList->getXchangeAverage(args);

//        if(v_result.type() == QMetaType::QJsonDocument) {
//            qWarning() << "NEED JSON PARSING";
//            return {QString(), QString()};
//        }

//        QString result = v_result.toString();

//        QRegularExpression regex(R"(Last rate: ([0-9\.]+) Last rate time: (.+) \(([0-9]+)\))");
//        QRegularExpressionMatch match = regex.match(result);

//        if(!match.hasMatch())
//        {
//            qWarning() << "The data output from the node has been changed.";
//            return {QString(), QString()};
//        }
//        auto list = match.capturedTexts();
//        QString rate = match.captured(1);
//        QString time = match.captured(3);
//        return {time, rate};
//    };

    auto firstData = cmdList->getXchangeAverage(args);
    auto secondData = cmdList->getXchangeAverage(QStringList() << params[0] << params[2] << params[1]);
    if(firstData.second.isEmpty() ||
       secondData.second.isEmpty())
    {
        return QJsonDocument();
    }

    qint64 firstTime = firstData.first;
    qint64 secondTime = secondData.first;

    QString rate;
    QString time;
    if(firstTime > secondTime)
    {
        rate = firstData.second;
        time = QString::number(firstData.first);
    }
    else
    {
        if(secondData.second == "0.0")
        {
            rate = secondData.second;
        }
        else
        {
            QRegularExpression regex(R"((\d+\.\d+))");
            QRegularExpressionMatch match = regex.match(secondData.second);

            if(!match.hasMatch())
            {
                qWarning() << "Problems with the number : " << secondData.second;
                rate = "1.0";
            }
            else
            {
                Dap::Coin oneVal = QString("1.0");
                Dap::Coin priceVal = secondData.second;
                rate = (oneVal/priceVal).toCoinsString();
            }
        }
        time = QString::number(secondData.first);
    }

    QJsonObject obj;
    obj.insert("token1",params[1]);
    obj.insert("token2",params[2]);
    obj.insert("network",params[0]);
    obj.insert("rate", rate);
    obj.insert("time", time);

    QJsonObject resultObj;
    resultObj.insert(RESULT_KEY,obj);
    addWeb3Result(resultObj, args);

    QJsonDocument resultDoc(resultObj);
    return resultDoc.toJson();
}

void DapGetXchangeTokenPriceAverage::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    QJsonDocument resultDoc(resultObj);

    emit serviceResponded(resultDoc.toJson());
}
