#include "DapGetXchangeTokenPriceHistory.h"
#include <QRandomGenerator>
#include "../DapTypes/DapCoin.h"

DapGetXchangeTokenPriceHistory::DapGetXchangeTokenPriceHistory(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetXchangeTokenPriceHistory::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QJsonArray arrHistory;
    QStringList params = args.toStringList();

    auto updateHistoryList = [&arrHistory, this](const QStringList& list, bool isContrary = false)
    {
                                                                ///   network    token1    token2     nodeMode
        auto resultJson = cmdList->getXchangeHistory(QStringList() << list[0] << list[1] << list[2]);
        auto resultStruct = cmdList->getJsonResult(resultJson, "DapGetXchangeTokenPair get pair list");

        auto result = resultStruct.result;
        if(result.isNull())
        {
            qWarning() << "[DapGetXchangeTokenPriceHistory] Unknow problem";
            return;
        }
        QJsonArray level1 = result.toArray();

        if(level1.isEmpty())
        {
            return;
        }
        if(!level1[0].isArray())
        {
            qWarning() << "[DapGetXchangeTokenPriceHistory] first element is not array";
            return;
        }

        QJsonArray orderArray = level1[0].toArray();

        for(const auto& itemValue: orderArray)
        {
            QJsonObject itemObject = itemValue.toObject();

            if(itemObject.contains("limit") || !itemObject.contains("changed_coins"))
            {
                continue;
            }
            if(!itemObject.contains("hash") || !itemObject.contains("rate")
                || !itemObject.contains("ts_created") )
            {
                qWarning() << "[DapGetXchangeTokenPriceHistory] Either the structure or the names of the keys have changed.";
                continue;
            }

            QString hash = itemObject["hash"].toString();
            QString rate = "";
            QString dateStr = "";
            QDateTime date;
            qint64 dateToSecs = date.toMSecsSinceEpoch();
            QString time = QString::number(dateToSecs);

            dateStr = itemObject["ts_created"].toString();
            date = QDateTime::fromString(dateStr, Qt::RFC2822Date);
            dateToSecs = date.toMSecsSinceEpoch();
            time = QString::number(dateToSecs);

            if(!isContrary)
            {
                rate = itemObject["rate"].toString();
            }
            else
            {
                Dap::Coin oneVal = QString("1.0");
                Dap::Coin priceVal = itemObject["rate"].toString();
                QString result = (oneVal/priceVal).toCoinsString();
                rate = result;
            }

            QJsonObject rateObject;
            rateObject.insert("rate",  rate);
            rateObject.insert("time",  time);
            rateObject.insert("date",  dateStr);
            rateObject.insert("hash",  hash);
            arrHistory.append(std::move(rateObject));
        }
    };

    // get rate history
    updateHistoryList(QStringList() << params[0] << params[1] << params[2]);
    updateHistoryList(QStringList() << params[0] << params[2] << params[1], true);
    
    QJsonObject resultObject;
    resultObject.insert("history", arrHistory);
    resultObject.insert("token1", params[1]);
    resultObject.insert("token2", params[2]);
    resultObject.insert("network", params[0]);

    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultObject}}));
    return resultDoc.toJson();
}

void DapGetXchangeTokenPriceHistory::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    QJsonDocument resultDoc(resultObj);

    emit serviceResponded(resultDoc.toJson());
}
