#include "DapGetXchangeTxList.h"

DapGetXchangeTxList::DapGetXchangeTxList(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapGetXchangeTxList::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    // params[0] = walletName
    // params[1] = timeFrom default empty
    // params[2] = timeTo default empty

    auto params = args.toStringList();
    QStringList netList = getListNetworks();
    
    QJsonArray resultArray;
    for(const auto& net: netList)
    {
        QString walletAddress = cmdList->getWalletAddress(net, params[0]).toString();

        auto resultJson = cmdList->getXchangeList(QStringList() << "GetOrdersPrivate" << net << walletAddress);
        auto resultStruct = cmdList->getJsonResult(resultJson, "DapGetXchangeTxList get list");

        auto result = resultStruct.result;
        if(result.isNull())
        {
            qWarning() << "[DapGetXchangeTxList] The result block is empty, possibly an error.";
            continue;
        }
        QJsonArray level1 = result.toArray();

        if(level1.isEmpty())
        {
            qWarning() << "[DapGetXchangeTxList] The data attachment level has probably been changed.";
            continue;
        }

        QJsonObject resultObject = level1[0].toObject();
        if(resultObject.isEmpty())
        {
            qWarning() << "[DapGetXchangeTxList] An empty object was received";
            continue;
        }

        if(!resultObject.contains("transactions"))
        {
            qWarning() << "[DapGetXchangeTxList] transactions notfound";
            continue;
        }
        QJsonArray transactions = resultObject.value("transactions").toArray();

        for(const auto& itemValue: transactions)
        {
            QJsonObject item = itemValue.toObject();
            if(!item.contains("hash"))
            {
                continue;
            }
            QJsonObject hashObject{{"hash", item.value("hash").toString()}};
            resultArray.append(hashObject);
        }
    }

    QJsonObject resultObject;
    resultObject.insert("walletName", params[0]);
    resultObject.insert("orderList", std::move(resultArray));

    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultObject}}));
    return resultDoc.toJson();
}

void DapGetXchangeTxList::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    QJsonDocument resultDoc(resultObj);

    emit serviceResponded(resultDoc.toJson());
}
