#include "DapLedgerTxHashCommand.h"

DapLedgerTxHashCommand::DapLedgerTxHashCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}


/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapLedgerTxHashCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();

    auto net = params[0];
    auto hash = params[1];

    QString errorMsg;

    auto result = cmdList->getLedgerTxHashCommand(net, hash, errorMsg).toJsonObject();

    QJsonObject resultObj;
    QJsonDocument resultDoc;

    auto getErrorMessageFromData = [](const QJsonObject& resultToCheck){
        const QString errsCode = "errors";
        const QString msgsCode = "message";
        if (resultToCheck.contains(errsCode) && resultToCheck[errsCode].isArray())
        {
            auto arr = resultToCheck[errsCode].toArray();
            if (!arr.isEmpty() && arr[0].isObject())
            {
                auto msgsObj = arr[0].toObject();
                if (msgsObj.contains(msgsCode) && msgsObj[msgsCode].isString())
                {
                    return msgsObj[msgsCode].toString();
                }
            }
        }
        return QString();
    };

    if(errorMsg.isEmpty())
    {
        QString dataErrMsg = getErrorMessageFromData(result);
        if (!dataErrMsg.isEmpty())
        {
            resultObj.insert(ERROR_KEY, dataErrMsg);
        }
        else
        {
            resultObj.insert(RESULT_KEY, result);
        }
    }
    else
    {
        resultObj.insert(ERROR_KEY, errorMsg);
    }

    addWeb3Result(resultObj, args);
    resultDoc.setObject(resultObj);
    return resultDoc.toJson();
}
