#include "DapMempoolListCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapMempoolListCommand::DapMempoolListCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapGetAllWalletHistoryCommandBase(asServicename, parent, asCliPath)
{
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1 Network array. Format "<network name 1>:<chain 1>:<address 1>/<network name 2>:<chain 2>:<address 2>/..."
/// @return Reply to client.
QVariant DapMempoolListCommand::respondToClient(const QVariant &args)
{
    QVariantList params = args.toList();
    qInfo() << "Get mempool list" << params;

    QJsonDocument resultDoc;
    QJsonObject resultObject;
    QJsonArray txHistory;

    auto net = params[0].toString();
    auto addr = params[1].toString();
    auto chain = params[2].toString();

    QSet<QString> hashSet;
    auto netInfo = parseNetCommissionInfo(getFee()[net]);

    QString mempoolResult = getMempool(net, "", addr, chain, netInfo, hashSet, txHistory);

    if(!mempoolResult.isEmpty())
    {
        qWarning() << "Error. " << mempoolResult;
        resultObject.insert(ERROR_KEY, mempoolResult);
    }

    resultObject.insert(RESULT_KEY, txHistory);
    addWeb3Result(resultObject, args);
    resultDoc.setObject(resultObject);

    return resultDoc.toJson();
}
