#include "DapMempoolProcessCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to cli nodes.
DapMempoolProcessCommand::DapMempoolProcessCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1 Network.
/// @param arg2 Chain.
/// @param arg3...arg10 Parameters.
/// @return Reply to client.
QVariant DapMempoolProcessCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    bool resultBool = true;
    QVariant v_result = cmdList->mempoolProcess(args);

    if(v_result.type() == QMetaType::QJsonDocument) {
        // if json
        qWarning() << "NEED JSON PARSING";
        //TODO
        resultBool = false;
    }
    else
    {
        // if string
        QString result = v_result.toString();
        if(result.isEmpty() || result.isNull() || result.contains("No records in mempool") )
        {
            resultBool = false;
        }
    }

    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultBool}}));
    return resultDoc.toJson();
}

void DapMempoolProcessCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    bool resultBool = replyObj[RESULT_KEY].toBool();

    emit serviceResponded(resultBool);
}
