#include "DapMoveWalletCommand.h"
#include <QDir>
#include <QFile>
#include <QFileInfo>

#if defined (Q_OS_MACOS)
#include "dap_common.h"
#endif

#ifdef Q_OS_WIN
#include "registry.h"
#endif

DapMoveWalletCommand::DapMoveWalletCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapMoveWalletCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    auto params = args.toStringList();
    QString walletName = params[0];
    QString currWalletAddr = params[1];
    auto walletAddr = getWalletsDir();

    auto moveFile = [&](const QString& sourceFilePath, const QString& destinationDirPath) -> bool
    {
        auto list = sourceFilePath.split('/');
        QString destPath = destinationDirPath + "/" + list.back();
        QFile destFile(destPath);
        if(destFile.exists())
        {
            qDebug() << "[DapMoveWalletCommand] The file already exists.";
            return true;
        }

        QFile sourceFile(sourceFilePath);

        if (sourceFile.exists())
        {
            QDir destinationDir(destinationDirPath);

            if (QFile::copy(sourceFilePath, destinationDirPath + "/" + list.back())) {
                qDebug() << "[DapMoveWalletCommand] Wallet copyed.";
                return true;
            } else {
                qDebug() << "[DapMoveWalletCommand] Wallet not copyed.";
                return false;
            }
        }
        else
        {
            qDebug() << "[DapMoveWalletCommand] File not found.";
            return false;
        }
        return false;
    };

    QJsonObject resulObject, errorObject;
    if(!currWalletAddr.contains(walletAddr))
    {
        if(moveFile(currWalletAddr, walletAddr))
        {
            resulObject.insert("success", true);
        }
        else
        {
            resulObject.insert("success", false);
            qDebug() << "The wallet could not be transferred";
            errorObject.insert("code", 0);
        }
    }
    else
    {
        qDebug() << "The wallet could not be transferred";
        errorObject.insert("code", 1);
        resulObject.insert("success", false);
    }
    QJsonObject result;
    if(!errorObject.isEmpty())
    {
        result.insert(ERROR_KEY, errorObject);
    }
    result.insert(RESULT_KEY, resulObject);
    QJsonDocument document;
    document.setObject(result);
    return document.toJson();
}

QString DapMoveWalletCommand::getWalletsDir() 
{
#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
    return QString("/opt/%1-node/var/lib/wallet/").arg(DAP_BRAND_BASE_LO);
#elif defined (Q_OS_MACOS)
    char * l_username = NULL;
    exec_with_ret(&l_username,"whoami|tr -d '\n'");
    if (!l_username)
    {
        qWarning() << "Fatal Error: Can't obtain username";
        return "";
    }
    else
    {
        QDir dirOld(QString("/Users/%1/Applications/Cellframe.app/Contents/Resources/var/lib/wallet/").arg(l_username));
        QDir dirNew("/Applications/CellframeNode.app/Contents/Resources/var/lib/wallet/");

        if(dirNew.exists())
            return QString(dirNew.absolutePath() + QDir::separator());
        else
            return QString(dirOld.absolutePath() + QDir::separator());
    }
#elif defined (Q_OS_WIN)
    return QString("%1\\cellframe-node\\var\\lib\\wallet\\").arg(regWGetUsrPath());
#elif defined Q_OS_ANDROID
    return QString("/sdcard/cellframe-node/var/lib/wallet/");
#endif
}
