#include "DapNetIdCommand.h"


DapNetIdCommand::DapNetIdCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}


/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapNetIdCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList params = args.toStringList();

    QString net = params[0];

    auto result = cmdList->getNetworkIdCommand(net).toString();

    auto list = result.split('\n');
    auto getValue = [](const QString& str) ->QString
    {
        auto value = str.trimmed().split(": ");
        if(value.size() != 2)
        {
            qWarning() << "The data output format has changed";
            return QString();
        }
        return value[1].trimmed();
    };
    QString network, id;
    for(const auto& str: list)
    {
        if(str.contains("network"))
        {
            network = getValue(str);
        }
        else if(str.contains("id"))
        {
            id = getValue(str);
        }
    }

    QJsonDocument resultDoc;
    QJsonObject resultObj;

    if(network.isEmpty() || id.isEmpty())
    {
        resultObj.insert(ERROR_KEY, result);
    }
    else
    {
        QJsonObject data;
        data.insert("network", network);
        data.insert("id", id);
        resultObj.insert(RESULT_KEY, data);
    }
    addWeb3Result(resultObj, args);
    resultDoc.setObject(resultObj);
    return resultDoc.toJson();
}

void DapNetIdCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}
