#include "DapNetworkGoToCommand.h"

#include <QRegularExpression>

const QString DapNetworkGoToCommand::NETWORK_NAME = "name";
const QString DapNetworkGoToCommand::STATE = "state";
const QString DapNetworkGoToCommand::TARGET_STATE = "targetState";

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
DapNetworkGoToCommand::DapNetworkGoToCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapNetworkGoToCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QString result = cmdList->networkGoTo(args).toString();

    auto list = result.split('\n');

    auto getValue = [](const QString& str) ->QString
    {
        auto value = str.trimmed().split(": ");
        if(value.size() != 2)
        {
            qWarning() << "The data output format has changed";
            return QString();
        }
        return value[1].trimmed();
    };

    QString network, current, start;

    for(auto& str: list)
    {
        if(str.contains("net:"))
        {
            network = getValue(str);
        }
        else if(str.contains("current:"))
        {
            current = getValue(str);
        }
        else if(str.contains("start:"))
        {
            start = getValue(str);
        }
    }

    if(network.isEmpty() || current.isEmpty() || start.isEmpty())
    {
        QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, {}}});
        return resultDoc.toJson();
    }

    QJsonObject resultObj{
        {NETWORK_NAME, network},
        {STATE       , current},
        {TARGET_STATE, start}
    };
    QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultObj}});
    return resultDoc.toJson();
}

void DapNetworkGoToCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();

    if(replyObj[RESULT_KEY].isObject())
    {
        QJsonObject resultObj = replyObj[RESULT_KEY].toObject();
        emit serviceResponded(resultObj);
    }
    else
    {
        emit serviceResponded({});
    }
}
