#include "DapNetworkSingleSyncCommand.h"

#include <QRegularExpression>

const QString DapNetworkSingleSyncCommand::NETWORK_NAME = "networkName";
const QString DapNetworkSingleSyncCommand::SUCCESS = "success";
const QString DapNetworkSingleSyncCommand::STATE = "state";

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
DapNetworkSingleSyncCommand::DapNetworkSingleSyncCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapNetworkSingleSyncCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList params = args.toStringList();

    QString network = params.first();

    QVariant v_result = cmdList->networkSingleSync(network);

    if(v_result.type() == QMetaType::QJsonDocument) {
        // if json
        qWarning() << "NEED JSON PARSING";
        //TODO

        QJsonObject resultObj{
            {NETWORK_NAME, network},
            {SUCCESS     , false}
        };
        QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultObj}}));
        return resultDoc.toJson();
    }
    else
    {
        // if string
        QString result = v_result.toString();

        QRegularExpression rx(R"(^SYNC_ALL state requested to state machine\. Current state: (\S+))");
        QRegularExpressionMatch match = rx.match(result);

        if (!match.hasMatch()) {
            QJsonObject resultObj{
                {NETWORK_NAME, network},
                {SUCCESS     , false}
            };
            QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultObj}}));
            return resultDoc.toJson();
        }
        else
        {
            QJsonObject resultObj{
                                  {NETWORK_NAME, network},
                                  {SUCCESS     , true},
                                  {STATE       , match.captured(1)},
                                  };
            QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, resultObj}}));
            return resultDoc.toJson();
        }
    }
}

void DapNetworkSingleSyncCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}
