#include "DapNodeConfigController.h"
#include <dapconfigreader.h>
#include <QSettings>
#include <QFile>

DapNodeConfigController::DapNodeConfigController(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

QVariant DapNodeConfigController::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList params = args.toStringList();

    qDebug() << "Params: " << params[0] << params[1];
    DapConfigReader reader;
    if (params[0] == "AddNewValue")
    {
        QString config_path = reader.getConfigPath() + "/cellframe-node.cfg";

        QFile configFile(config_path);
        if (!configFile.open(QIODevice::ReadWrite | QIODevice::Text))
        {
            qDebug() << "Config file not open";
        }
        else
        {
            QString data = configFile.readAll();
            int pos = data.indexOf("auto_online");
            QString newData;

            if (data[pos + 12] == 't')
            {
                data = data.remove(pos, 16);
                newData = "auto_online=false";
            }
            else
            {
                data = data.remove(pos, 17);
                newData = "auto_online=true";
            }

            data.insert(pos, newData);
            configFile.resize(0);

            configFile.write(data.toLatin1());

            configFile.close();

            // restart node
            cmdList->restartNode();
        }

        QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, QString("")}}));
        return resultDoc.toJson();
    }
    QJsonDocument resultDoc(QJsonObject({{RESULT_KEY, QString("Invalid Parameters")}}));
    return resultDoc.toJson();
}

void DapNodeConfigController::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QString resultStr = replyObj[RESULT_KEY].toString();

    emit serviceResponded(resultStr);
}
