#include "DapNodeListCommand.h"

DapNodeListCommand::DapNodeListCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}


/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapNodeListCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QStringList params = args.toStringList();
    auto net = params[0];

    auto result = cmdList->nodeListCommand(net).toString();

    QJsonObject resultObj;

    QRegularExpression regular(
        R"(^(\b(?:[0-9A-Fa-f]{4}::){3}[0-9A-Fa-f]{4})(\s*)(\S*)(\s*)(\S*)(\s*)(.+) )", QRegularExpression::MultilineOption);
    QRegularExpressionMatchIterator matchItr = regular.globalMatch(result);

    if(matchItr.hasNext())
    {
        QJsonArray arr;

        while (matchItr.hasNext())
        {
            QRegularExpressionMatch match = matchItr.next();
            auto list = match.capturedTexts();
            QJsonObject obj;

            obj.insert("node address", match.captured(1));
            obj.insert("ipv4", match.captured(3));
            obj.insert("port", match.captured(5));
            obj.insert("date", match.captured(7));

            arr.append(obj);
        }

        resultObj.insert(RESULT_KEY, arr);
    }
    else
    {
        resultObj.insert(ERROR_KEY, result);
    }

    addWeb3Result(resultObj, args);
    QJsonDocument resultDoc(resultObj);
    return resultDoc.toJson();
}
