#include "DapOrderCreateStakerCommand.h"

DapOrderCreateStakerCommand::DapOrderCreateStakerCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

QVariant DapOrderCreateStakerCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QString errorMsg;
    QStringList params = args.toStringList();
    if(!params.contains("feeAdded"))
    {
        QString fee = "50000000000000000";
        auto docFee = cmdList->getFee(params[0]);
        if(!docFee.validatorFeeList["median"].datoshi.isEmpty())
            fee = docFee.validatorFeeList["median"].datoshi;
        params.insert(4, fee);
    }

    QJsonObject transactionObj, resultObject;

    bool isWalletBlocked = false;
    QString walletName = params[1];
    if(!walletName.isEmpty())
    {
        auto wallets = getListWallets();
        if(wallets.contains(walletName) && wallets[walletName] == WALLET_NON_ACTIVE_KEY)
        {
            resultObject.insert(ERROR_KEY, "The wallet is blocked.");
            transactionObj.insert(SUCCESS,QJsonValue(false));
            transactionObj.insert(MESSAGE,QJsonValue(""));
            isWalletBlocked = true;
        }
    }

    if(!isWalletBlocked)
    {
        auto returnError = [this, &args](const QJsonObject& object) -> QVariant
        {
            QJsonObject errorObject;
            errorObject.insert(ERROR_KEY, object);
            QJsonObject transactionObj;
            transactionObj.insert(SUCCESS,QJsonValue(false));
            transactionObj.insert(MESSAGE,QJsonValue(""));
            errorObject.insert(RESULT_KEY, transactionObj);
            addWeb3Result(errorObject, args);

            QJsonDocument resultDoc;
            resultDoc.setObject(errorObject);
            return resultDoc.toJson();
        };
        auto requestDoc = cmdList->createOrderStaker(params);
        auto request = requestDoc.toJsonObject();
        if(!request["errors"].isNull())
        {
            QJsonArray errArr = request["errors"].toArray();
            QJsonObject errObj = errArr[0].toObject()["error"].toObject();
            return returnError({{"message", errObj["message"].toString()}});
        }
        else
        {
            QJsonArray result;
            bool isSuccess = false;
            QString hash;
            QString orderHash;
            if(request.contains("result"))
            {
                result = request["result"].toArray();

            }
            if(result.isEmpty())
            {
                return returnError({{"message", "It is possible to change the output of the node. there is no result."}});
            }
            else
            {
                QJsonObject firstValue = result[0].toObject();
                for(const QString& itemKey: firstValue.keys())
                {
                    if(itemKey == "status" && firstValue.value(itemKey).toString() == "success")
                    {
                        isSuccess = true;
                        hash = firstValue.value("tx_hash").toString();
                        orderHash = firstValue.value("order_hash").toString();
                    }
                    if(itemKey == "errors")
                    {
                        auto errorArray = firstValue.value(itemKey).toArray();
                        if(errorArray.isEmpty())
                        {
                            return returnError({{"message", "Error. The structure of the json output for displaying the error has changed."}});
                        }
                        auto errorObject = errorArray.first().toObject();
                        if(!errorObject.contains("message") && !errorObject.contains("code"))
                        {
                            return returnError({{"message", "Error. The message or/and error code items were not found."}});
                        }

                        return returnError({{"message", errorObject.value("message")}
                                            ,{"code", errorObject.value("code")}});
                    }
                }
            }

            transactionObj.insert(SUCCESS,QJsonValue(isSuccess));
            if(isSuccess)
            {
                transactionObj.insert(HASH, QJsonValue(hash));
                transactionObj.insert(HASH_ORDER, QJsonValue(orderHash));
            }
            else
            {
                transactionObj.insert(MESSAGE,QJsonValue(result));
            }
        }
    }

    if(!errorMsg.isEmpty())
    {
        resultObject.insert(ERROR_KEY, errorMsg);
    }
    addWeb3Result(resultObject, args);
    resultObject.insert(RESULT_KEY, transactionObj);
    QJsonDocument resultDoc;
    resultDoc.setObject(resultObject);
    return resultDoc.toJson();
}
