#include "DapQuitApplicationCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
DapQuitApplicationCommand::DapQuitApplicationCommand(const QString &asServicename, QObject *parent)
    : DapAbstractCommand(asServicename, parent)
{

}

/// Send a notification to the client. At the same time, you should not expect a response from the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
void DapQuitApplicationCommand::notifyToClient(const QVariant &args)
{
    DapAbstractCommand::notifyToClient(args);

    DapRpcLocalServer * server = dynamic_cast<DapRpcLocalServer *>(m_parent);

    Q_ASSERT(server);

    connect(server, SIGNAL(onClientDisconnected()), qApp, SLOT(quit()));
}

/// Process the notification from the service on the client side.
/// @details Performed on the client side.
/// @param arg1...arg10 Parameters.
void DapQuitApplicationCommand::notifedFromService(const QVariant &args)
{
    Q_UNUSED(args)

    qApp->quit();
}
