#include "DapRemoveWalletCommand.h"
#include <QFile>
#include <QDir>
#include <QRegExp>
#include <QRegularExpression>

#if defined (Q_OS_MACOS)
#include "dap_common.h"
#endif

#ifdef Q_OS_WIN
#include "registry.h"
#endif

const QString DapRemoveWalletCommand::SUCCESS = "success";
const QString DapRemoveWalletCommand::MESSAGE = "message";

QString DapRemoveWalletCommand::getWalletsDir() {
#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
    return QString("/opt/%1-node/var/lib/wallet/").arg(DAP_BRAND_BASE_LO);
#elif defined (Q_OS_MACOS)
    char * l_username = NULL;
    exec_with_ret(&l_username,"whoami|tr -d '\n'");
    if (!l_username)
    {
        qWarning() << "Fatal Error: Can't obtain username";
        return "";
    }
    else
    {
        QDir dirOld(QString("/Users/%1/Applications/Cellframe.app/Contents/Resources/var/lib/wallet/").arg(l_username));
        QDir dirNew("/Applications/CellframeNode.app/Contents/Resources/var/lib/wallet/");
        QDir dirNewNew("/Library/Application\ Support/CellframeNode/var/lib/wallet/");

        if(dirNewNew.exists())
            return QString(dirNewNew.absolutePath() + QDir::separator());
        else if(dirNew.exists())
            return QString(dirNew.absolutePath() + QDir::separator());
        else
            return QString(dirOld.absolutePath() + QDir::separator());
    }
#elif defined (Q_OS_WIN)
    return QString("%1\\cellframe-node\\var\\lib\\wallet\\").arg(regWGetUsrPath());
#elif defined Q_OS_ANDROID
    return QString("/sdcard/cellframe-node/var/lib/wallet/");
#endif
}

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
DapRemoveWalletCommand::DapRemoveWalletCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
    WALLETS_PATH = getWalletsDir();
}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapRemoveWalletCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList filesForRemove;
    QJsonObject resultObj;

    QStringList params = args.toStringList();
    QString walletName = params[0];

    // check directory
    QDir directory(WALLETS_PATH);
    if (!directory.exists()) {
        qWarning() << "Directory not found:" << WALLETS_PATH;

        resultObj.insert(SUCCESS,QJsonValue(false));
        resultObj.insert(MESSAGE,QJsonValue("0"));

        //return resultObj;

        QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultObj}});
        return resultDoc.toJson();
    }
    QStringList files = directory.entryList(QDir::Files);

    // add wallet
    filesForRemove.append(QString("%1%2.dwallet").arg(WALLETS_PATH).arg(walletName));

    // find backups
    QRegularExpression regex;
    regex.setPattern(QString("^%1_\\d*\\.backup$").arg(walletName));
    for(auto fileName: files) {
        QRegularExpressionMatch match = regex.match(fileName);
        if(match.hasMatch()) {
            filesForRemove.append(QString("%1%2").arg(WALLETS_PATH).arg(fileName));
        }
    }

    // remove all wallets
    int success = 0;
    for(auto filePath: filesForRemove) {
        QFile file(filePath);
        if(file.remove()) {
            ++success;
        } else {
            qWarning() << "Error. Can't remove file " << filePath << file.errorString();
        }
    }

    auto result = QString("%1/%2").arg(success).arg(filesForRemove.count());
    resultObj.insert(MESSAGE,QJsonValue(result));

    if(success < filesForRemove.count()) {
        resultObj.insert(SUCCESS,QJsonValue(false));
    } else {
        resultObj.insert(SUCCESS,QJsonValue(true));
    }

    QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultObj}});
    return resultDoc.toJson();
}

void DapRemoveWalletCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}

