#include "DapSaveHistoryExecutedCmdCommand.h"
#include <QDateTime>

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
/// @param asCliPath The path to history file.
DapSaveHistoryExecutedCmdCommand::DapSaveHistoryExecutedCmdCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
    QDir().mkpath(QFileInfo(cmdHistory).path());
    m_File = new QFile(cmdHistory, this);
}

/// Process the notification from the client on the service side.
/// @details Performed on the service side.
/// @param arg1 Command.
/// @param arg2...arg10 Parameters.
void DapSaveHistoryExecutedCmdCommand::notifedFromClient(const QVariant &args)
{
    QJsonDocument doc = QJsonDocument::fromJson(args.toString().toUtf8());
    QJsonArray arr = doc.array();

    if (!m_File->open(QIODevice::Append | QIODevice::ReadWrite))
        return;

    QDateTime currentDateTime = QDateTime::currentDateTime();
    QString formattedDateTime = currentDateTime.toString("dd.MM.yy - hh:mm:ss");

    m_File->write('[' + formattedDateTime.toUtf8() + "] " + arr.at(0).toString().toUtf8() + '\n');
    m_File->close();
}
