#ifndef DAPSAVEHISTORYEXECUTEDCMDCOMMAND_H
#define DAPSAVEHISTORYEXECUTEDCMDCOMMAND_H

#include <QFile>
#include <QDir>

#include "DapAbstractCommand.h"

#ifdef Q_OS_WIN
#include "registry.h"
#endif

class DapSaveHistoryExecutedCmdCommand : public DapAbstractCommand
{
    /// Command history file.
    QFile * m_File{nullptr};

private:

#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
    const QString cmdHistory = QString("/opt/%1/data/cmd_history.txt").arg(QString(DAP_BRAND).toLower());
#elif defined (Q_OS_MACOS)
    const QString cmdHistory = QString("/tmp/cmd_history.txt");
#elif defined (Q_OS_WIN)
    const QString cmdHistory = QString("%1/%2/data/cmd_history.txt").arg(regWGetUsrPath()).arg(DAP_BRAND);
#elif defined (Q_OS_ANDROID)
    static QAndroidJniObject l_pathObj = QtAndroid::androidContext().callObjectMethod(
        "getExternalFilesDir"
        , "(Ljava/lang/String;)Ljava/io/File;"
        , QAndroidJniObject::fromString(QString("")).object());
    const QString cmdHistory = QString("%1/data/cmd_history.txt").arg(l_pathObj.toString());
#else
    const QString cmdHistory = "/tmp/cmd_history.txt";
#endif

public:
    /// Overloaded constructor.
    /// @param asServiceName Service name.
    /// @param parent Parent.
    /// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
    /// @param asCliPath The path to history file.
    DapSaveHistoryExecutedCmdCommand(const QString &asServicename, QObject *parent = nullptr, const QString &asCliPath = QString());

public slots:
    /// Process the notification from the client on the service side.
    /// @details Performed on the service side.
    /// @param arg1 Command.
    /// @param arg2...arg10 Parameters.
    void notifedFromClient(const QVariant &args = QVariant()) override;
};

#endif // DAPSAVEHISTORYEXECUTEDCMDCOMMAND_H
