#include "DapTXCondCreateCommand.h"

DapTXCondCreateCommand::DapTXCondCreateCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

QVariant DapTXCondCreateCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QJsonObject resultObj;

    auto request = cmdList->getJsonResult(cmdList->txCondCreate(args), "condTransaction");

    auto getErrorResult = [this](const QJsonObject& errorObject) -> QVariant
    {
        QJsonObject mainObj;
        mainObj.insert(ERROR_KEY, errorObject);
        QJsonDocument resultDoc(mainObj);
        return resultDoc.toJson();
    };

    if(!request.result.isNull())
    {
        auto arrayObject = request.result.toArray();
        if(arrayObject.isEmpty())
        {
            qWarning() << "[DapTXCondCreateCommand] The node output has been changed. The first list is empty.";
            return getErrorResult(QJsonObject({{"message", "The node output has been changed."}}));
        }

        auto firstValue = arrayObject[0].toObject();
        if(firstValue.contains("errors"))
        {
            QJsonArray errorArray = firstValue.value("errors").toArray();
            if(errorArray.isEmpty())
            {
                qWarning() << "[DapTXCondCreateCommand] The node output has been changed. The error list is empty.";
                return getErrorResult(QJsonObject({{"message", "The node output has been changed."}}));
            }
            QJsonObject errorObject = errorArray[0].toObject();
            if(errorObject.isEmpty())
            {
                qWarning() << "[DapTXCondCreateCommand] The node output has been changed. The error object is empty.";
                return getErrorResult(QJsonObject({{"message", "The node output has been changed."}}));
            }
            return getErrorResult(errorObject);
        }
        else if(!firstValue.contains("create_tx_cond"))
        {
            QString message = QString("No keyword found %1").arg("create_tx_cond");
            qWarning() << "[DapTXCondCreateCommand] The node output has been changed. " << message;
            return getErrorResult(QJsonObject({{"message", "The node output has been changed."}}));
        }

        bool isCond = firstValue["create_tx_cond"].toBool();
        if(isCond)
        {
            resultObj.insert(SUCCESS,QJsonValue(true));
            resultObj.insert(MESSAGE,QJsonValue(request.result.toString().trimmed()));
            resultObj.insert(HASH, QJsonValue(firstValue["hash"].toString()));
        }
        else
        {
            resultObj.insert(SUCCESS,QJsonValue(false));
            resultObj.insert(MESSAGE,QJsonValue(request.result.toString().trimmed()));
        }
        QJsonObject mainObj;
        mainObj.insert(RESULT_KEY, resultObj);
        QJsonDocument resultDoc(mainObj);
        return resultDoc.toJson();
    }

    qWarning() << "[DapTXCondCreateCommand] Unforeseen difficulties have arisen. ";
    return getErrorResult(QJsonObject({{"message", "The node output has been changed."}}));
}

