#include "DapTokenEmissionCommand.h"

const QString DapTokenEmissionCommand::SUCCESS = "success";
const QString DapTokenEmissionCommand::MESSAGE = "message";

DapTokenEmissionCommand::DapTokenEmissionCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}


/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant DapTokenEmissionCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QJsonObject resultObj;

    QVariant v_result = cmdList->tokenEmission(args);

    if(v_result.type() == QMetaType::QJsonDocument)
    {
        // if json
        qWarning() << "NEED JSON PARSING";
        //TODO
    }
    else
    {
        // if string
        QString result = v_result.toString();
        if(result.contains("is placed in datum pool"))
        {
            resultObj.insert(SUCCESS,QJsonValue(true));
            resultObj.insert(MESSAGE,QJsonValue("Placed to mempool"));
        }
        else
        {
            resultObj.insert(SUCCESS,QJsonValue(false));
            resultObj.insert(MESSAGE,QJsonValue(result));
        }
    }

    QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultObj}});
    return resultDoc.toJson();
}

void DapTokenEmissionCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}
