#include "DapTransactionsInfoQueueCommand.h"
#include "TransactionQueue/DapTransactionQueueController.h"

DapTransactionsInfoQueueCommand::DapTransactionsInfoQueueCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

QVariant DapTransactionsInfoQueueCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    DapTransactionQueueController* controller = DapTransactionQueueController::getTransactionController();


    auto params = args.toStringList();
    QString error = "";
    auto getParams = [](const QStringList& list, const QString& paramName, QString& param) ->bool
    {
        if(list.contains(paramName))
        {
            int index = list.indexOf(paramName);
            if(list.size() < index + 2)
            {
                qWarning() << "A parameter may have been omitted: " << paramName;
                param.clear();
                return false;
            }
            if(!list[index+1].isEmpty())
            {
                param = list[index+1];
                return true;
            }
        }
        return false;
    };

    QString idQueue = "";
    getParams(params, "idQueue", idQueue);
    QString network = "";
    getParams(params, "network", network);
    QString wallet = "";
    getParams(params, "wallet", wallet);

    QJsonObject resultObject;

    if(network.isEmpty() || wallet.isEmpty() || idQueue.isEmpty())
    {
        if(network.isEmpty()) error = "The network parameter is not specified.";
        if(wallet.isEmpty()) error = "The wallet parameter is not specified.";
        if(idQueue.isEmpty()) error = "The idQueue parameter is not specified.";
    }
    else
    {
        if(controller->getQueue().contains(network))
        {
            auto& netQueue = controller->getQueue()[network];
            if(netQueue.contains(wallet))
            {
                auto& walletQueue = netQueue[wallet];
                int size = walletQueue.size();
                for(int i = 0; i < size; i++)
                {
                    if(walletQueue[i].queueHash != idQueue)
                    {
                        if(i == size - 1)
                        {
                            error = "The transaction with id " + idQueue + " was not found.";
                        }
                        continue;
                    }
                    resultObject.insert("idQueue", idQueue);
                    resultObject.insert("wallet", wallet);
                    resultObject.insert("network", network);
                    QString hash = walletQueue[i].hash;
                    if(!hash.isEmpty())
                    {
                        resultObject.insert("txHash", hash);
                    }
                    resultObject.insert("command", walletQueue[i].nameCommand);
                    resultObject.insert("placeQueue", i);
                    break;
                }
            }
            else
            {
                error = "No queue found for wallet " + wallet;
            }
        }
        else
        {
            error = "No queue found for network " + network;
        }
    }

    QJsonObject mainObj;
    if(!error.isEmpty())
    {
        mainObj.insert("error", error);
    }
    mainObj.insert("result", resultObject);

    addWeb3Result(mainObj, args);

    QJsonDocument resultDoc;
    resultDoc.setObject(mainObj);
    return resultDoc.toJson();
}

