#include "DapUpdateLogsCommand.h"

/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
DapUpdateLogsCommand::DapUpdateLogsCommand(const QString &asServiceName, QObject *parent, const QString &asLogFile)
    : DapAbstractCommand(asServiceName, parent), m_csLogFile(asLogFile)
{
    qInfo() << "Initialization of DapUpdateLogsCommand...";
}

/// Process the notification from the client on the service side.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
QVariant DapUpdateLogsCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList params = args.toStringList();
    m_bufferSize = params[0].toInt();
    readLogFile();

    QJsonArray resultArr;
    for(const auto &item: qAsConst(m_bufLog))
    {
        resultArr.append(item);
    }
    QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultArr}});
    return resultDoc.toJson();
}

void DapUpdateLogsCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonArray resultArr = replyObj[RESULT_KEY].toArray();

    QStringList resultList;
    for(int i=0; i<resultArr.size(); ++i)
    {
        resultList.append(resultArr[i].toString());
    }

    emit serviceResponded(resultList);
}

///The slot reads logs to the buffer.
void DapUpdateLogsCommand::readLogFile()
{
    QFile dapLogFile(m_csLogFile);
    if (!dapLogFile.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        qWarning() << "The node log file does not open";
        return;
    }
    QTextStream readFile(&dapLogFile);
    m_bufLog.clear();
    while(!readFile.atEnd())
    {
        m_bufLog.append(readFile.readLine());

        if(m_bufLog.size() > m_bufferSize)
        {
             m_bufLog.removeFirst();
        }
    }

    if(readFile.status()!= QTextStream::Ok)
    {
        qWarning() << "Error reading log file";
    }
    dapLogFile.close();
}
