#ifndef DAPUPDATELOGSCOMMAND_H
#define DAPUPDATELOGSCOMMAND_H

#include <QFile>
#include <QFileInfo>
#include <QFileSystemWatcher>
#include <QDateTime>

#include "DapAbstractCommand.h"

#define DEFAULT_BUFFER_SIZE 20

class DapUpdateLogsCommand : public DapAbstractCommand
{
    ///The cursor position from which to start reading the file.
    qint64 m_seekFile {0};
    ///The number of rows that are stored in memory.
    int m_bufferSize {DEFAULT_BUFFER_SIZE};
    ///The container with the lines from the log.
    QStringList m_bufLog;

    const QString m_csLogFile;
    ///Monitors changes in the log file.

    uint64_t m_saveTime;

public:
    /// Overloaded constructor.
    /// @param asServiceName Service name.
    /// @param parent Parent.
    explicit DapUpdateLogsCommand(const QString &asServicename, QObject *parent, const QString &asLogFile = QString());

protected slots:
    ///The slot reads logs to the buffer.
    void readLogFile();

public slots:
    /// Process the notification from the client on the service side.
    /// @details Performed on the service side.
    /// @param arg1...arg10 Parameters.
    QVariant respondToClient(const QVariant &args = QVariant()) override;
    /// Reply from service.
    /// @details Performed on the service side.
    /// @return Service reply.
    void replyFromService() override;
};

#endif // DAPUPDATELOGSCOMMAND_H
