#ifndef DAPVERSIONCONTROLLER_H
#define DAPVERSIONCONTROLLER_H

#include <QObject>
#include <QProcess>
#include <QString>
#include <QTextCodec>
#include <QCoreApplication>
#include <QFile>
#include <QProcess>
#include <QTimer>
#include <QNetworkReply>

#include "DapAbstractCommand.h"

class DapVersionController : public DapAbstractCommand
{
public:

    static const QString LAST_VERSION;
    static const QString HAS_UPDATE;
    static const QString MESSAGE;
    static const QString URL;

    explicit DapVersionController(const QString &asServicename, QObject *parent = nullptr, const QString &cliPath = QString());

public slots:
    /// Send a response to the client.
    /// @details Performed on the service side.
    /// @param arg1...arg10 Parameters.
    /// @return Reply to client.
    QVariant respondToClient(const QVariant &args = QVariant()) override;
    /// Reply from service.
    /// @details Performed on the service side.
    /// @return Service reply.
    void replyFromService() override;


private:
    bool s_flagWait;
    bool s_hasUpdate;
    QString s_error;
    QString s_urlPath;
    QString s_suffix;
    QString s_downloadFilePath;
    QVariant s_lastVersion;
    QString s_numberVersion;
    QNetworkAccessManager* s_netManager;
    QFile* s_installScript;
    QFile* s_downloadfile;

private:
    void checkUpdate();
    bool compareVersion();
    void createInstallScript();
    bool installPack();

private slots:
    void downloadPack();

};

#endif // DAPVERSIONCONTROLLER_H
