#include "DapVoitingDumpCommand.h"

DapVoitingDumpCommand::DapVoitingDumpCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{}

QVariant DapVoitingDumpCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    // auto result = cmdList->votingDump(args);
    QJsonObject resultObject;
    auto requestDoc = cmdList->votingDump(args);
    auto request = requestDoc.toJsonObject();
    QString errorMsg;
    if(!request["errors"].isNull())
    {
        QJsonArray errArr = request["errors"].toArray();
        QJsonObject errObj = errArr[0].toObject()["error"].toObject();
    }
    else
    {
        QJsonArray result;
        QString hash;
        if(request.contains("result"))
        {
            result = request["result"].toArray();

        }
        if(result.isEmpty())
        {
            errorMsg = "It is possible to change the output of the node. there is no result.";
        }
        else
        {
            QJsonObject firstValue = result[0].toObject();

            QJsonObject voteObject;
            QJsonArray voteArray, additionalInfo;
            if(firstValue.contains(HASH_NODE_KEY))
            {
                voteObject.insert(HASH_KEY, firstValue.value(HASH_NODE_KEY).toString());
            }
            else
            {
                errorMsg = "Problems getting the hash of the vote.";
            }
            if(firstValue.contains(QUESTION_NODE_KEY))
            {
                voteObject.insert(QUESTION_KEY, firstValue.value(QUESTION_NODE_KEY).toString());
            }
            else
            {
                errorMsg = "Problems getting the question of the vote.";
            }
            if(firstValue.contains(RESULTS_NODE_KEY))
            {
                QJsonArray answerOptions = firstValue.value(RESULTS_NODE_KEY).toArray();
                for(const auto optionValue: answerOptions)
                {
                    QJsonObject resultOption;
                    auto optionObject = optionValue.toObject();

                    {
                        if(!optionObject.contains(OPTION_ID_NODE_KEY) || !optionObject.contains(VOTE_TX_NODE_KEY))
                        {
                            qWarning() << "[DapVoitingDumpCommand] Error getting a option or index, the output of the node may have changed. item: " << optionObject;
                            QJsonObject errorOption;
                            errorOption.insert("Error", "Error getting a option or index");
                            voteArray.append(errorOption);
                            continue;
                        }
                        else
                        {
                            resultOption.insert(MUMBER_KEY, optionObject.value(OPTION_ID_NODE_KEY).toInt());
                            resultOption.insert(OPTION_KEY, optionObject.value(VOTE_TX_NODE_KEY).toString());
                        }
                    }
                    {
                        if(!optionObject.contains(VOTING_POWER_NODE_KEY) || !optionObject.contains(VOTE_SHARE_NODE_KEY))
                        {
                            qWarning() << "[DapVoitingDumpCommand] Problems getting the number of votes, the output of the node may have changed. item: " << optionObject;
                            resultOption.insert(VOTES_KEY, "Incorrect data");
                        }
                        else
                        {
                            QJsonObject votes;
                            int count = optionObject.value(VOTING_POWER_NODE_KEY).toInt();
                            int percent = optionObject.value(VOTE_SHARE_NODE_KEY).toInt();
                            votes.insert(COUNT_KEY, count);
                            votes.insert(PERCENT_KEY, percent);
                            resultOption.insert(VOTES_KEY, std::move(votes));
                        }

                    }
                    {
                        if(!optionObject.contains(VOTE_SUM_NODE_KEY) || !optionObject.contains(VOTE_SUM_DATOSHI_NODE_KEY) ||
                            !optionObject.contains(VOTE_SUM_WEIGHT_NODE_KEY))
                        {
                            qWarning() << "[DapVoitingDumpCommand] Problems getting the number of weight, the output of the node may have changed. item: " << optionObject;
                            resultOption.insert(WEIGHT_KEY, "Incorrect data");
                        }
                        else
                        {
                            QJsonObject weight;
                            weight.insert(COIN_KEY, optionObject.value(VOTE_SUM_NODE_KEY).toString());
                            weight.insert(DATOSHI_KEY, optionObject.value(VOTE_SUM_DATOSHI_NODE_KEY).toString());
                            weight.insert(PERCENT_KEY, optionObject.value(VOTE_SUM_WEIGHT_NODE_KEY).toString());
                            resultOption.insert(WEIGHT_KEY, std::move(weight));
                        }

                    }
                    voteArray.append(resultOption);
                }
                voteObject.insert(OPTIONS_KEY, voteArray);
                if(!firstValue.contains(HASH_NODE_KEY) || !firstValue.contains(QUESTION_NODE_KEY)
                    || !firstValue.contains(TOKEN_NODE_KEY) || !firstValue.contains(TOTAL_SUM_DATOSHI_NODE_KEY)
                    || !firstValue.contains(TOTAL_SUM_NODE_KEY) || !firstValue.contains(VOTES_COUNT_NODE_KEY))
                {
                    qWarning() << "[DapVoitingDumpCommand] Data acquisition error. item: " << firstValue;
                    errorMsg = "Data acquisition error.";
                }
                else
                {
                    voteObject.insert(HASH_KEY, firstValue.value(HASH_NODE_KEY).toString());
                    voteObject.insert(QUESTION_KEY, firstValue.value(QUESTION_NODE_KEY).toString());
                    voteObject.insert(TOKEN_KEY, firstValue.value(TOKEN_NODE_KEY).toString());
                    voteObject.insert(TOTAL_DATOSHI_KEY, firstValue.value(TOTAL_SUM_DATOSHI_NODE_KEY).toString());
                    voteObject.insert(TOTAL_COINS_KEY, firstValue.value(TOTAL_SUM_NODE_KEY).toString());
                    voteObject.insert(TOTAL_NAMBER_KEY, firstValue.value(VOTES_COUNT_NODE_KEY).toInt());
                }
                resultObject.insert(RESULT_KEY, std::move(voteObject));
            }
            else
            {
                errorMsg = "Problems getting voting response options.";
            }

        }
    }

    if(!errorMsg.isEmpty())
    {
        resultObject.insert(ERROR_KEY, errorMsg);
    }

    addWeb3Result(resultObject, args);

    QJsonDocument resultDoc;
    resultDoc.setObject(resultObject);
    return resultDoc.toJson();
}
