#include "DapVoitingListCommand.h"

DapVoitingListCommand::DapVoitingListCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapVoitingListCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    auto result = cmdList->votingList(args);
    QJsonObject reply = result.toJsonObject();

    QJsonObject resultObject;
    if(reply.isEmpty())
    {
        resultObject.insert(ERROR_KEY, "Error. Node gave an empty answer.");
        addWeb3Result(resultObject, args);

        QJsonDocument resultDoc;
        resultDoc.setObject(resultObject);
        return resultDoc.toJson();
    }
    if(reply.contains("errors"))
    {
        resultObject.insert(ERROR_KEY, reply["errors"].toString());
        addWeb3Result(resultObject, args);

        QJsonDocument resultDoc;
        resultDoc.setObject(resultObject);
        return resultDoc.toJson();
    }
    if(!reply["result"].isArray() || !reply["result"].toArray().first().isArray())
    {
        resultObject.insert(ERROR_KEY, "Error. Answer is does't array.");
        addWeb3Result(resultObject, args);

        QJsonDocument resultDoc;
        resultDoc.setObject(resultObject);
        return resultDoc.toJson();
    }

    QJsonArray voteArray = reply["result"].toArray().first().toArray();

    QJsonArray resultArray;

    for(auto item: voteArray)
    {
        QJsonObject vote = item.toObject();

        QJsonObject object;
        object.insert("hash",     vote["poll_tx"].toString());
        object.insert("question", vote["question"].toString());
        resultArray.append(std::move(object));
    }

    resultObject.insert(RESULT_KEY, std::move(resultArray));
    addWeb3Result(resultObject, args);

    QJsonDocument resultDoc;
    resultDoc.setObject(resultObject);
    return resultDoc.toJson();
}
