#include "DapVoitingVoteCommand.h"

DapVoitingVoteCommand::DapVoitingVoteCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{
}

QVariant DapVoitingVoteCommand::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);
    QStringList params = args.toStringList();
    QString errorMsg;

    if(!params.contains("fee"))
    {
        QString fee = "50000000000000000";
        auto docFee = getFee()[getParamsByKey("net", params)];
        if(!docFee.validatorFeeList["median"].datoshi.isEmpty())
            fee = docFee.validatorFeeList["median"].datoshi;

        params.append("fee");
        params.append(fee);
    }

    QJsonObject transactionObj, resultObject;
    bool isWalletBlocked = false;
    QString tmpParam;
    cmdList->getParamForPairList(params, "walletName", tmpParam);
    if(!tmpParam.isEmpty())
    {
        auto wallets = getListWallets();
        if(wallets.contains(tmpParam) && wallets[tmpParam] == WALLET_NON_ACTIVE_KEY)
        {
            resultObject.insert(ERROR_KEY, "The wallet is blocked.");
            isWalletBlocked = true;
        }
    }

    if(!isWalletBlocked)
    {
        QJsonObject votingAnswer;

        auto requestDoc = cmdList->votingVote(params);
        auto request = requestDoc.toJsonObject();
        if(!request["errors"].isNull())
        {
            QJsonArray errArr = request["errors"].toArray();
            QJsonObject errObj = errArr[0].toObject()["error"].toObject();
            errorMsg = errObj["message"].toString();
        }
        else
        {
            QJsonArray result;
            bool isSuccess = false;
            QString hash;
            if(request.contains("result"))
            {
                result = request["result"].toArray();

            }
            if(result.isEmpty())
            {
                errorMsg = "It is possible to change the output of the node. there is no result.";
            }
            else
            {
                QJsonObject firstValue = result[0].toObject();
                for(const QString& itemKey: firstValue.keys())
                {
                    if(itemKey.contains("successfully"))
                    {
                        isSuccess = true;
                        hash = firstValue.value(itemKey).toString();
                    }
                }
            }

            transactionObj.insert(SUCCESS,QJsonValue(isSuccess));
            transactionObj.insert(MESSAGE,QJsonValue(result));
            if(isSuccess)
            {
                transactionObj.insert(HASH, QJsonValue(hash));
            }
        }
    }

    addWeb3Result(resultObject, args);
    resultObject.insert(RESULT_KEY, transactionObj);
    if(!errorMsg.isEmpty())
    {
        resultObject.insert(ERROR_KEY, errorMsg);
    }
    QJsonDocument resultDoc;
    resultDoc.setObject(resultObject);
    return resultDoc.toJson();
}
