#include "DapWalletActivateOrDeactivateCommand.h"

DapWalletActivateOrDeactivateCommand::DapWalletActivateOrDeactivateCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapWalletActivateOrDeactivateCommand::respondToClient(const QVariant &args)
{
    qDebug() << "[" << this->getName() << "] [respondToClient]";
    QStringList params = args.toStringList();

    QJsonObject resultObj;

    auto resultJson = cmdList->walletActivateOrDeactivate(args);
    auto resultStruct = cmdList->getJsonResult(resultJson, "DapWalletActivateOrDeactivateCommand");

    auto result = resultStruct.result;

    auto defaultError = [&resultObj, &result, &params, this]() -> QVariant
    {
        resultObj.insert("success",QJsonValue(false));
        resultObj.insert("message",QJsonValue(result));
        resultObj.insert("cmd",QJsonValue(params[1]));
        QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultObj}});
        return resultDoc.toJson();
    };


    if(result.isNull())
    {
        qWarning() << "[DapWalletActivateOrDeactivateCommand] The result block is empty, possibly an error.";
        return defaultError();
    }

    QJsonArray level1 = result.toArray();

    if(level1.isEmpty())
    {
        qWarning() << "[DapWalletActivateOrDeactivateCommand] The data attachment level has probably been changed.";
        return defaultError();
    }
    QJsonArray level2 = level1[0].toArray();

    if(level2.isEmpty())
    {
        qWarning() << "[DapWalletActivateOrDeactivateCommand] The data attachment level has probably been changed.";
        return defaultError();
    }
    QJsonObject objectData = level2[0].toObject();
    if(objectData.contains("errors"))
    {
        resultObj.insert("success",QJsonValue(false));
        resultObj.insert("message",QJsonValue("Invalid password"));
        resultObj.insert("cmd",QJsonValue(params[1]));
    }
    else if(!objectData.contains("protection"))
    {
        return defaultError();
    }
    else
    {
        QString protection = objectData.value("protection").toString();
        if(protection == "is deactivated")
        {
            resultObj.insert("success",QJsonValue(true));
            resultObj.insert("message",QJsonValue("Wallet deactivated"));
            resultObj.insert("cmd",QJsonValue(params[1]));
        }
        else
        {
            resultObj.insert("success",QJsonValue(true));
            resultObj.insert("message",QJsonValue("Wallet activated"));
            resultObj.insert("cmd",QJsonValue(params[1]));
        }
    }

    QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultObj}});
    return resultDoc.toJson();
}

void DapWalletActivateOrDeactivateCommand::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}
