#include "DapXchangeOrderPurchase.h"

const QString DapXchangeOrderPurchase::SUCCESS = "success";
const QString DapXchangeOrderPurchase::MESSAGE = "message";

DapXchangeOrderPurchase::DapXchangeOrderPurchase(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapXchangeOrderPurchase::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    auto resultJson = cmdList->xchangeOrderPurchase(args);
    auto resultStruct = cmdList->getJsonResult(resultJson, "DapXchangeOrderPurchase");

    auto result = resultStruct.result;
    auto failTransaction = [&args, &result, this]() -> QVariant
    {
        QJsonObject obj;
        obj.insert(SUCCESS,QJsonValue(false));
        obj.insert(MESSAGE,QJsonValue(result));

        QJsonObject resultObj;
        resultObj.insert(RESULT_KEY, obj);
        resultObj.insert(ERROR_KEY, result);
        addWeb3Result(resultObj, args);

        QJsonDocument resultDoc(resultObj);
        return resultDoc.toJson();
    };

    if(result.isNull())
    {
        qWarning() << "Result is null";
        return failTransaction();
    }
    QJsonArray level1 = result.toArray();

    if(level1.isEmpty())
    {
        qWarning() << "[DapXchangeOrderCreate] Jason's structure has changed.";
        return failTransaction();
    }

    QJsonObject infoTransactions = level1[0].toObject();

    if(!infoTransactions.contains("hash") || !infoTransactions.contains("status"))
    {
        qDebug() << "[DapXchangeOrderCreate] Jason's structure has changed. Or the transaction is rejected";
        return failTransaction();
    }

    QString hash = infoTransactions.value("hash").toString();
    QString status = infoTransactions.value("status").toString();

    if(hash.isEmpty())
    {
        qWarning() << "[DapXchangeOrderCreate] The hash string is empty.";
        return failTransaction();
    }

    QJsonObject obj;
    obj.insert(SUCCESS,QJsonValue(true));
    obj.insert(MESSAGE,QJsonValue("Exchange transaction has done"));
    obj.insert(HASH, QJsonValue(hash));

    QJsonObject resultObj;
    resultObj.insert(RESULT_KEY, obj);
    addWeb3Result(resultObj, args);

    QJsonDocument resultDoc(resultObj);

    return resultDoc.toJson();
}

void DapXchangeOrderPurchase::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);

}
