#include "DapXchangeOrderRemove.h"

const QString DapXchangeOrderRemove::SUCCESS = "success";
const QString DapXchangeOrderRemove::MESSAGE = "message";

DapXchangeOrderRemove::DapXchangeOrderRemove(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

QVariant DapXchangeOrderRemove::respondToClient(const QVariant &args)
{
    DapAbstractCommand::respondToClient(args);

    QJsonObject resultObj;

    auto resultJson = cmdList->xchangeOrderRemove(args);
    auto resultStruct = cmdList->getJsonResult(resultJson, "DapXchangeOrderRemove");

    auto result = resultStruct.result;
    auto failTransaction = [&result, this]() -> QVariant
    {
        QJsonObject resultObj;
        resultObj.insert(SUCCESS,QJsonValue(false));
        resultObj.insert(MESSAGE,QJsonValue(result));
        QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultObj}});
        return resultDoc.toJson();
    };

    if(result.isNull())
    {
        qWarning() << "Result is null";
        return failTransaction();
    }
    QJsonArray level1 = result.toArray();

    if(level1.isEmpty())
    {
        qWarning() << "[DapXchangeOrderCreate] Jason's structure has changed.";
        return failTransaction();
    }

    QJsonObject infoTransactions = level1[0].toObject();

    if(!infoTransactions.contains("Created inactivate tx with hash") || !infoTransactions.contains("status"))
    {
        qDebug() << "[DapXchangeOrderCreate] Jason's structure has changed. Or the transaction is rejected";
        return failTransaction();
    }

    QString hash = infoTransactions.value("Created inactivate tx with hash").toString();
    QString status = infoTransactions.value("status").toString();

    if(hash.isEmpty())
    {
        qWarning() << "[DapXchangeOrderCreate] The hash string is empty.";
        return failTransaction();
    }

    resultObj.insert(SUCCESS,QJsonValue(true));
    resultObj.insert(MESSAGE,QJsonValue("Order successfully removed"));
    resultObj.insert(HASH, QJsonValue(hash));

    QJsonDocument resultDoc(QJsonObject{{RESULT_KEY, resultObj}});
    return resultDoc.toJson();
}

void DapXchangeOrderRemove::replyFromService()
{
    DapRpcServiceReply *reply = static_cast<DapRpcServiceReply *>(sender());
    QVariant replyVariant = reply->response().toJsonValue().toVariant();

    QJsonDocument replyDoc = QJsonDocument::fromJson(replyVariant.toByteArray());
    QJsonObject replyObj = replyDoc.object();
    QJsonObject resultObj = replyObj[RESULT_KEY].toObject();

    emit serviceResponded(resultObj);
}
