#include "MempoolCheckCommand.h"


/// Overloaded constructor.
/// @param asServiceName Service name.
/// @param parent Parent.
/// @details The parent must be either DapRPCSocket or DapRPCLocalServer.
MempoolCheckCommand::MempoolCheckCommand(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapAbstractCommand(asServicename, parent, asCliPath)
{

}

/// Send a response to the client.
/// @details Performed on the service side.
/// @param arg1...arg10 Parameters.
/// @return Reply to client.
QVariant MempoolCheckCommand::respondToClient(const QVariant &args)
{
    QJsonObject resultObj;
    QJsonObject mempoolAnswer;
    QString errorMsg;

    auto requestDoc = cmdList->getMempoolHashInfo(args);
    auto request = requestDoc.toJsonObject();
    if(!request["errors"].isNull())
    {
        QJsonArray errArr = request["errors"].toArray();
        QJsonObject errObj = errArr[0].toObject()["error"].toObject();
        errorMsg = errObj["message"].toString();
    }
    else
    {
        auto resultValue = cmdList->getJsonResult(requestDoc, "mempool check").result.toArray();
        mempoolAnswer = resultValue[0].toObject();

        if(!mempoolAnswer["errors"].isNull())
        {
            QJsonArray errArr = mempoolAnswer["errors"].toArray();
            if (!errArr.isEmpty())
            {
                errorMsg = errArr[0].toObject()["message"].toString();
            }
        }
    }

    if(args.toStringList().contains(WEB3_KEY))
    {
        if(!errorMsg.isEmpty())
        {
            resultObj.insert(ERROR_KEY, errorMsg);
        }
        else
        {
            QJsonObject data;

            data.insert("chain", mempoolAnswer["chain"].toString());
            data.insert("find", mempoolAnswer["find"].toBool());
            data.insert("hash", mempoolAnswer["hash"].toString());
            data.insert("net", mempoolAnswer["net"].toString());
            data.insert("source", mempoolAnswer["source"].toString());

            resultObj.insert(RESULT_KEY, data);
        }
        addWeb3Result(resultObj, args);
    }
    else
    {
        if(!errorMsg.isEmpty())
            qWarning() << "Error." << errorMsg;

        resultObj.insert(RESULT_KEY, mempoolAnswer);
    }

    QJsonDocument resultDoc(resultObj);
    return resultDoc.toJson();
}
