#pragma once
#include <QObject>
#include <QDateTime>
#include "../DapAbstractCommand.h"

class DapCommandStackManager : public QObject
{
    Q_OBJECT
public:
    using RespondCallback = std::function<QVariant(const QVariant&)>;
    using InfoCallback = std::function<QString(const QStringList&)>;
    using Web3Update = std::function<void (QJsonObject&, const QVariant&)>;

    DapCommandStackManager(const DapAbstractCommand* command);

    void setParentCalback(RespondCallback handler) {m_parentCallback = handler;}
    void setCommandCalback(RespondCallback handler) {m_commandCallback = handler;}

    void setWalletNameCalback(InfoCallback handler) {m_walletNameCallback = handler;}
    void setNetworkNameCalback(InfoCallback handler) {m_networkNameCallback = handler;}

    void setWeb3UpdateCalback(Web3Update handler) {m_addWeb3InfoCallBack = handler;}
public slots:
    QVariant respondToClient(const QVariant &args = QVariant());
private:
    // hash generator
    QString getRandomString();
    QString getNewId();
    //==============
private:
    DapAbstractCommand* m_command = nullptr;
    QString m_nameCommand = "";

    RespondCallback m_parentCallback = nullptr;
    RespondCallback m_commandCallback = nullptr;

    InfoCallback m_walletNameCallback = nullptr;
    InfoCallback m_networkNameCallback = nullptr;

    Web3Update m_addWeb3InfoCallBack = nullptr;

    const QString HASH_QUEUE_KEY = "queueHash";
};

