#include "DapOrderCreateStakerCommandStack.h"
#include "TransactionQueue/DapTransactionQueueController.h"

DapOrderCreateStakerCommandStack::DapOrderCreateStakerCommandStack(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapOrderCreateStakerCommand(asServicename, parent, asCliPath)
    , m_commandManager(new DapCommandStackManager(this))
{
    m_commandManager->setCommandCalback([&](const QVariant &args) -> QVariant { return this->respondToClient(args);});
    m_commandManager->setParentCalback([&](const QVariant &args) -> QVariant { return DapOrderCreateStakerCommand::respondToClient(args); });
    m_commandManager->setWalletNameCalback([&](const QStringList &list) -> QString { return list[1]; });
    m_commandManager->setNetworkNameCalback([&](const QStringList &list) -> QString { return list[0];});
    m_commandManager->setWeb3UpdateCalback([&](QJsonObject& result, const QVariant& args){ addWeb3Result(result, args); });


    DapTransactionQueueController* controller = DapTransactionQueueController::getTransactionController();
    DapTransactionQueueController::ParamsFromCommand callbacks;
    callbacks.addressCallback       = [](const QStringList &list) -> QString { return "master node";};
    callbacks.feeCallback           = [](const QStringList &list) -> QString { return list[4];};
    callbacks.tokenCallback         = [this](const QStringList &list) -> QString { return m_tokens.value(list[0]).first;};
    callbacks.valueCallback         = [](const QStringList &list) -> QString { return list[2];};
    callbacks.walletNameCallback    = [](const QStringList &list) -> QString { return list[1];};
    callbacks.directionCallback     = [](const QStringList &list) -> QString { return "to";};
    callbacks.testExpenses          = []() -> bool { return true;};
    callbacks.feeInfo               = [&]() -> const QMap<QString, NetworkSpace::FeeInfoStruct> { return getFee();};
    controller->setParamsCommandCallback(this->getName(), callbacks);
}

QVariant DapOrderCreateStakerCommandStack::respondToClient(const QVariant &args)
{
    QStringList params = args.toStringList();
    if(!params.contains("self") && params.contains(WEB3_KEY) && !params.contains("feeAdded"))
    {
        QString fee = "50000000000000000";
        auto docFee = getFee()[params[0]];
        if(!docFee.validatorFeeList["median"].datoshi.isEmpty())
            fee = docFee.validatorFeeList["median"].datoshi;

        params.insert(4, fee);
        params.append("feeAdded");
        return m_commandManager->respondToClient(params);
    }
    return m_commandManager->respondToClient(args);
}

DapOrderCreateStakerCommandStack::~DapOrderCreateStakerCommandStack()
{
    delete m_commandManager;
}
