#include "DapSrvStakeDelegateCommandStack.h"
#include "TransactionQueue/DapTransactionQueueController.h"

DapSrvStakeDelegateCommandStack::DapSrvStakeDelegateCommandStack(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapSrvStakeDelegateCommand(asServicename, parent, asCliPath)
    , m_commandManager(new DapCommandStackManager(this))
{
    m_commandManager->setCommandCalback([&](const QVariant &args) -> QVariant { return this->respondToClient(args);});
    m_commandManager->setParentCalback([&](const QVariant &args) -> QVariant { return DapSrvStakeDelegateCommand::respondToClient(args); });
    m_commandManager->setWalletNameCalback([&](const QStringList &list) -> QString { return list[2]; });
    m_commandManager->setNetworkNameCalback([&](const QStringList &list) -> QString { return list[1];});
    m_commandManager->setWeb3UpdateCalback([&](QJsonObject& result, const QVariant& args){ addWeb3Result(result, args); });


    DapTransactionQueueController* controller = DapTransactionQueueController::getTransactionController();
    DapTransactionQueueController::ParamsFromCommand callbacks;
    callbacks.addressCallback       = [](const QStringList &list) -> QString { return "";};
    callbacks.feeCallback           = [](const QStringList &list) -> QString { return list[4];};
    callbacks.tokenCallback         = [](const QStringList &list) -> QString { return "";};
    callbacks.valueCallback         = [](const QStringList &list) -> QString { return list[3];};
    callbacks.walletNameCallback    = [](const QStringList &list) -> QString { return list[2];};
    callbacks.directionCallback     = [](const QStringList &list) -> QString { return "to";};
    callbacks.testExpenses          = []() -> bool { return true;};
    callbacks.feeInfo               = [&]() -> const QMap<QString, NetworkSpace::FeeInfoStruct> { return getFee();};
    controller->setParamsCommandCallback(this->getName(), callbacks);
}

QVariant DapSrvStakeDelegateCommandStack::respondToClient(const QVariant &args)
{
    QStringList argsTest = args.toStringList();
    if(argsTest.size() < 5)
    {
        QJsonDocument resultDoc(QJsonObject{{ERROR_KEY, "There are not enough parameters"}});
        return resultDoc.toJson();
    }
    return m_commandManager->respondToClient(args);
}

DapSrvStakeDelegateCommandStack::~DapSrvStakeDelegateCommandStack()
{
    delete m_commandManager;
}
