#include "DapStakeLockTakeCommandStack.h"
#include "TransactionQueue/DapTransactionQueueController.h"

DapStakeLockTakeCommandStack::DapStakeLockTakeCommandStack(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapStakeLockTakeCommand(asServicename, parent, asCliPath)
    , m_commandManager(new DapCommandStackManager(this))
{
    m_commandManager->setCommandCalback([&](const QVariant &args) -> QVariant { return this->respondToClient(args);});
    m_commandManager->setParentCalback([&](const QVariant &args) -> QVariant { return DapStakeLockTakeCommand::respondToClient(args); });
    m_commandManager->setWalletNameCalback([&](const QStringList &list) -> QString { return list[1]; });
    m_commandManager->setNetworkNameCalback([&](const QStringList &list) -> QString { return list[0];});
    m_commandManager->setWeb3UpdateCalback([&](QJsonObject& result, const QVariant& args){ addWeb3Result(result, args); });

    DapTransactionQueueController* controller = DapTransactionQueueController::getTransactionController();
    DapTransactionQueueController::ParamsFromCommand callbacks;
    callbacks.addressCallback       = [](const QStringList &list) -> QString { return QString();};
    callbacks.feeCallback           = [](const QStringList &list) -> QString { return list[3];};
    callbacks.tokenCallback         = [](const QStringList &list) -> QString { return list[4];};
    callbacks.valueCallback         = [](const QStringList &list) -> QString { return list[5];};
    callbacks.walletNameCallback    = [](const QStringList &list) -> QString { return list[1];};
    callbacks.directionCallback     = [](const QStringList &list) -> QString { return "from";};
    callbacks.testExpenses          = []() -> bool { return false;};
    callbacks.feeInfo               = [&]() -> const QMap<QString, NetworkSpace::FeeInfoStruct> { return getFee();};
    controller->setParamsCommandCallback(this->getName(), callbacks);
    controller->addCommandWithoutInfoWallet(this->getName());
}

QVariant DapStakeLockTakeCommandStack::respondToClient(const QVariant &args)
{
    QStringList params = args.toStringList();
    if(!params.contains("self") && params.contains(WEB3_KEY))
    {
        QString fee = "50000000000000000";
        auto docFee = getFee()[params[0]];
        if(!docFee.validatorFeeList["median"].datoshi.isEmpty())
            fee = docFee.validatorFeeList["median"].datoshi;

        params.insert(3, fee);

        auto infoTransaction = cmdList->getJsonResult(cmdList->historyInfoByHash(QStringList() << params[0] << params[2]), "DapStakeLockTakeCommandStack get data");
        QString tokenName, value, error;
        if(infoTransaction.result.isArray())
        {
            const auto& mainInfo = infoTransaction.result[0].toObject();
            if(mainInfo.contains("token_ticker"))
            {
                tokenName = mainInfo["token_ticker"].toString();
            }
            else if(mainInfo.contains("token ticker"))
            {
                tokenName = mainInfo["token ticker"].toString();
            }
            if(mainInfo.contains("ITEMS"))
            {
                for(const auto& item: mainInfo["ITEMS"].toArray())
                {
                    auto itemObject = item.toObject();
                    if(itemObject.contains("subtype") && itemObject["subtype"].toString() != "DAP_CHAIN_TX_OUT_COND_SUBTYPE_SRV_STAKE_LOCK")
                    {
                        continue;
                    }

                    if(itemObject.contains("value"))
                    {
                        value = itemObject["value"].toString();
                    }
                }
            }
            if(tokenName.isEmpty() || value.isEmpty())
            {
                qDebug() << "The output data structure has probably been changed.";
                error = "The output data structure has probably been changed.";
            }
            else
            {
                params.insert(4, tokenName);
                params.insert(5, value);
            }
        }
        else
        {
            qDebug() << "Hash transaction not found";
            error = "Hash transaction not found";
        }

        if(error.isEmpty())
        {
            return m_commandManager->respondToClient(params);
        }

        QJsonObject result;
        result.insert("success", false);

        QJsonDocument resultDoc;
        result.insert("error", error);
        result.insert("result", result);
        addWeb3Result(result,args);

        resultDoc.setObject(result);

        return resultDoc.toJson();
    }
    return m_commandManager->respondToClient(args);
}

DapStakeLockTakeCommandStack::~DapStakeLockTakeCommandStack()
{
    delete m_commandManager;
}
