#include "DapTXCondCreateCommandStack.h"
#include "TransactionQueue/DapTransactionQueueController.h"

DapTXCondCreateCommandStack::DapTXCondCreateCommandStack(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapTXCondCreateCommand(asServicename, parent, asCliPath)
    , m_commandManager(new DapCommandStackManager(this))
{
    m_commandManager->setCommandCalback([&](const QVariant &args) -> QVariant { return this->respondToClient(args);});
    m_commandManager->setParentCalback([&](const QVariant &args) -> QVariant { return DapTXCondCreateCommand::respondToClient(args); });
    m_commandManager->setWeb3UpdateCalback([&](QJsonObject& result, const QVariant& args){ addWeb3Result(result, args); });

    m_commandManager->setWalletNameCalback([&](const QStringList &list) -> QString { return list[2]; });
    m_commandManager->setNetworkNameCalback([&](const QStringList &list) -> QString { return list[0];});

    DapTransactionQueueController* controller = DapTransactionQueueController::getTransactionController();
    DapTransactionQueueController::ParamsFromCommand callbacks;
    callbacks.addressCallback       = [](const QStringList &list) -> QString { return QString();};
    callbacks.feeCallback           = [](const QStringList &list) -> QString { return list[5];};
    callbacks.tokenCallback         = [](const QStringList &list) -> QString { return list[1];};
    callbacks.valueCallback         = [](const QStringList &list) -> QString { return list[4];};
    callbacks.walletNameCallback    = [](const QStringList &list) -> QString { return list[2];};
    callbacks.directionCallback     = [](const QStringList &list) -> QString { return "to";};
    callbacks.testExpenses          = []() -> bool { return false;};
    callbacks.feeInfo               = [&]() -> const QMap<QString, NetworkSpace::FeeInfoStruct> { return getFee();};
    controller->setParamsCommandCallback(this->getName(), callbacks);
    controller->addCommandWithoutInfoWallet(this->getName());
}

QVariant DapTXCondCreateCommandStack::respondToClient(const QVariant &args)
{
    QStringList params = args.toStringList();
    if(!params.contains("self"))
    {
        QString fee = "50000000000000000";
        auto docFee = getFee()[params[0]];
        if(!docFee.validatorFeeList["median"].datoshi.isEmpty())
            fee = docFee.validatorFeeList["median"].datoshi;

        params.insert(5, fee);
        return m_commandManager->respondToClient(params);
    }
    return m_commandManager->respondToClient(args);
}

DapTXCondCreateCommandStack::~DapTXCondCreateCommandStack()
{
    delete m_commandManager;
}
