#include "DapVoitingCreateCommandStack.h"
#include "TransactionQueue/DapTransactionQueueController.h"

DapVoitingCreateCommandStack::DapVoitingCreateCommandStack(const QString &asServicename, QObject *parent, const QString &asCliPath)
    : DapVoitingCreateCommand(asServicename, parent, asCliPath)
    , m_commandManager(new DapCommandStackManager(this))
{
    m_commandManager->setCommandCalback([&](const QVariant &args) -> QVariant { return this->respondToClient(args);});
    m_commandManager->setParentCalback([&](const QVariant &args) -> QVariant { return DapVoitingCreateCommand::respondToClient(args); });
    m_commandManager->setWalletNameCalback([&](const QStringList &list) -> QString {
        QString result;
        cmdList->getParamForPairList(list, "walletName", result);
        return result;
    });
    m_commandManager->setNetworkNameCalback([&](const QStringList &list) -> QString {
        QString result;
        cmdList->getParamForPairList(list, "net", result);
        return result;
    });
    m_commandManager->setWeb3UpdateCalback([&](QJsonObject& result, const QVariant& args){ addWeb3Result(result, args); });


    DapTransactionQueueController* controller = DapTransactionQueueController::getTransactionController();
    DapTransactionQueueController::ParamsFromCommand callbacks;
    callbacks.addressCallback       = [](const QStringList &list) -> QString { return "voiting";};
    callbacks.feeCallback           = [this](const QStringList &list) -> QString {
        QString result;
        cmdList->getParamForPairList(list, "fee", result);
        return result;
    };
    callbacks.tokenCallback         = [](const QStringList &list) -> QString { return "";};
    callbacks.valueCallback         = [](const QStringList &list) -> QString { return "0.0";};
    callbacks.walletNameCallback    = [this](const QStringList &list) -> QString {
        QString result;
        cmdList->getParamForPairList(list, "walletName", result);
        return result;
    };
    callbacks.directionCallback     = [](const QStringList &list) -> QString { return "to";};
    callbacks.testExpenses          = []() -> bool { return false;};
    callbacks.feeInfo               = [&]() -> const QMap<QString, NetworkSpace::FeeInfoStruct> { return getFee();};
    controller->setParamsCommandCallback(this->getName(), callbacks);
}

QVariant DapVoitingCreateCommandStack::respondToClient(const QVariant &args)
{
    QStringList params = args.toStringList();
    if(!params.contains("self") && params.contains(WEB3_KEY))
    {
        if(!params.contains("fee"))
        {
            QString fee = "50000000000000000";
            auto docFee = getFee()[getParamsByKey("net", params)];
            if(!docFee.validatorFeeList["median"].datoshi.isEmpty())
                fee = docFee.validatorFeeList["median"].datoshi;

            params.append("fee");
            params.append(fee);
        }
        return m_commandManager->respondToClient(params);
    }
    return m_commandManager->respondToClient(args);
}

DapVoitingCreateCommandStack::~DapVoitingCreateCommandStack()
{
    delete m_commandManager;
}
