#include "DapServiceClientNativeWin.h"
#include <windows.h>
#include "dap_common.h"

DapServiceClientNativeWin::DapServiceClientNativeWin(const QString& asServiceName)
    : DapServiceClientNativeAbstract(asServiceName)
{
}

bool DapServiceClientNativeWin::isServiceRunning()
{
#ifdef QT_DEBUG
    return true;
#endif
    SC_HANDLE schSCManager;
    SC_HANDLE schService;
    DWORD size;
    SERVICE_STATUS_PROCESS scStatus;
    wchar_t binPath[MAX_PATH];
    GetModuleFileName(nullptr, binPath, MAX_PATH);
    schSCManager = OpenSCManager(nullptr, nullptr, SC_MANAGER_CONNECT);
    assert(schSCManager != nullptr);
    schService = OpenService(
            schSCManager,
            L"" DAP_BRAND "Service",
            SERVICE_QUERY_STATUS);
    if (!schService) {
        CloseServiceHandle(schSCManager);
        return false;
    }
    bool ret = false;
    QueryServiceStatusEx(
                schService,
                SC_STATUS_PROCESS_INFO,
                (LPBYTE)&scStatus,
                sizeof(SERVICE_STATUS_PROCESS),
                &size);
    switch (scStatus.dwCurrentState) {
    case SERVICE_RUNNING:
    case SERVICE_START_PENDING:
        ret = true;
        qDebug() << "Service running";
        break;
    default:
        ret = false;
        qDebug() << "Service not running";
        break;
    }
    CloseServiceHandle(schService);
    CloseServiceHandle(schSCManager);
    return ret;
}

DapServiceError DapServiceClientNativeWin::serviceInstallAndRun() {
#ifdef QT_DEBUG
    return DapServiceError::NO_ERRORS;
#else
    return serviceStart();
#endif
}

DapServiceError DapServiceClientNativeWin::serviceStart() {
    SC_HANDLE schSCManager;
    SC_HANDLE schService;
    wchar_t binPath[MAX_PATH];
    GetModuleFileName(nullptr, binPath, MAX_PATH);
    schSCManager = OpenSCManager(nullptr, nullptr, SC_MANAGER_CONNECT);
    assert(schSCManager != nullptr);
    schService = OpenService(
            schSCManager,
            L"" DAP_BRAND "Service",
            SERVICE_START);
    if (!schService) {
        CloseServiceHandle(schSCManager);
        return DapServiceError::UNABLE_START_SERVICE;
    }
    StartService(schService, 0, nullptr);
    CloseServiceHandle(schService);
    CloseServiceHandle(schSCManager);
    return isServiceRunning() ? DapServiceError::NO_ERRORS : DapServiceError::UNABLE_START_SERVICE;
}

DapServiceError DapServiceClientNativeWin::serviceRestart() {
    return DapServiceError::NO_ERRORS;
}

DapServiceError DapServiceClientNativeWin::serviceStop() {
    return DapServiceError::NO_ERRORS;
}

DapServiceClientNativeWin::~DapServiceClientNativeWin() {

}
